/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui;


import java.awt.Component;

import javax.swing.JOptionPane;

import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.string.StringUtils;


/**
 * Utility methods to show JOptionPane dialogs using translations provided by the LanguageSupport.
 * 
 * @author Olivier Berlanger
 */
public class SystemDialog {

    // tag of options for JOptionPane
    public static final String[] OPTIONS_OK = { "OK" };
    public static final String[] OPTIONS_YES_NO = { "YES", "NO" };
    public static final String[] OPTIONS_YES_NO_CANCEL = { "YES", "NO", "CANCEL" };
    public static final String[] OPTIONS_OK_CANCEL = { "OK", "CANCEL" };


    /**
     * Show a JOptionPane dialog using translations provided by the LanguageSupport.
     * 
     * @param parent
     *            the component initiating the dialog
     * @param titleTag
     *            bundle key for dialog title
     * @param messageTag
     *            bundle key for dialog message
     * @param messageParam
     *            parameters of the dialog message
     * @param optionsType
     *            type of options, one of JOptionPane.DEFAULT_OPTION, JOptionPane.YES_NO_OPTION, JOptionPane.YES_NO_CANCEL_OPTION or
     *            JOptionPane.OK_CANCEL_OPTION.
     * @param dialogType
     *            type of dialog, one of JOptionPane.ERROR_MESSAGE, JOptionPane.INFORMATION_MESSAGE, JOptionPane.WARNING_MESSAGE or
     *            JOptionPane.QUESTION_MESSAGE.
     * @return index of the selected option or -1 if dialog was closed with top-right cross.
     */
    public static final int showDialog(Component parent, String titleTag, String messageTag, Object[] messageParam,
            int optionsType, int dialogType) {
        String[] optionsTags;
        switch (optionsType) {
            case JOptionPane.DEFAULT_OPTION:
                optionsTags = OPTIONS_OK;
                break;
            case JOptionPane.YES_NO_OPTION:
                optionsTags = OPTIONS_YES_NO;
                break;
            case JOptionPane.YES_NO_CANCEL_OPTION:
                optionsTags = OPTIONS_YES_NO_CANCEL;
                break;
            case JOptionPane.OK_CANCEL_OPTION:
                optionsTags = OPTIONS_OK_CANCEL;
                break;
            default:
                throw new IllegalArgumentException("Unknown options type: " + optionsType);
        }
        return showDialog(parent, titleTag, messageTag, messageParam, optionsTags, optionsTags[0], dialogType);
    }


    /**
     * Show a JOptionPane dialog using translations provided by the LanguageSupport.
     * 
     * @param parent
     *            the component initiating the dialog
     * @param titleTag
     *            bundle key for dialog title
     * @param messageTag
     *            bundle key for dialog message
     * @param messageParam
     *            parameters of the dialog message
     * @param optionsTags
     *            bundle key for all possible options
     * @param defaultOptionTag
     *            bundle key of the default options
     * @param dialogType
     *            type of dialog, one of JOptionPane.ERROR_MESSAGE, JOptionPane.INFORMATION_MESSAGE, JOptionPane.WARNING_MESSAGE or
     *            JOptionPane.QUESTION_MESSAGE.
     * @return index of the selected option or -1 if dialog was closed with top-right cross.
     */
    public static final int showDialog(Component parent, String titleTag, String messageTag, Object[] messageParam,
            String[] optionsTags, String defaultOptionTag, int dialogType) {
        String titre = LanguageSupport.getLocalizedString(titleTag);
        String mess = LanguageSupport.getLocalizedString(messageTag, messageParam);
        int nbrOptions = optionsTags.length;
        String[] options = new String[nbrOptions];
        for (int i = 0; i < nbrOptions; i++)
            options[i] = LanguageSupport.getOptionalLocalizedString(optionsTags[i],
                    StringUtils.firstOfWordsUpperCase(optionsTags[i], true));
        String defaultOption = LanguageSupport.getLocalizedString(defaultOptionTag);
        return JOptionPane.showOptionDialog(parent, mess, titre, 0, dialogType, null, options, defaultOption);
    }

}
