/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.asynch;


import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.Timer;

import net.sf.sfac.gui.asynch.AsynchStatusProvider.ActionStatus;
import net.sf.sfac.gui.cmp.SizedDialog;
import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.setting.Settings;


/**
 * Modal dialog showing the status reported by an AsynchStatusProvider.
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class AsynchStatusDialog extends SizedDialog {


    private AsynchStatusProvider provider;

    private JLabel[] labels;

    private ActionStatus currentStatus;
    private JProgressBar progressBar;
    private JButton okButton;
    private JButton cancelButton;
    private Timer updateTimer;


    public AsynchStatusDialog(Component owner, AsynchStatusProvider prov, Settings sett) {
        super(owner, sett, prov.getProviderId() + ".dial");
        setModal(true);
        if (!prov.displayCancelButton()) setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        provider = prov;
        setTitle(LanguageSupport.getOptionalLocalizedString(prov.getProviderId() + "_title", prov.getProviderId()));
        buildGui();
        setDefaultBounds(LocationType.RELATIVE, new Rectangle(20, 50, 450, getPreferredHeight(provider)));
    }


    private static int getPreferredHeight(AsynchStatusProvider prov) {
        int height = 50;
        if (prov.displayCancelButton() || prov.displayOkButton()) height += 50;
        if (prov.displayProgressBar()) height += 30;
        height += 24 * prov.getStatusLineCount();
        return height;
    }


    public void startUpdates() {
        updateTimer = new Timer(500, new ActionListener() {


            public void actionPerformed(ActionEvent newE) {
                updateView();
            }

        });
        updateTimer.start();
    }


    private void buildGui() {
        JPanel contentPane = new JPanel(new GridBagLayout());
        contentPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        int len = provider.getStatusLineCount();
        labels = new JLabel[len];
        int row = 0;
        for (int i = 0; i < len; i++) {
            labels[i] = new JLabel();
            contentPane.add(labels[i], new GridBagConstraints(0, row, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER,
                    GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
            row++;
        }
        if (provider.displayProgressBar()) {
            progressBar = new JProgressBar(0, 100);
            progressBar.setStringPainted(true);
            contentPane.add(progressBar, new GridBagConstraints(0, row, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER,
                    GridBagConstraints.HORIZONTAL, new Insets(8, 0, 2, 0), 0, 0));
            row++;
        }
        if (provider.displayOkButton() || provider.displayCancelButton()) {
            JPanel buttonPane = new JPanel();
            if (provider.displayOkButton()) {
                String okLabel = LanguageSupport.getOptionalLocalizedString("OK", "Ok");
                okButton = new JButton(okLabel);
                okButton.addActionListener(new ActionListener() {


                    public void actionPerformed(ActionEvent newE) {
                        done();
                    }

                });
                buttonPane.add(okButton);
            }
            if (provider.displayCancelButton()) {
                String cancelLabel = LanguageSupport.getOptionalLocalizedString("CANCEL", "Cancel");
                cancelButton = new JButton(cancelLabel);
                cancelButton.addActionListener(new ActionListener() {


                    public void actionPerformed(ActionEvent newE) {
                        cancel();
                    }

                });
                buttonPane.add(cancelButton);
            }
            contentPane.add(buttonPane, new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
                    GridBagConstraints.NONE, new Insets(6, 0, 0, 2), 0, 0));
            row++;
        }
        setContentPane(contentPane);
        updateView();
    }


    public boolean isCancelAllowed() {
        return currentStatus == ActionStatus.RUNNING;
    }


    void done() {
        provider.okAction();
        updateView();
    }


    @Override
    public void dispose() {
        if (updateTimer != null) updateTimer.stop();
        super.dispose();
    }


    void cancel() {
        provider.cancelAction();
        updateView();
    }


    void updateView() {
        int len = provider.getStatusLineCount();
        for (int i = 0; i < len; i++) {
            if (provider.hasStatusLineChanged(i)) {
                labels[i].setText(provider.getStatusLine(i));
                labels[i].setMinimumSize(labels[i].getPreferredSize());
            }
        }
        if (currentStatus != provider.getStatus()) {
            currentStatus = provider.getStatus();
            if (currentStatus == ActionStatus.TO_DISPOSE) {
                dispose();
            } else {
                boolean running = (currentStatus == ActionStatus.RUNNING);
                if (okButton != null) okButton.setEnabled(!running);
                if (cancelButton != null) cancelButton.setEnabled(provider.isCancelAllowed());
            }
        }
        if (progressBar != null) {
            int percent = 100;
            if (currentStatus != ActionStatus.DONE) {
                percent = provider.getPercentComplete();
            }
            progressBar.setValue(percent);
        }
    }
}
