/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.cmp;


import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.border.AbstractBorder;


/**
 * Border like a lineBorder but where only a part of the border can be painted. The painted part can be defined using the TOP,
 * BOTTOM, LEFT and RIGHT constants (they can be combined bitwise like: (TOP | LEFT | BOTTOM)).
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class PartialLineBorder extends AbstractBorder {

    /** The constant used to require border display on the top. */
    public static final int TOP = 0x0001;
    /** The constant used to require border display on the bottom. */
    public static final int BOTTOM = 0x0002;
    /** The constant used to require border display on the left. */
    public static final int LEFT = 0x0004;
    /** The constant used to require border display on the right. */
    public static final int RIGHT = 0x0008;

    /** Color of the border. */
    private Color lineColor;
    /** Variable holding wich of the border will be painted (bitwise combination of TOP, BOTTOM, LEFT and RIGHT). */
    private int linePart;
    /** Thickness of the border line. */
    private int lineThickness;
    /** Additional (transparent) insets inside the border. */
    private int inset;


    /**
     * Create a new PartialLineBorder with a 1-pixel thick line and no additional insets.
     * 
     * @param color
     *            Color of the border
     * @param paintedPart
     *            painted part of the border (bitwise combination of TOP, BOTTOM, LEFT and RIGHT).
     */
    public PartialLineBorder(Color color, int paintedPart) {
        this(color, paintedPart, 0, 1);
    }


    /**
     * Create a new PartialLineBorder with a 1-pixel thick line.
     * 
     * @param color
     *            Color of the border
     * @param paintedPart
     *            painted part of the border (bitwise combination of TOP, BOTTOM, LEFT and RIGHT).
     * @param borderInset
     *            Additional (transparent) insets inside the border
     */
    public PartialLineBorder(Color color, int paintedPart, int borderInset) {
        this(color, paintedPart, borderInset, 1);
    }


    /**
     * Create a new PartialLineBorder.
     * 
     * @param color
     *            Color of the border
     * @param paintedPart
     *            painted part of the border (bitwise combination of TOP, BOTTOM, LEFT and RIGHT).
     * @param borderInset
     *            Additional (transparent) insets inside the border
     * @param thickness
     *            border line thickness.
     */
    public PartialLineBorder(Color color, int paintedPart, int borderInset, int thickness) {
        lineColor = color;
        linePart = paintedPart;
        inset = borderInset;
        lineThickness = thickness;
    }


    /**
     * Paints the border for the specified component with the specified position and size.
     * 
     * @param c
     *            the component for which this border is being painted
     * @param g
     *            the paint graphics
     * @param x
     *            the x position of the painted border
     * @param y
     *            the y position of the painted border
     * @param width
     *            the width of the painted border
     * @param height
     *            the height of the painted border
     */
    @Override
    public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
        int startX;
        int endX;
        int startY;
        int endY;
        Color oldColor = g.getColor();
        g.setColor(lineColor);
        for (int i = 0; i < lineThickness; i++) {
            startX = x + i;
            endX = x + width - (2 * i) - 1;
            startY = y + i;
            endY = y + height - (2 * i) - 1;
            if ((linePart & TOP) != 0) g.drawLine(startX, startY, endX, startY);
            if ((linePart & LEFT) != 0) g.drawLine(startX, startY, startX, endY);
            if ((linePart & BOTTOM) != 0) g.drawLine(startX, endY, endX, endY);
            if ((linePart & RIGHT) != 0) g.drawLine(endX, startY, endX, endY);
        }
        g.setColor(oldColor);
    }


    /**
     * Returns the total insets of the border.
     * 
     * @param c
     *            the component for which this border insets value applies
     * @return The total insets of the border.
     */
    @Override
    public Insets getBorderInsets(Component c) {
        return new Insets(inset + (((linePart & TOP) == 0) ? 0 : lineThickness), inset
                + (((linePart & LEFT) == 0) ? 0 : lineThickness), inset + (((linePart & BOTTOM) == 0) ? 0 : lineThickness), inset
                + (((linePart & RIGHT) == 0) ? 0 : lineThickness));
    }


    /**
     * Reinitialize the insets parameter with this Border's current Insets.
     * 
     * @param c
     *            the component for which this border insets value applies
     * @param insets
     *            the object to be reinitialized
     * @return the insets parameter reinitialized with this Border's current Insets.
     */
    @Override
    public Insets getBorderInsets(Component c, Insets insets) {
        insets.top = inset + (((linePart & TOP) == 0) ? 0 : lineThickness);
        insets.left = inset + (((linePart & LEFT) == 0) ? 0 : lineThickness);
        insets.bottom = inset + (((linePart & BOTTOM) == 0) ? 0 : lineThickness);
        insets.right = inset + (((linePart & RIGHT) == 0) ? 0 : lineThickness);
        return insets;
    }


    /**
     * Returns the color of the border.
     * 
     * @return The color of the border.
     */
    public Color getLineColor() {
        return lineColor;
    }


    /**
     * Returns the painted part of the border. (bitwise combination of TOP, BOTTOM, LEFT and RIGHT).
     * 
     * @return The painted part of the border.
     */
    public int getPaintedPart() {
        return linePart;
    }


    /**
     * Returns the lineThickness of the border.
     * 
     * @return The line thickness of the border.
     */
    public int getThickness() {
        return lineThickness;
    }


    /**
     * Returns whether or not the border is opaque.
     * 
     * @return true if the border is opaque, false otherwise.
     */
    @Override
    public boolean isBorderOpaque() {
        return inset == 0;
    }

}
