/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor;


import java.util.Collection;

import net.sf.sfac.setting.Settings;


/**
 * The context of an ObjectEditor.
 * 
 * @author Olivier Berlanger
 */
public interface EditorContext {


    /**
     * Check if the context can display/edit the objects of the given class. <br>
     * If this method returns false, the editInContext(..) method cannot be called with corresponding parameters.
     * 
     * @param create
     *            If true, check if the context can create new object and edit them, if false, check if the context can edit
     *            existing objects.
     * @param objectClass
     *            class of the object to edit.
     * @param source
     *            the ObjectEditor asking the edition. This editor edits objects of the same type as the object passed as first
     *            parameter.
     * @return true iff the context is able to display/edit the given object.
     */
    public boolean canEditInContext(boolean create, Class<?> objectClass, ObjectEditor source);


    /**
     * Ask the context to edit another object. <br>
     * The context will create an appropriate editor and display it to edit the object.
     * 
     * @param obj
     *            the object to edit. If null, a new object will be created.
     * @param objectClass
     *            class of the object to edit (useful when the object itself is null).
     * @param source
     *            the ObjectEditor asking the edition (can be null).
     * @return the selected/created object if the launched editor was closed on a selection, null otherwise.
     */
    public <T> T editInContext(T obj, Class<T> objectClass, ObjectEditor source);


    /**
     * Get the editor factory. <br>
     * If there is no custom factory specified, implementation can simply return the default factory with
     * <code>EditorFactory.getInstance()</code>.
     */
    public EditorFactory getEditorFactory();


    /**
     * Get a settings object that the ObjectEditors can use to store their preferences.
     */
    public Settings getSettings();


    /**
     * Get a custom object provided by the context.
     * 
     * @param objectKey
     *            the object key.
     * @return the object corresponding to the given key.
     */
    public Object getContextObject(String objectKey);


    /**
     * Get the editors of this context in a collection. <br>
     * The root context contains usually only one editor, but "container" editors implementing both ObjectEditor and
     * EditorContext aggregate a set of child editors. <br>
     * The returned collection should be treated as immutable (though is not necessary the case).
     */
    public Collection<ObjectEditor> getEditors();


}
