/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor;


import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.io.File;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.sf.sfac.editor.EditorConfig;
import net.sf.sfac.gui.editor.access.BasicModelAccess;
import net.sf.sfac.gui.editor.cmp.BooleanEditor;
import net.sf.sfac.gui.editor.cmp.ColorEditor;
import net.sf.sfac.gui.editor.cmp.DateEditor;
import net.sf.sfac.gui.editor.cmp.DimensionEditor;
import net.sf.sfac.gui.editor.cmp.EnumEditor;
import net.sf.sfac.gui.editor.cmp.FileEditor;
import net.sf.sfac.gui.editor.cmp.FontEditor;
import net.sf.sfac.gui.editor.cmp.GenericObjectEditor;
import net.sf.sfac.gui.editor.cmp.ListObjectEditor;
import net.sf.sfac.gui.editor.cmp.NumberEditor;
import net.sf.sfac.gui.editor.cmp.StringEditor;
import net.sf.sfac.utils.Comparison;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import sun.misc.Service;


/**
 * Factory for object editors. <br>
 * This factory defines default editors for known types. You can defines your default editors (or override those defined by this
 * class) by calling:
 * 
 * <pre>
 * EditorFactory.getInstance().addEditorMapping(myClass.class, myEditorClass.class);
 * </pre>
 * 
 * @author Olivier Berlanger
 */
public class EditorFactory {


    private static Log log = LogFactory.getLog(EditorFactory.class);
    public static final String FACTORY_CLASS_NAME = "editor.factory.class";
    private static EditorFactory instance;

    private Map<String, Class<?>> editorClasses = new HashMap<String, Class<?>>();


    public static Class<?> getEditorClass(EditorConfig annotation) {
        Class<?> clazz = null;
        String className = annotation.editor();
        if (Comparison.isDefined(className) && !EditorConfig.UNDEFINED.equals(className)) {
            try {
                return Class.forName(className);
            } catch (Exception e) {
                throw new IllegalArgumentException("Unable to instantiate class <" + className + ">", e);
            }
        }
        return clazz;
    }


    public static boolean containsEditorAnnotations(Class<?> editedClass) {
        if (getClassAnnotation(editedClass) != null) return true;
        Method[] methods = editedClass.getDeclaredMethods();
        int len = methods.length;
        for (int i = 0; i < len; i++) {
            if (methods[i].isAnnotationPresent(EditorConfig.class)) return true;
        }
        return false;
    }


    /**
     * Get the first annotation found in the class hierarchy.
     */
    public static EditorConfig getClassAnnotation(Class<?> theEditedClass) {
        Class<?> clazz = theEditedClass;
        EditorConfig annotation = null;
        while ((annotation == null) && (clazz != null)) {
            annotation = clazz.getAnnotation(EditorConfig.class);
            clazz = clazz.getSuperclass();
        }
        return annotation;
    }


    public static synchronized EditorFactory getInstance() {
        if (instance == null) {
            // Try to find a specific implementation with SPI
            instance = createInstanceWithSPI();
            if (instance == null) {
                // Try with System property
                instance = createInstanceWithSystemProperty();
                if (instance == null) {
                    // default case: use default instance
                    instance = new EditorFactory();
                    log.info("Use default EditorFactory: " + instance.getClass().getName());
                }
            }
        }
        return instance;
    }


    private static EditorFactory createInstanceWithSPI() {
        EditorFactory fact = null;
        Class<?> thisClass = EditorFactory.class;
        Iterator<?> it = Service.providers(thisClass, thisClass.getClassLoader());
        if (it.hasNext()) {
            fact = (EditorFactory) it.next();
            log.info("Use EditorFactory provided with SPI: " + fact.getClass().getName());
            if (it.hasNext()) {
                log.warn("Several implementation of EditorFactory are provided with SPI, ignored: "
                        + it.next().getClass().getName() + " ...");
            }
        }
        return fact;
    }


    private static EditorFactory createInstanceWithSystemProperty() {
        EditorFactory fact = null;
        String instanceClass = System.getProperty(FACTORY_CLASS_NAME);
        if (instanceClass != null) {
            try {
                fact = (EditorFactory) Class.forName(instanceClass).newInstance();
                log.info("Use EditorFactory provided with System property: " + fact.getClass().getName());
            } catch (Exception e) {
                log.error("Unable to instantiate EditorFactory of class " + instanceClass, e);
            }
        }
        return fact;
    }


    protected EditorFactory() {
        // add default editors
        addEditorMapping(String.class, StringEditor.class);
        addEditorMapping(Boolean.class, BooleanEditor.class);
        addEditorMapping(Boolean.TYPE, BooleanEditor.class);
        addEditorMapping(Integer.TYPE, NumberEditor.class);
        addEditorMapping(Long.TYPE, NumberEditor.class);
        addEditorMapping(Double.TYPE, NumberEditor.class);
        addEditorMapping(Float.TYPE, NumberEditor.class);
        addEditorMapping(Short.TYPE, NumberEditor.class);
        addEditorMapping(Character.TYPE, NumberEditor.class);
        addEditorMapping(Byte.TYPE, NumberEditor.class);
        addEditorMapping(File.class, FileEditor.class);
        addEditorMapping(Date.class, DateEditor.class);
        addEditorMapping(Dimension.class, DimensionEditor.class);
        addEditorMapping(Color.class, ColorEditor.class);
        addEditorMapping(Font.class, FontEditor.class);
    }


    public void addEditorMapping(Class<?> objectClass, Class<?> editorClass) {
        editorClasses.put(objectClass.getName(), editorClass);
    }


    public void addEditorMapping(String objectClassName, Class<?> editorClass) {
        editorClasses.put(objectClassName, editorClass);
    }


    public ObjectEditor createEditor(Class<?> editedObjectClass) {
        return createEditor(editedObjectClass, GenericObjectEditor.class);
    }


    public ObjectEditor createEditor(Class<?> editedObjectClass, Class<?> defaultEditorClass) {
        if (editedObjectClass == null) throw new NullPointerException("Cannot create an editor for a null class");
        ObjectEditor ed = createEditor(new BasicModelAccess(editedObjectClass), GenericObjectEditor.class);
        EditorConfig editorClassAnnotation = editedObjectClass.getAnnotation(EditorConfig.class);
        EditorConfigHelper.setProperties(ed, editorClassAnnotation, editedObjectClass.getSimpleName());
        return ed;
    }


    public ObjectEditor createEditor(ModelAccess access, Class<?> defaultEditorClass) {
        if (access == null) throw new NullPointerException("Cannot create an editor for a null ModelAccess");
        if (defaultEditorClass == null) throw new NullPointerException("The provided default editor class cannot be null");
        ObjectEditor editor;
        Class<?> editorClass = getEditorClass(access.getModelClass());
        if (editorClass == null) editorClass = defaultEditorClass;
        try {
            log.info("Creating editor <" + editorClass.getName() + "> with model access " + access);
            editor = newEditorInstance(access, editorClass);
        } catch (Exception e) {
            throw new IllegalStateException("Unable to instantiate editor " + editorClass, e);
        }
        return editor;
    }


    public static ObjectEditor newEditorInstance(ModelAccess access, Class<?> editorClass) {
        try {
            ObjectEditor ed = (ObjectEditor) editorClass.newInstance();
            ed.setModelAccess(access);
            return ed;
        } catch (Exception e) {
            throw new IllegalStateException("Cannot instanciate editor " + editorClass + " with ModelAccess " + access, e);
        }
    }


    protected Class<?> getEditorClass(Class<?> modelClass) {
        if (modelClass == null) throw new NullPointerException("Cannot create an editor for a null class");

        Class<?> editorClass = getEditorClassFromAnnotation(modelClass);
        if (editorClass == null) editorClass = getEditorClassForClassName(modelClass.getName());
        if (editorClass == null) editorClass = getEditorClassByClassType(modelClass);
        return editorClass;
    }


    private Class<?> getEditorClassFromAnnotation(Class<?> modelClass) {
        Class<?> editorClass = null;
        // get editor class name from annotation
        EditorConfig editorClassAnnotation = getClassAnnotation(modelClass);
        if (editorClassAnnotation != null) {
            editorClass = getEditorClass(editorClassAnnotation);
        }
        return editorClass;
    }


    protected Class<?> getEditorClassForClassName(String modelClassName) {
        return editorClasses.get(modelClassName);
    }


    protected Class<?> getEditorClassByClassType(Class<?> modelClass) {
        Class<?> editorClass = null;
        if (modelClass.isEnum()) editorClass = EnumEditor.class;
        if ((editorClass == null) && Number.class.isAssignableFrom(modelClass)) editorClass = NumberEditor.class;
        if ((editorClass == null) && List.class.isAssignableFrom(modelClass)) editorClass = ListObjectEditor.class;
        return editorClass;
    }


}
