/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.access;


import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.List;

import net.sf.sfac.gui.editor.ModelAccess;
import net.sf.sfac.utils.ReflectionUtils;


/**
 * Implementation of the ModelAccess interface used by Table/List editors to access indexed data. <br>
 * To be usable by this accessors the 'xxx' indexed property must be exposed as a List like:
 * <ul>
 * <li>public List<ItemType> getXxx();
 * </ul>
 * <p>
 * With this kind of access the list is by default non-editable (you can just reorder the items) and you have to write a custom
 * Editor to enable edition. <br>
 * Alternatively you can expose the indexed property with indexed methods like:
 * </p>
 * <ul>
 * <li>public int getXxxCount();
 * <li>public ItemType getXxx(int index);
 * <li>public void setXxx(int index, ItemType item);
 * <li>public void addXxx(int index, ItemType item);
 * <li>public void removeXxx(int index);
 * <li>public void switchXxx(int index1, int index2);
 * </ul>
 * Where only the two first methods are mandatory for read-only access and the other are required to enable edition and
 * re-ordering of the list. Those methods enable more flexible and fine-grained handling of the indexed property.
 * 
 * @author Olivier Berlanger
 */
public class IndexedModelAccess extends ChainableModelAccess {


    private String path;
    private Class<?> itemClass;
    private boolean listAcccess;

    private Method sizer;
    private Method getter;
    private Method setter;
    private Method adder;
    private Method remover;
    private Method switcher;


    public static boolean isIndexedGetter(Method getter) {
        Class<?> returnType = getter.getReturnType();
        if (returnType.equals(void.class)) return false;
        if (List.class.isAssignableFrom(returnType)) return true;
        Class<?>[] params = getter.getParameterTypes();
        if ((params == null) || (params.length != 1)) return false;
        return params[0].equals(Integer.TYPE);
    }


    public static boolean isListGetter(Method getter) {
        if (getter.getReturnType().equals(void.class)) return false;
        Class<?>[] params = getter.getParameterTypes();
        if ((params != null) && (params.length > 0)) return false;
        return List.class.isAssignableFrom(getter.getReturnType());
    }


    public IndexedModelAccess(String fieldName, Class<?> enclosingClass) {
        this(fieldName, enclosingClass, null);
    }


    public IndexedModelAccess(String fieldName, Class<?> enclosingClass, Class<?> fieldClass) {
        Method fieldGetter = ReflectionUtils.getMethod(enclosingClass, "get", fieldName, null, false, Integer.TYPE);
        if (fieldGetter == null) {
            fieldGetter = ReflectionUtils.getMethod(enclosingClass, "get", fieldName, null);
        }
        setup(enclosingClass, fieldGetter, fieldName, fieldClass);
    }


    public IndexedModelAccess(Class<?> enclosingClass, Method fieldGetter) {
        this(enclosingClass, fieldGetter, null);
    }


    public IndexedModelAccess(Class<?> enclosingClass, Method fieldGetter, Class<?> fieldClass) {
        setup(enclosingClass, fieldGetter, ReflectionUtils.getFieldName(fieldGetter), fieldClass);
    }


    public IndexedModelAccess(ModelAccess access) {
        if (!List.class.isAssignableFrom(access.getModelClass())) {
            throw new IllegalArgumentException("IndexedModelAccess needs a inner Model access returning List instances, not: "
                    + access.getModelClass());
        }
        setParent(access);
        listAcccess = true;
    }


    private void setup(Class<?> enclosingClass, Method fieldGetter, String fieldName, Class<?> simpleItemClass) {
        if (fieldGetter == null) throw new IllegalArgumentException("Getter cannot be null");
        getter = fieldGetter;
        itemClass = simpleItemClass;
        path = fieldName;
        if (isListGetter(fieldGetter)) {
            listAcccess = true;
        } else {
            if (!isIndexedGetter(fieldGetter)) {
                throw new IllegalStateException("The the method " + fieldGetter
                        + " must be indexed getter like: public Object getSomething(int index)");
            }
            if (itemClass == null) itemClass = fieldGetter.getReturnType();
            setupAccessMethods(fieldName, enclosingClass, itemClass);
        }
    }


    @SuppressWarnings("unchecked")
    private List<Object> getList(Object baseObject) {
        List<Object> theList = null;
        Object editedObject = super.getModelValue(baseObject);
        if (editedObject != null) {
            try {
                if (getter == null) {
                    theList = (List<Object>) editedObject;
                } else {
                    theList = (List<Object>) getter.invoke(editedObject, (Object[]) null);
                }
            } catch (ClassCastException cce) {
                throw new IllegalStateException("Object obtained by ModelAccess is not a List", cce);
            } catch (Exception e) {
                throw new IllegalStateException("Unable to invoke indexed getter " + getter + " on object " + editedObject, e);
            }
        }
        return theList;
    }


    public int getItemCount(Object baseObject) {
        if (listAcccess) {
            List<Object> items = getList(baseObject);
            return (items == null) ? 0 : items.size();
        } else {
            Object editedObject = super.getModelValue(baseObject);
            if (editedObject == null) return 0;
            try {
                Object value = sizer.invoke(editedObject, (Object[]) null);
                return ((Integer) value).intValue();
            } catch (Exception e) {
                throw new IllegalStateException("Unable to invoke method " + sizer + " on object " + editedObject, e);
            }
        }

    }


    public Object getItem(Object baseObject, int index) {
        if (listAcccess) {
            return getList(baseObject).get(index);
        } else {
            Object editedObject = super.getModelValue(baseObject);
            if (editedObject == null) return null;
            try {
                Object value = getter.invoke(editedObject, Integer.valueOf(index));
                return value;
            } catch (Exception e) {
                throw new IllegalStateException("Unable to invoke indexed getter " + getter + " on object " + editedObject, e);
            }
        }
    }


    public boolean canSetItem() {
        return listAcccess || (setter != null);
    }


    public void setItem(Object baseObject, int index, Object value) {
        if (listAcccess) {
            getList(baseObject).set(index, value);
        } else {
            Object editedObject = super.getModelValue(baseObject);
            if (editedObject != null) {
                try {
                    setter.invoke(editedObject, Integer.valueOf(index), value);
                } catch (Exception e) {
                    throw new IllegalStateException("Unable to invoke indexed setter " + setter + " on object " + editedObject,
                            e);
                }
            }
        }
    }


    public boolean canAddItem() {
        if (listAcccess)
            return (itemClass != null) && (!itemClass.isInterface()) && (!Modifier.isAbstract(itemClass.getModifiers()));
        return adder != null;
    }


    public void addItem(Object baseObject, int index, Object value) {
        try {
            if ((value == null) && (itemClass != null)) value = itemClass.newInstance();
            if (listAcccess) {
                getList(baseObject).add(index, value);
            } else {
                Object editedObject = super.getModelValue(baseObject);
                if (editedObject != null) {
                    adder.invoke(editedObject, Integer.valueOf(index), value);
                }
            }
        } catch (Exception e) {
            throw new IllegalStateException("Unable to invoke method " + adder + " on object " + baseObject, e);
        }
    }


    public boolean canRemoveItem() {
        if (listAcccess) return canAddItem();
        return remover != null;
    }


    public void removeItem(Object baseObject, int index) {
        if (listAcccess) {
            getList(baseObject).remove(index);
        } else {
            Object editedObject = super.getModelValue(baseObject);
            if (editedObject != null) {
                try {
                    remover.invoke(editedObject, Integer.valueOf(index));
                } catch (Exception e) {
                    throw new IllegalStateException("Unable to invoke method " + remover + " on object " + editedObject, e);
                }
            }
        }
    }


    public boolean canSwitchItem() {
        return (switcher != null) || listAcccess;
    }


    public void switchItems(Object baseObject, int index1, int index2) {
        if (listAcccess) {
            List<Object> items = getList(baseObject);
            Object value = items.get(index1);
            items.set(index1, items.get(index2));
            items.set(index2, value);
        } else {
            Object editedObject = super.getModelValue(baseObject);
            if (editedObject != null) {
                try {
                    switcher.invoke(editedObject, Integer.valueOf(index1), Integer.valueOf(index2));
                } catch (Exception e) {
                    throw new IllegalStateException("Unable to invoke method " + switcher + " on object " + editedObject, e);
                }
            }
        }
    }


    private void setupAccessMethods(String attrName, Class<?> enclosingClass, Class<?> simpleItemClass) {
        sizer = ReflectionUtils.getMethod(enclosingClass, "get", attrName, "Count");
        if (simpleItemClass != null) {
            setter = ReflectionUtils.getMethod(enclosingClass, "set", attrName, null, false, Integer.TYPE, simpleItemClass);
            adder = ReflectionUtils.getMethod(enclosingClass, "add", attrName, null, false, Integer.TYPE, simpleItemClass);
        }
        remover = ReflectionUtils.getMethod(enclosingClass, "remove", attrName, null, false, Integer.TYPE);
        switcher = ReflectionUtils.getMethod(enclosingClass, "switch", attrName, null, false, Integer.TYPE, Integer.TYPE);
    }


    @Override
    public Object getModelValue(Object baseObject) {
        if (listAcccess) return getList(baseObject);
        return null;
    }


    public void setModelValue(Object newModelValue, Object editedObject) {
        throw new UnsupportedOperationException("Access to model must be indexed");
    }


    public boolean canUpdateModel() {
        if (listAcccess) return true;
        return (setter != null) || (adder != null) || (remover != null) || (switcher != null);
    }


    @Override
    public Class<?> getThisEditedObjectClass() {
        return (getter == null) ? List.class : getter.getDeclaringClass();
    }


    public void setModelClass(Class<?> modelClass) {
        itemClass = modelClass;
    }


    public Class<?> getModelClass() {
        return itemClass;
    }


    @Override
    public String getThisFieldPath() {
        return path;
    }


}
