/*-------------------------------------------------------------------------
 Copyright 2012 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.access;


import java.lang.reflect.Method;

import net.sf.sfac.gui.editor.ModelAccess;
import net.sf.sfac.utils.Comparison;


/**
 * A factory to build ModelAccess from a String and a type. <br>
 * The given String can be of from:
 * <ul>
 * <li>{$someMethod} : Then a model access calling the given method (using exact provided name without parameters) on the editor
 * model is build.
 * <li>{@someAttribute} : Then a model access calling the getters/setter getSomeAttribute() (or isSomeAttribute() for booleans)
 * and setSomeAttribute(T value) is build. The presence of the getter in the model is mandatory, but the setter is optional.
 * <li>other : Then a model access always returning a fixed value of type T is build. In this case, if 'translate' is true, the
 * created model access is combined with a TranslationModelAccess to translate the value (using LanguageSupport).
 * </ul>
 * 
 * @author Olivier Berlanger
 */
public class ModelAccessFactory {


    public static ModelAccess createAccess(Object value, Class<?> returnedClass, Class<?> modelClass, boolean translate) {
        if (value == null) return null;
        if (value instanceof ModelAccess) return (ModelAccess) value;
        return buildModelAccess(value.toString(), returnedClass, modelClass, translate);
    }


    public static ModelAccess buildModelAccess(String pattern, Class<?> returnedClass, Class<?> modelClass, boolean translate) {
        ModelAccess access = null;
        if (Comparison.isDefined(pattern)) {
            pattern = pattern.trim();
            int len = pattern.length();
            boolean endsWithBrace = pattern.endsWith("}");
            if (pattern.startsWith("{$") && endsWithBrace) {
                access = buildMethodAccess(pattern.substring(2, len - 1).trim(), modelClass);
            } else if (pattern.startsWith("{@") && endsWithBrace) {
                access = buildAttributeAccess(pattern.substring(2, len - 1).trim(), returnedClass, modelClass);
            } else {
                access = buildFixedAccess(pattern, returnedClass);
                if (translate) access = new TranslationModelAccess(access);
            }
        }
        return access;
    }


    private static ModelAccess buildFixedAccess(String stringValue, Class<?> fieldClass) {
        Object fixedValue;
        if (fieldClass.equals(Boolean.class) || fieldClass.equals(Boolean.TYPE)) {
            fixedValue = Boolean.valueOf(stringValue);
        } else if (fieldClass.equals(String.class)) {
            fixedValue = stringValue;
        } else {
            throw new UnsupportedOperationException("Unsupported fixed value type: " + fieldClass.getName());
        }
        return new FixedModelAccess(fixedValue);
    }


    public static ModelAccess buildAttributeAccess(String attributeName, Class<?> returnedClass, Class<?> modelClass) {
        return new ReflexionModelAccess(attributeName, modelClass, returnedClass);
    }


    public static ModelAccess buildMethodAccess(String methodName, Class<?> modelClass) {
        try {
            Method getter = modelClass.getMethod(methodName, (Class[]) null);
            return new ReflexionModelAccess(modelClass, getter);
        } catch (Exception e) {
            throw new IllegalArgumentException("No method " + methodName + "() found on " + modelClass, e);
        }
    }
}
