/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.Component;
import java.awt.Container;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JComponent;
import javax.swing.JLabel;

import net.sf.sfac.gui.editor.EditorContext;
import net.sf.sfac.gui.editor.EditorModificationEvent;
import net.sf.sfac.gui.editor.EditorModificationListener;
import net.sf.sfac.gui.editor.EditorModificationSupport;
import net.sf.sfac.gui.editor.ModelAccess;
import net.sf.sfac.gui.editor.ObjectEditor;
import net.sf.sfac.gui.editor.access.BasicModelAccess;
import net.sf.sfac.gui.editor.access.ModelAccessFactory;
import net.sf.sfac.setting.Settings;


/**
 * Base class that can be derived to implement an object editor. <br>
 * This base class handle the generic behavior of <code>ObjectEditor</code>:
 * <ul>
 * <li>context
 * <li>label and labelComponent
 * <li>description
 * <li>ModelAccess
 * <li>editedObject
 * <li>editable flag
 * <li>enabled flag
 * <li>visibility flag
 * <li>Event support
 * </ul>
 * 
 * @author Olivier Berlanger
 */
public abstract class BaseObjectEditor implements ObjectEditor {

    private static final ModelAccess DEFAULT_ACCESS = new BasicModelAccess(Object.class);

    private EditorContext context;
    private ModelAccess access = DEFAULT_ACCESS;
    private Object editedObject;
    private Map<String, Object> properties;
    private EditorModificationSupport modificationSupport = new EditorModificationSupport(this);
    private EditorModificationListener parentListener;
    private FocusListener theFocusListener;

    private ModelAccess labelAccess;
    private ModelAccess descriptionAccess;
    private ModelAccess editableAccess;
    private ModelAccess enabledAccess;
    private ModelAccess visibilityAccess;

    private JComponent labelComponent;


    public BaseObjectEditor() {
    }


    public void edit(Object objToEdit) {
        boolean oldEnabled = isEnabled();
        editedObject = objToEdit;
        updateView();
        if (oldEnabled != isEnabled()) synchronizeEditableState();
    }


    public Object getEditedObject() {
        return editedObject;
    }


    public Object getModelValue() {
        return access.getModelValue(getEditedObject());
    }


    public void setModelValue(Object newModelValue) {
        access.setModelValue(newModelValue, getEditedObject());
    }


    public ObjectEditor getParentEditor() {
        EditorContext contxt = getEditorContext();
        if (contxt instanceof ObjectEditor) return (ObjectEditor) contxt;
        return null;
    }


    public ObjectEditor getRootEditor() {
        ObjectEditor root = this;
        ObjectEditor parent = getParentEditor();
        while (parent != null) {
            root = parent;
            EditorContext parentContext = root.getEditorContext();
            if (parentContext instanceof ObjectEditor) parent = (ObjectEditor) parentContext;
        }
        return root;
    }


    @Override
    public String toString() {
        return getClass().getSimpleName() + "[" + getModelAccess().getFieldPath() + "]";
    }


    public String getTranslatedLabel() {
        return (labelAccess == null) ? "" : (String) labelAccess.getModelValue(getEditedObject());
    }


    public String getTranslatedDescription() {
        return (descriptionAccess == null) ? "" : (String) descriptionAccess.getModelValue(getEditedObject());
    }


    public ModelAccess getModelAccess() {
        return access;
    }


    public void setModelAccess(ModelAccess newAccess) {
        if (newAccess == null) throw new IllegalArgumentException("Model access cannot be null");
        if (access != DEFAULT_ACCESS) throw new IllegalStateException("Model access cannot be changed");
        access = newAccess;
    }


    /**
     * Get the visibility of the currently displayed components.
     * 
     * @return the visibility of the currently displayed components.
     */
    protected boolean getComponentVisibility() {
        return getObjectComponent().isVisible();
    }


    /**
     * Get the visibility of the currently displayed components.
     * 
     * @return the visibility of the currently displayed components.
     */
    protected void setComponentVisibility(boolean visible) {
        if (labelComponent != null) labelComponent.setVisible(visible);
        setComponentVisible(getObjectComponent(), visible);
    }


    protected static void setComponentVisible(Component cmp, boolean visible) {
        if (cmp != null) {
            cmp.setVisible(visible);
            if (cmp instanceof Container) {
                Component[] subComponents = ((Container) cmp).getComponents();
                if (subComponents != null) {
                    for (Component sub : subComponents) {
                        setComponentVisible(sub, visible);
                    }
                }
            }
        }
    }


    public ModelAccess getLabelAccess() {
        return labelAccess;
    }


    public void setLabelAccess(ModelAccess newLabelAccess) {
        labelAccess = newLabelAccess;
        updateLabelText();
    }


    public ModelAccess getDescriptionAccess() {
        return descriptionAccess;
    }


    public void setDescriptionAccess(ModelAccess newDescriptionAccess) {
        descriptionAccess = newDescriptionAccess;
        updateDescriptionText();
    }


    protected ModelAccess getVisibilityAccess() {
        return visibilityAccess;
    }


    protected void setVisibilityAccess(ModelAccess newVisibilityAccess) {
        visibilityAccess = newVisibilityAccess;
        checkViewVisibility();
    }


    public ModelAccess getEditableAccess() {
        return editableAccess;
    }


    public void setEditableAccess(ModelAccess newEditableAccess) {
        editableAccess = newEditableAccess;
        synchronizeEditableState();
    }


    public ModelAccess getEnabledAccess() {
        return enabledAccess;
    }


    public void setEnabledAccess(ModelAccess newEnabledAccess) {
        enabledAccess = newEnabledAccess;
        synchronizeEditableState();
    }


    protected void checkViewVisibility() {
        if (visibilityAccess != null) {
            throw new UnsupportedOperationException("Visibility access set to <" + visibilityAccess
                    + "> while checkVisibility() not implemented on: " + this);
        }
    }


    public void setEditorContext(EditorContext newContext) {
        if ((context != null) && (newContext != null) && (context != newContext)) {
            throw new IllegalStateException("Context (or parent) cannot change from " + context + " to " + newContext);
        }
        if (parentListener == null) {
            parentListener = new EditorModificationListener() {

                public void editorModified(EditorModificationEvent eme) {
                    parentModified(eme);
                }
            };
        }
        ObjectEditor oldParent = getParentEditor();
        if (oldParent != null) {
            oldParent.removeEditorModificationListener(parentListener);
        }
        context = newContext;
        ObjectEditor newParent = getParentEditor();
        if (newParent != null) {
            newParent.addEditorModificationListener(parentListener);
        }
    }


    protected void parentModified(EditorModificationEvent eme) {
        switch (eme.getModificationType()) {
            case EDITED_VALUE:
                if (getVisibilityAccess() != null) checkViewVisibility();
                if ((getEditableAccess() != null) || (getEnabledAccess() != null)) synchronizeEditableState();
                break;
            case COMPONENT_EDITABLE:
                synchronizeEditableState();
                break;
            case COMPONENT_VISIBILITY:
                checkViewVisibility();
                break;
            case COMPONENT_LAYOUT:
                // do nothing
        }
    }


    public EditorContext getEditorContext() {
        return context;
    }


    public boolean isEditable() {
        boolean editable = false;
        if ((getEditedObject() != null)) {
            if (getEditableAccess() != null) {
                editable = ((Boolean) getEditableAccess().getModelValue(getEditedObject())).booleanValue();
            } else {
                if (getModelAccess().canUpdateModel()) {
                    ObjectEditor parent = getParentEditor();
                    editable = (parent == null) || parent.isEditable();
                }
            }
        }
        return editable;
    }


    public boolean isEnabled() {
        boolean enabled = false;
        if ((getEditedObject() != null)) {
            if (getEnabledAccess() != null) {
                enabled = ((Boolean) getEnabledAccess().getModelValue(getEditedObject())).booleanValue();
            } else {
                ObjectEditor parent = getParentEditor();
                enabled = (parent == null) || parent.isEnabled();
            }
        }
        return enabled;
    }


    public boolean isVisible() {
        ObjectEditor parent = getParentEditor();
        boolean visible = (parent == null) || parent.isVisible();
        if (visible && (getVisibilityAccess() != null)) {
            visible = ((Boolean) getVisibilityAccess().getModelValue(getEditedObject())).booleanValue();
        }
        return visible;
    }


    /**
     * Must be implemented by sub-classes to synchronize the component disabled and read-only state with corresponding internal
     * flags of this class.
     */
    protected abstract void synchronizeEditableState();


    public Settings getSettings() {
        return (context == null) ? null : context.getSettings();
    }


    public JComponent getLabelComponent() {
        if ((labelComponent == null) && (labelAccess != null)) {
            labelComponent = createLabelComponent();
            updateLabelText();
            updateDescriptionText();
        }
        return labelComponent;
    }


    protected JComponent createLabelComponent() {
        JComponent lbl = null;
        if (labelAccess != null) {
            lbl = new JLabel();
        }
        return lbl;
    }


    protected void updateLabelText() {
        if ((labelComponent != null) && (labelComponent instanceof JLabel)) {
            ((JLabel) labelComponent).setText(getTranslatedLabel());
        }
    }


    protected void updateDescriptionText() {
        if (labelComponent != null) {
            labelComponent.setToolTipText(getTranslatedDescription());
        }
    }


    public void setProperty(String name, Object value) {
        if (name.equals(ObjectEditor.EDITOR_LABEL_PROPERTY)) {
            setLabelAccess(ModelAccessFactory.createAccess(value, String.class, getEditedObjectClass(), true));
        } else if (name.equals(ObjectEditor.EDITOR_DESCRIPTION_PROPERTY)) {
            setDescriptionAccess(ModelAccessFactory.createAccess(value, String.class, getEditedObjectClass(), true));
        } else if (name.equals(ObjectEditor.VISIBILITY_PROPERTY)) {
            setVisibilityAccess(ModelAccessFactory.createAccess(value, Boolean.class, getEditedObjectClass(), false));
        } else if (name.equals(ObjectEditor.EDITABLE_PROPERTY)) {
            setEditableAccess(ModelAccessFactory.createAccess(value, Boolean.class, getEditedObjectClass(), false));
        } else if (name.equals(ObjectEditor.ENABLED_PROPERTY)) {
            setEnabledAccess(ModelAccessFactory.createAccess(value, Boolean.class, getEditedObjectClass(), false));
        } else {
            if (properties == null) properties = new HashMap<String, Object>();
            properties.put(name, value);
        }
    }


    protected Class<?> getEditedObjectClass() {
        return getModelAccess().getEditedObjectClass();
    }


    protected Class<?> getModelClass() {
        return getModelAccess().getModelClass();
    }


    public Object getProperty(String name, Object defaultValue) {
        if (name.equals(ObjectEditor.EDITOR_LABEL_PROPERTY)) {
            return getLabelAccess();
        } else if (name.equals(ObjectEditor.EDITOR_DESCRIPTION_PROPERTY)) {
            return getDescriptionAccess();
        } else if (name.equals(ObjectEditor.VISIBILITY_PROPERTY)) {
            return getVisibilityAccess();
        } else if (name.equals(ObjectEditor.EDITABLE_PROPERTY)) {
            return getEditableAccess();
        } else if (name.equals(ObjectEditor.ENABLED_PROPERTY)) {
            return getEnabledAccess();
        } else {
            Object propValue = (properties == null) ? null : properties.get(name);
            return (propValue == null) ? defaultValue : propValue;
        }
    }


    public int getIntProperty(String name, int defaultValue) {
        Object value = getProperty(name, null);
        if (value == null) return defaultValue;
        if (value instanceof Number) return ((Number) value).intValue();
        return Integer.parseInt(value.toString());
    }


    public double getDoubleProperty(String name, double defaultValue) {
        Object value = getProperty(name, null);
        if (value == null) return defaultValue;
        if (value instanceof Number) return ((Number) value).doubleValue();
        return Double.parseDouble(value.toString());
    }


    public void addEditorModificationListener(EditorModificationListener lis) {
        modificationSupport.addEditorModificationListener(lis);
    }


    public void removeEditorModificationListener(EditorModificationListener lis) {
        modificationSupport.removeEditorModificationListener(lis);
    }


    protected EditorModificationSupport getEditorModificationSupport() {
        return modificationSupport;
    }


    protected void registerFocusListener(Component cmp) {
        if (theFocusListener == null) {
            theFocusListener = new FocusListener() {

                public void focusLost(FocusEvent newE) {
                    updateModel();
                }


                public void focusGained(FocusEvent newE) {
                }

            };
        }
        cmp.addFocusListener(theFocusListener);
        if (cmp instanceof Container) {
            Container containr = (Container) cmp;
            int len = containr.getComponentCount();
            for (int i = 0; i < len; i++) {
                registerFocusListener(containr.getComponent(i));
            }
        }
    }

}
