/*-------------------------------------------------------------------------
 Copyright 2006 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.AbstractListModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;

import net.sf.sfac.lang.LanguageSupport;


/**
 * This class is an editor for java.awt.Color objects.
 */
public class ColorEditor extends ComboBoxEditor {

    static final Color[] DEFAULT_COLORS = new Color[] { null, Color.WHITE, Color.YELLOW, Color.ORANGE, Color.RED, Color.PINK,
            Color.MAGENTA, Color.CYAN, Color.blue, Color.GREEN, Color.LIGHT_GRAY, Color.GRAY, Color.DARK_GRAY, Color.BLACK };
    static final String[] DEFAULT_COLOR_NAMES = new String[] { "<Default color>", "White", "Yellow", "Orange", "Red", "Pink",
            "Magenta", "Cyan", "Blue", "Green", "Light gray", "Gray", "Dark gray", "Black" };

    static final Color DEFAULT_BACKGROUND_COLOR = new JLabel().getBackground();
    static final Dimension DEFAULT_SIZE = new Dimension(16, 24);


    public ColorEditor() {
        super(new ColorEditorModel());
        setRenderer(new ColorRenderer());
    }


    @Override
    protected void fireObjectSelectionChanged() {
        super.fireObjectSelectionChanged();
        JComboBox comboBox = getComboBox();
        Color selected = (Color) comboBox.getSelectedItem();
        if (selected == null) selected = DEFAULT_BACKGROUND_COLOR;
        comboBox.setBackground(selected);
    }


    /**
     * Enable the 'choose color' button. <br>
     * Note: The aim of the 'open' button of the ComboBoxEditor was different, the original intention is
     * <i>"view details of selected objects"</i> while here it is <i>"open color chooser"</i>. It's the reason why the two methods
     * <code>canViewItem()</code> and <code>openSelectedObject()</code> have strange names in this context.
     */
    @Override
    protected boolean canViewItem() {
        return true;
    }


    /**
     * Show the color chooser. <br>
     * Note: The aim of the 'open' button of the ComboBoxEditor was different, the original intention is
     * <i>"view details of selected objects"</i> while here it is <i>"open color chooser"</i>. It's the reason why the two methods
     * <code>canViewItem()</code> and <code>openSelectedObject()</code> have strange names in this context.
     */
    @Override
    protected void openSelectedObject() {
        Color current = (Color) getViewValue();
        if (current == null) current = Color.WHITE;
        String chooserTitle = LanguageSupport.getOptionalLocalizedString("SELECT_COLOR", "Select a color");
        Color chosen = JColorChooser.showDialog(getComboBox(), chooserTitle, current);
        if (chosen != null) {
            setViewValue(chosen);
            updateModel();
        }
    }

    @SuppressWarnings("serial")
    static class ColorEditorModel extends AbstractListModel implements ComboBoxEditorModel {

        private Color selected;


        ColorEditorModel() {
        }


        public void fireAllDataChanged() {
            fireContentsChanged(this, 0, getSize());
        }


        public Object getSelectedItem() {
            return selected;
        }


        public void setSelectedItem(Object newSelected) {
            if (selected != newSelected) {
                selected = (Color) newSelected;
                fireContentsChanged(this, -1, -1);
            }
        }


        public Object getElementAt(int index) {
            return DEFAULT_COLORS[index];
        }


        public int getSize() {
            return DEFAULT_COLORS.length;
        }

    }

    @SuppressWarnings("serial")
    static class ColorRenderer extends DefaultListCellRenderer {

        @Override
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean hasFocus) {
            Color col = (value == null) ? DEFAULT_BACKGROUND_COLOR : (Color) value;
            if (index < 0) index = indexOf(value);
            String text = (index >= 0) ? DEFAULT_COLOR_NAMES[index] : "Custom color";
            super.getListCellRendererComponent(list, text, index, isSelected, hasFocus);
            setOpaque(true);
            setBackground(col);
            setPreferredSize(DEFAULT_SIZE);
            return this;
        }


        private int indexOf(Object value) {
            if (value == null) return 0;
            int len = DEFAULT_COLORS.length;
            for (int i = 0; i < len; i++) {
                if (value.equals(DEFAULT_COLORS[i])) return i;
            }
            return -1;
        }
    }

}
