/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import net.sf.sfac.gui.editor.EditorContext;
import net.sf.sfac.gui.utils.ComboBoxKeyManager;
import net.sf.sfac.gui.utils.ObjectSelectionListener;
import net.sf.sfac.gui.utils.ObjectSelector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Abstract editor for object to be displayed in JComboBox. <br>
 * Clients of this editor must provide a ComboBoxModel.
 * 
 * @author Olivier Berlanger
 */
public class ComboBoxEditor extends SimpleObjectEditor implements ObjectSelector {


    private static Log log = LogFactory.getLog(ComboBoxEditor.class);
    private ComboBoxEditorModel comboModel;
    private List<ObjectSelectionListener> listeners;

    private JPanel editionPane;
    private JPanel objectPane;
    private JComboBox combo;
    private ListCellRenderer renderer;
    private JButton openButton;
    private boolean allowedToOpen = true;


    public ComboBoxEditor(ComboBoxEditorModel model) {
        comboModel = model;
        comboModel.addListDataListener(new ListDataListener() {


            public void intervalRemoved(ListDataEvent e) {
            }


            public void intervalAdded(ListDataEvent e) {
            }


            public void contentsChanged(ListDataEvent e) {
                // this is the hack used by comboBox models to fire selection change.
                if ((e.getIndex0() < 0) && (e.getIndex1() < 0)) {
                    updateModel();
                    fireObjectSelectionChanged();
                }
            }

        });
    }


    public ComboBoxEditorModel getComboBoxModel() {
        return comboModel;
    }


    public ListCellRenderer getRenderer() {
        return renderer;
    }


    public void setRenderer(ListCellRenderer comboRenderer) {
        renderer = comboRenderer;
    }


    public boolean isAllowedToOpen() {
        return allowedToOpen;
    }


    public void setAllowedToOpen(boolean newAllowedToOpen) {
        allowedToOpen = newAllowedToOpen;
    }


    @Override
    public void updateView() {
        try {
            // block all possible unwanted events during update view
            getEditorModificationSupport().setEnableEvents(false);
            comboModel.fireAllDataChanged();
            super.updateView();
        } finally {
            // allow events again
            getEditorModificationSupport().setEnableEvents(true);
        }
    }


    @Override
    protected boolean isViewReady() {
        return (combo != null);
    }


    @Override
    protected Object getViewValue() {
        return comboModel.getSelectedItem();
    }


    @Override
    protected void setViewValue(Object modelValue) {
        comboModel.setSelectedItem(modelValue);
    }


    @Override
    protected void synchronizeEditableState() {
        if (combo != null) {
            boolean listEnabled = isEditable() && isEnabled();
            combo.setEnabled(listEnabled);
        }
    }


    public JComponent getObjectComponent() {
        if (objectPane == null) {
            objectPane = new JPanel(new BorderLayout());
            combo = new JComboBox(comboModel);
            combo.setKeySelectionManager(new ComboBoxKeyManager());
            combo.addItemListener(new ItemListener() {


                public void itemStateChanged(ItemEvent ie) {
                    if (ie.getStateChange() == ItemEvent.SELECTED) {
                        updateModel();
                    }
                }

            });
            if (renderer != null) combo.setRenderer(renderer);
            objectPane.add(combo, BorderLayout.CENTER);
            if ((getEditorContext() != null) && isAllowedToOpen()) {
                openButton = new JButton("...");
                objectPane.add(openButton, BorderLayout.EAST);
                openButton.addActionListener(new ActionListener() {


                    public void actionPerformed(ActionEvent newE) {
                        openSelectedObject();
                    }

                });
                openButton.setEnabled(canViewItem());
            }
            updateView();
            synchronizeEditableState();
        }
        return objectPane;
    }


    protected boolean canViewItem() {
        EditorContext edContext = getEditorContext();
        return (edContext != null) && edContext.canEditInContext(false, getModelAccess().getModelClass(), this);
    }


    @SuppressWarnings("unchecked")
    protected void openSelectedObject() {
        EditorContext editorContext = getEditorContext();
        Object selected = comboModel.getSelectedItem();
        if (editorContext != null) {
            openButton.setEnabled(false);
            Object result = editorContext.editInContext(selected, (Class<Object>) getModelAccess().getModelClass(), this);
            if ((result != null) && isEditable() && isEnabled()) {
                setViewValue(result);
                updateModel();
            }
            openButton.setEnabled(true);
        } else {
            log.warn("Cannot open " + selected + " because editorContext is not defined");
        }
    }


    @Override
    public JComponent getEditorComponent() {
        if (editionPane == null) {
            editionPane = new JPanel(new BorderLayout());
            editionPane.add(getObjectComponent(), BorderLayout.CENTER);
            editionPane.add(getLabelComponent(), BorderLayout.WEST);
        }
        return editionPane;
    }


    protected void fireObjectSelectionChanged() {
        if (listeners != null) {
            for (ObjectSelectionListener lis : listeners) {
                lis.objectSelectionChanged(this);
            }
        }
    }


    protected JComboBox getComboBox() {
        return combo;
    }


    // ------------------- ObjectSelector interface implementation --------------------------------------


    public String getId() {
        return getClass().getSimpleName() + "@" + getModelAccess().getFieldPath();
    }


    public Object getSelectedObject() {
        return comboModel.getSelectedItem();
    }


    public void clearSelection() {
        comboModel.setSelectedItem(null);
    }


    public void selectDefaultItem() {
        if ((combo != null) && (comboModel.getSize() > 0)) {
            combo.setSelectedIndex(0);
        }
    }


    public int getSelectedObjectCount() {
        return (comboModel.getSelectedItem() == null) ? 0 : 1;
    }


    public List<Object> getSelectedObjects() {
        Object selected = getSelectedObject();
        return (selected == null) ? null : Collections.singletonList(selected);
    }


    public boolean isMultiSelectionEnabled() {
        return false;
    }


    public boolean canSelectObjects(List<?> toSelect) {
        return (toSelect == null) || (toSelect.size() <= 1);
    }


    public void selectObjects(List<?> toSelect) {
        Object newSelection = null;
        if ((toSelect != null) && (toSelect.size() == 1)) {
            newSelection = toSelect.get(0);
        }
        comboModel.setSelectedItem(newSelection);
    }


    public void addObjectSelectionListener(ObjectSelectionListener lis) {
        if (listeners == null) listeners = new ArrayList<ObjectSelectionListener>();
        listeners.add(lis);
    }


    public void removeObjectSelectionListener(ObjectSelectionListener lis) {
        if (listeners != null) {
            listeners.remove(lis);
        }
    }

}
