/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.Component;
import java.awt.Container;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JComponent;

import net.sf.sfac.gui.editor.ObjectEditor;


/**
 * Special editor editing a enumerated value as usual but adding the possibility to control the visibility of visual components
 * of other editors. <br>
 * To achieve this visibility control, just implements the <code>updateComponentVisibility</code> method with calls to the
 * <code>setComponentVisible</code> method.
 * 
 * @author Olivier Berlanger
 */
public abstract class ComponentVisibilityEditor extends EnumEditor {


    private Map<String, ComponentHolder> holders;


    @Override
    public void updateView() {
        super.updateView();
        updateComponentVisibility();
    }


    @Override
    public void updateModel() {
        super.updateModel();
        if (holders != null) updateComponentVisibility();
    }


    public abstract void updateComponentVisibility();


    protected void setComponentVisible(String editorPath, boolean visible) {
        if (holders == null) holders = new HashMap<String, ComponentHolder>();
        ComponentHolder holder = holders.get(editorPath);
        if (holder == null) {
            ObjectEditor ed = getSiblingEditor(editorPath);
            if (ed == null)
                throw new IllegalArgumentException("No editor with path '" + editorPath + "' in context " + getEditorContext());
            holder = new ComponentHolder(ed);
        }
        holder.setVisible(visible);
    }


    protected ObjectEditor getSiblingEditor(String editorPath) {
        for (ObjectEditor child : getEditorContext().getEditors()) {
            if (editorPath.equals(child.getModelAccess().getFieldPath())) return child;
        }
        return null;
    }


    private static class ComponentHolder {


        private JComponent label;
        private JComponent objectCmp;


        ComponentHolder(ObjectEditor ed) {
            label = ed.getLabelComponent();
            objectCmp = ed.getObjectComponent();
        }


        public void setVisible(boolean visible) {
            setComponentVisible(label, visible);
            setComponentVisible(objectCmp, visible);
        }


        private void setComponentVisible(Component cmp, boolean visible) {
            if (cmp != null) {
                cmp.setVisible(visible);
                if (cmp instanceof Container) {
                    Component[] subComponents = ((Container) cmp).getComponents();
                    if (subComponents != null) {
                        for (Component sub : subComponents) {
                            setComponentVisible(sub, visible);
                        }
                    }
                }
            }
        }

    }


}
