/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import net.sf.sfac.gui.UiUtils;
import net.sf.sfac.gui.cmp.RollOverSplitPane;
import net.sf.sfac.gui.editor.EditorContext;
import net.sf.sfac.gui.editor.EditorFactory;
import net.sf.sfac.gui.editor.EditorModificationEvent;
import net.sf.sfac.gui.editor.EditorModificationListener;
import net.sf.sfac.gui.editor.ObjectEditor;
import net.sf.sfac.gui.editor.access.ReflexionModelAccess;
import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.utils.Comparison;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Base implementation for ObjectEditorContainer. <br>
 * This class performs the layout of child components (from child editors) using a gridbagLayout. The labels are added in column 0
 * and the object editors are added in column 1. <br>
 * Following properties can be set on child editors to customize the way the child components are added in the generated form:
 * <ul>
 * <li>compositeObjectEditor.position : "left", "top" or "center" = if value is "left" or "top" the component is added at the left
 * or to of the created edition form (using a split pane). If value is center (= the default), the component is added in the edition
 * form (using gridBagLayout).
 * <li>tab : String = the container will create a tabbed pane (for the form) and put the given object on a tab with the given name.
 * </ul>
 * <p>
 * As the CompositeObjectEditor uses internally a GridBagLayout, most of the default GridBagConstraints values can be modified:
 * </p>
 * <ul>
 * <li>gridx : Integer = the gridx gridbagConstraints value to use when adding the object component in gridBagLayout. When defined,
 * the label is placed at (gridx-1). The default value is 1.
 * <li>gridy : Integer = the gridx gridbagConstraints value to use when adding the object component in gridBagLayout. By default
 * gridy set to the next available value (so the label and object component are added just below the previous one). If you gives a
 * positive number, it will take this value 'as is' and put label and object component at the specified row. Then the next
 * sub-editor will be added just below this row. If you give a negative number, it will be added relatively to the next available
 * value (so -1 means same line as the preceding sub-editor). In this case, the the next available value is not affected.
 * <li>gridwidth : Integer = the gridwidth gridbagConstraints value to use when adding the object component in gridBagLayout (1 by
 * default). Note: this value doesn't affect the label which always use gridwith=1.
 * <li>gridheight : Integer = the gridheight gridbagConstraints value to use when adding the label and object component in
 * gridBagLayout (1 by default).
 * <li>weightx : Double = the weightx gridbagConstraints value to use when adding the object component in the gridbagLayout (0.0 by
 * default). Note: this value doesn't affect the label which always use weightx=0.
 * <li>weighty : Double = the weighty gridbagConstraints value to use when adding the object component in the gridbagLayout (0.0 by
 * default). Note: this value doesn't affect the label which always use weighty=0.
 * <li>fill : Integer = the fill gridbagConstraints value to use when adding the object component in the gridbagLayout
 * (GridBagConstraints.HORIZONTAL by default). Note: this value doesn't affect the label which always use fill=NONE.
 * <li>anchor : Integer = the anchor gridbagConstraints value to use when adding the object component in the gridbagLayout
 * (GridBagConstraints.WEST by default). Note: this value doesn't affect the label which always use anchor=EAST.
 * </ul>
 * 
 * @author Olivier Berlanger
 */
public class CompositeObjectEditor extends BaseObjectEditor implements EditorContext {

    private static Log log = LogFactory.getLog(CompositeObjectEditor.class);

    private static final EditorComparator EDITOR_COMPARATOR = new EditorComparator();
    public static final Integer MAX_INTEGER = Integer.valueOf(Integer.MAX_VALUE);
    private static final int UNSPECIFIED = Integer.MIN_VALUE;

    private List<ObjectEditor> subEditors;
    private boolean editable = true;
    private boolean enabled = true;

    private JComponent editorCmp;
    private JComponent leftCmp;
    private JComponent topCmp;
    private JComponent objectCmp;
    private JTabbedPane tabbedPane;
    private JLabel descriptionLabel;

    /** The number of gridBagLayout columns used by the object components of child editors (1 by default). */
    private int nbrColumnsForEditor = 1;
    /**
     * A flag saying to add an empty label at last row to fill the remaining space. <br>
     * By default it is true if no child component was added with vertical weight.
     */
    private boolean addEmptyLabelAtEnd;


    public CompositeObjectEditor() {
        subEditors = new ArrayList<ObjectEditor>();
    }


    @Override
    public void edit(Object objToEdit) {
        super.edit(objToEdit);
        Object edited = getModelValue();
        int len = getChildEditorCount();
        for (int i = 0; i < len; i++) {
            getChildEditorAt(i).edit(edited);
        }
    }


    public JComponent getEditorComponent() {
        if (editorCmp == null) {
            editorCmp = createEditorComponent();
        }
        return editorCmp;
    }


    /**
     * Get the ID of this editor (used to store settings).
     * 
     * @return
     */
    protected String getEditorId() {
        return getClass().getSimpleName();
    }


    protected JComponent createEditorComponent() {
        JComponent editorPane;
        JComponent centerComponent = getObjectComponent();
        // potentially add left pane
        if (leftCmp != null) {
            RollOverSplitPane roll = new RollOverSplitPane(RollOverSplitPane.HORIZONTAL_SPLIT, leftCmp, centerComponent);
            String splitLocationKey = "editor." + getEditorId() + ".leftSplitLocation";
            roll.useSettings(getSettings(), splitLocationKey, 300);
            centerComponent = roll;
        }
        // potentially add top pane
        if (topCmp != null) {
            centerComponent.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
            RollOverSplitPane roll = new RollOverSplitPane(RollOverSplitPane.VERTICAL_SPLIT, topCmp, centerComponent);
            String splitLocationKey = "editor." + getEditorId() + ".topSplitLocation";
            roll.useSettings(getSettings(), splitLocationKey, 300);
            editorPane = roll;
        } else {
            JComponent titleLbl = getLabelComponent();
            if (titleLbl != null) {
                // String
                centerComponent.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
                editorPane = new JPanel(new BorderLayout());
                UiUtils.resizeLabelFont(titleLbl, 2f, true);
                editorPane.add(titleLbl, BorderLayout.NORTH);
                editorPane.add(centerComponent, BorderLayout.CENTER);
            } else {
                editorPane = centerComponent;
            }
        }
        editorPane.setBorder(BorderFactory.createEmptyBorder(5, 10, 10, 10));
        return editorPane;
    }


    public JComponent getObjectComponent() {
        if (objectCmp == null) {
            sortChildEditors();
            objectCmp = createObjectComponent();
        }
        updateView();
        return objectCmp;
    }


    protected JComponent createObjectComponent() {
        addEmptyLabelAtEnd = true;
        JComponent cmp = createEmptyObjectComponent();
        // one row per sub-editor
        int len = getChildEditorCount();
        int row = 0;
        ObjectEditor childEditor;
        row = addDescriptionLabel(cmp, row);
        int maxRow = row;
        for (int i = 0; i < len; i++) {
            childEditor = getChildEditorAt(i);
            String position = (String) childEditor.getProperty("compositeObjectEditor.position", "center");
            if ("left".equals(position)) {
                if (leftCmp != null) throw new IllegalStateException("Already an object as left pane");
                leftCmp = childEditor.getObjectComponent();
            } else if ("top".equals(position)) {
                if (topCmp != null) throw new IllegalStateException("Already an object as top pane");
                topCmp = childEditor.getObjectComponent();
            } else {
                String tab = (String) childEditor.getProperty(ObjectEditor.TAB_PROPERTY, null);
                if (Comparison.isEmpty(tab)) {
                    row = addChildComponentInPanel(cmp, childEditor, row);
                } else {
                    String tabTitle = LanguageSupport.getLocalizedString(tab);
                    if (tabbedPane == null) {
                        tabbedPane = new JTabbedPane();
                        cmp.add(tabbedPane, new GridBagConstraints(0, 100, getNbrColumnsForEditor() + 1, 1, 1.0, 1.0,
                                GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
                    }
                    int tabIndex = tabbedPane.indexOfTab(tabTitle);
                    JPanel tabPanel = null;
                    if (tabIndex >= 0) {
                        tabPanel = (JPanel) tabbedPane.getComponent(tabIndex);
                    } else {
                        tabPanel = new JPanel(new GridBagLayout());
                        tabPanel.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
                        tabbedPane.addTab(tabTitle, tabPanel);
                    }
                    row = addChildComponentInPanel(tabPanel, childEditor, row);
                    addEmptyLabelAtEnd = false;
                }
            }
            if (maxRow < row) maxRow = row;
        }
        // last row: empty label to fill remaining space (if weighty = 0)
        addEmptyLabel(cmp, maxRow + 1);
        return cmp;
    }


    protected JComponent createEmptyObjectComponent() {
        return new JPanel(new GridBagLayout());
    }


    protected int addDescriptionLabel(JComponent pane, int row) {
        String descr = getTranslatedDescription();
        if (Comparison.isDefined(descr)) {
            descriptionLabel = new JLabel(descr);
            descriptionLabel.setHorizontalAlignment(JLabel.CENTER);
            descriptionLabel.setForeground(Color.DARK_GRAY);
            pane.add(descriptionLabel, new GridBagConstraints(0, row, getNbrColumnsForEditor() + 1, 1, 0.0, 0.0,
                    GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 10, 5), 0, 0));
            row++;
        }
        return row;
    }


    @Override
    protected void updateDescriptionText() {
        if (descriptionLabel != null) {
            descriptionLabel.setText(getTranslatedDescription());
        }
    }


    protected int addChildComponentInPanel(JComponent pane, ObjectEditor childEditor, int row) {
        // get the overridden constraints from properties
        int gridx = childEditor.getIntProperty("gridx", 1);
        int gridy = childEditor.getIntProperty("gridy", row);
        int gridwidth = childEditor.getIntProperty("gridwidth", getNbrColumnsForEditor());
        int gridheight = childEditor.getIntProperty("gridwidth", 1);
        double weightx = childEditor.getDoubleProperty("weightx", 0.0);
        double weighty = childEditor.getDoubleProperty("weighty", 0.0);
        int fill = childEditor.getIntProperty("fill", GridBagConstraints.HORIZONTAL);
        int anchor = childEditor.getIntProperty("anchor", GridBagConstraints.WEST);
        int nextRow;
        if (gridy < 0) {
            // take value relative to current row
            gridy = row + gridy;
            // in this case the 'next row' will be unaffected
            nextRow = row;
        } else {
            // normal case: the next row will be just below the one used here
            nextRow = gridy + 1;
        }
        if (gridwidth == UNSPECIFIED) gridwidth = getNbrColumnsForEditor();
        // add label and object component for this child editor
        JComponent label;
        JComponent subEditorCmp;
        label = childEditor.getLabelComponent();
        subEditorCmp = childEditor.getObjectComponent();
        if (log.isDebugEnabled()) log.debug("Add components of child editor: " + childEditor + " at position " + gridx + ","
                + gridy);
        if (weighty > 0.0) addEmptyLabelAtEnd = false;
        if (label != null) {
            if (label instanceof JLabel) {
                ((JLabel) label).setLabelFor(subEditorCmp);
            }
            pane.add(label, new GridBagConstraints(gridx - 1, gridy, 1, gridheight, 0.0, 0.0, GridBagConstraints.EAST,
                    GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
        }
        pane.add(subEditorCmp, new GridBagConstraints(gridx, gridy, gridwidth, gridheight, weightx, weighty, anchor, fill,
                new Insets(0, 0, 0, 0), 0, 0));
        return nextRow;
    }


    protected void addEmptyLabel(JComponent pane, int row) {
        if (addEmptyLabelAtEnd) {
            if (log.isDebugEnabled()) log.debug("Adding empty label at grid position: 0," + row + " with weighty=1.0");
            JLabel emptyLabel = new JLabel();
            pane.add(emptyLabel, new GridBagConstraints(0, row, 1, 1, 0.0, 1.0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));
        }
    }


    public void updateView() {
        try {
            // block all possible unwanted events during update view
            getEditorModificationSupport().setEnableEvents(false);
            updateLabelText();
            updateDescriptionText();
            int len = getChildEditorCount();
            for (int i = 0; i < len; i++) {
                getChildEditorAt(i).updateView();
            }
        } finally {
            // allow events again
            getEditorModificationSupport().setEnableEvents(true);
        }
    }


    @Override
    protected void synchronizeEditableState() {
        boolean newEditable = isEditable();
        boolean newEnabled = isEnabled();
        if ((editable != newEditable) || (enabled != newEnabled)) {
            editable = newEditable;
            enabled = newEnabled;
            getEditorModificationSupport().fireEditorEditabilityModified();
        }
    }


    public void updateModel() {
        if (isEditable() && isEnabled()) {
            int len = getChildEditorCount();
            for (int i = 0; i < len; i++) {
                getChildEditorAt(i).updateModel();
            }
        }
    }


    /**
     * Sort the child editors using the "index" property of those editors.
     */
    protected void sortChildEditors() {
        Collections.sort(subEditors, EDITOR_COMPARATOR);
    }

    static class EditorComparator implements Comparator<ObjectEditor> {

        public int compare(ObjectEditor ed1, ObjectEditor ed2) {
            Integer index1 = (Integer) ed1.getProperty("index", MAX_INTEGER);
            Integer index2 = (Integer) ed2.getProperty("index", MAX_INTEGER);
            return Comparison.compareTo(index1, index2);
        }

    }


    // -------------------- ObjectEditorContainer interface implementation -------------------------------------------------

    public int getChildEditorCount() {
        return subEditors.size();
    }


    public ObjectEditor getChildEditor(String childEditorPath) {
        for (ObjectEditor ed : subEditors) {
            if (childEditorPath.equals(ed.getModelAccess().getFieldPath())) return ed;
        }
        return null;
    }


    public ObjectEditor getChildEditorAt(int index) {
        return subEditors.get(index);
    }


    public void addChildEditor(ObjectEditor newChild) {
        newChild.setEditorContext(this);
        newChild.addEditorModificationListener(new EditorModificationListener() {

            public void editorModified(EditorModificationEvent eme) {
                subEditorModified(eme);
            }
        });
        subEditors.add(newChild);
    }


    public void addChildEditor(ObjectEditor newChild, String fieldName, String label, int index) {
        Class<?> modelClass = getModelClass();
        if ((modelClass == null) || (modelClass == Object.class)) {
            throw new IllegalStateException("The main editor model (and ModelAccess) must be defined before to call method"
                    + " addChildEditor(ObjectEditor, fieldName, label, index)");
        }
        newChild.setModelAccess(new ReflexionModelAccess(fieldName, modelClass));
        if (label != null) newChild.setProperty(ObjectEditor.EDITOR_LABEL_PROPERTY, label);
        if (index >= 0) newChild.setProperty(ObjectEditor.EDITOR_INDEX_PROPERTY, Integer.valueOf(index));
        addChildEditor(newChild);
    }


    void subEditorModified(EditorModificationEvent eme) {
        switch (eme.getModificationType()) {
            case COMPONENT_LAYOUT:
            case COMPONENT_VISIBILITY:
                relayout();
                break;
            case EDITED_VALUE:
            case COMPONENT_EDITABLE:
                // do nothing
        }
        getEditorModificationSupport().fireSubEditorModified(eme);
    }


    private void relayout() {
        if (editorCmp != null) {
            editorCmp.invalidate();
            editorCmp.repaint();
        } else if (objectCmp != null) {
            objectCmp.invalidate();
            objectCmp.repaint();
        }
    }


    public void removeChildEditor(ObjectEditor child) {
        subEditors.remove(child);
        child.setEditorContext(null);
        child.removeEditorModificationListener(getEditorModificationSupport().getListenerForSubEditors());
    }


    // -------------------- EditorContext interface implementation -------------------------------------------------

    public boolean canEditInContext(boolean create, Class<?> objectClass, ObjectEditor source) {
        EditorContext context = getEditorContext();
        return (context == null) ? false : context.canEditInContext(create, objectClass, source);
    }


    public <T> T editInContext(T obj, Class<T> objectClass, ObjectEditor source) {
        return getEditorContext().editInContext(obj, objectClass, source);
    }


    public Object getContextObject(String objectKey) {
        return getEditorContext().getContextObject(objectKey);
    }


    public EditorFactory getEditorFactory() {
        EditorContext editorContext = getEditorContext();
        EditorFactory factory = (editorContext == null) ? null : editorContext.getEditorFactory();
        return (factory == null) ? EditorFactory.getInstance() : factory;
    }


    /**
     * Return the sub-editors of this editor container.
     */
    public Collection<ObjectEditor> getEditors() {
        return subEditors;
    }


    protected int getNbrColumnsForEditor() {
        return nbrColumnsForEditor;
    }


    protected void setNbrColumnsForEditor(int nbrColumns) {
        nbrColumnsForEditor = nbrColumns;
    }

}
