/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Arrays;

import javax.swing.AbstractListModel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import net.sf.sfac.gui.editor.ModelAccess;
import net.sf.sfac.gui.utils.ComboBoxKeyManager;
import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.utils.Comparison;


/**
 * Editor used to edit a enum value. <br>
 * 
 * @author Olivier Berlanger
 */
public class EnumEditor extends SimpleObjectEditor {

    private EnumHolder[] enumValues;
    private JComponent component;
    private JRadioButton[] radios;
    private JComboBox combo;
    private boolean allowNullValue = true;


    @SuppressWarnings("unchecked")
    @Override
    public void setModelAccess(ModelAccess newAccess) {
        super.setModelAccess(newAccess);
        initEnumValues((Class<? extends Enum<?>>) newAccess.getModelClass());
    }


    private void initEnumValues(Class<? extends Enum<?>> enumerationClass) {
        Enum<?>[] values = enumerationClass.getEnumConstants();
        if (values == null) throw new IllegalArgumentException("" + enumerationClass + " does not have accessible enum values");
        int len = values.length;
        enumValues = new EnumHolder[len];
        for (int i = 0; i < len; i++) {
            enumValues[i] = new EnumHolder(values[i]);
        }
        Arrays.sort(enumValues);
    }


    public JComponent getObjectComponent() {
        if (component == null) {
            if (enumValues.length > 4) {
                // Use a JComboBox
                combo = new JComboBox(new EnumComboModel());
                combo.setKeySelectionManager(new ComboBoxKeyManager());
                combo.addItemListener(new ItemListener() {

                    public void itemStateChanged(ItemEvent ie) {
                        if (ie.getStateChange() == ItemEvent.SELECTED) {
                            updateModel();
                        }
                    }

                });
                component = combo;
            } else {
                component = new JPanel();
                component.setLayout(new BoxLayout(component, BoxLayout.X_AXIS));
                ButtonGroup group = new ButtonGroup();
                int len = enumValues.length;
                radios = new JRadioButton[len];
                ActionListener al = new ActionListener() {

                    public void actionPerformed(ActionEvent newE) {
                        updateModel();
                    }

                };
                for (int i = 0; i < len; i++) {
                    JRadioButton radio = new JRadioButton(String.valueOf(enumValues[i]));
                    radio.addActionListener(al);
                    adjustSize(radio);
                    group.add(radio);
                    if (i > 0) component.add(Box.createHorizontalStrut(5));
                    component.add(radio);
                    radios[i] = radio;
                }
            }
            synchronizeEditableState();
            updateView();
        }
        return component;
    }


    @Override
    protected void synchronizeEditableState() {
        boolean cmpEnabled = isEditable() && isEnabled();
        if (combo != null) {
            combo.setEnabled(cmpEnabled);
        } else if (radios != null) {
            for (JRadioButton radio : radios) {
                radio.setEnabled(cmpEnabled);
            }
        }
    }


    @Override
    protected boolean isViewReady() {
        return (combo != null) || (radios != null);
    }


    @Override
    protected Object getViewValue() {
        Object value = null;
        if (combo != null) {
            EnumHolder holder = (EnumHolder) combo.getSelectedItem();
            value = (holder == null) ? null : holder.getModelValue();
        } else if (radios != null) {
            int len = enumValues.length;
            for (int i = 0; i < len; i++) {
                if (radios[i].isSelected()) {
                    value = enumValues[i].getModelValue();
                    break;
                }
            }
        }
        return value;
    }


    @Override
    protected void setViewValue(Object modelValue) {
        if (combo != null) {
            combo.setSelectedItem(getHolder((Enum<?>) modelValue));
        } else if (radios != null) {
            int len = enumValues.length;
            for (int i = 0; i < len; i++) {
                if (enumValues[i].getModelValue().equals(modelValue)) {
                    radios[i].setSelected(true);
                    break;
                }
            }
        }
    }


    private EnumHolder getHolder(Enum<?> modelValue) {
        if (modelValue != null) {
            int len = enumValues.length;
            for (int i = 0; i < len; i++) {
                if (enumValues[i].getModelValue().equals(modelValue)) {
                    return enumValues[i];
                }
            }
        }
        return null;
    }


    boolean getAllowNullValue() {
        return allowNullValue;
    }


    protected EnumHolder[] getEnumValues() {
        return enumValues;
    }

    @SuppressWarnings("serial")
    class EnumComboModel extends AbstractListModel implements ComboBoxModel {

        private Object selected;


        public Object getSelectedItem() {
            return selected;
        }


        public void setSelectedItem(Object anItem) {
            if ((anItem != null) || getAllowNullValue()) {
                selected = anItem;
                fireContentsChanged(this, -1, -1);
            }
        }


        public Object getElementAt(int newIndex) {
            return getEnumValues()[newIndex];
        }


        public int getSize() {
            return getEnumValues().length;
        }

    }

    static class EnumHolder implements Comparable<EnumHolder> {

        private Enum<?> modelValue;
        private String displayValue;


        public EnumHolder(Enum<?> value) {
            modelValue = value;
            String key = value.getClass().getSimpleName() + "." + modelValue.name();
            displayValue = LanguageSupport.getOptionalLocalizedString(key, modelValue.toString());
        }


        public boolean match(Enum<?> value) {
            return modelValue == value;
        }


        public Enum<?> getModelValue() {
            return modelValue;
        }


        @Override
        public String toString() {
            return displayValue;
        }


        public int compareTo(EnumHolder other) {
            if (other == null) return -1;
            return Comparison.compareStrings(displayValue, other.displayValue);
        }


        @Override
        public boolean equals(Object other) {
            if (other == null) return false;
            if (getClass() != other.getClass()) return false;
            return compareTo((EnumHolder) other) == 0;
        }


        @Override
        public int hashCode() {
            return (displayValue == null) ? 0 : displayValue.hashCode();
        }
    }

}
