/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.sfac.gui.UiUtils;
import net.sf.sfac.gui.editor.EditorContext;
import net.sf.sfac.gui.framework.ActionRepository;
import net.sf.sfac.gui.framework.SharedResources;
import net.sf.sfac.gui.utils.ObjectSelectionListener;
import net.sf.sfac.gui.utils.ObjectSelector;
import net.sf.sfac.lang.LanguageSupport;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Editor for list of objects to be displayed in a JList. <br>
 * The ListObjectModel delegates all access to the underlying indexed model access. <br>
 * Note: a JList cannot be used directly to edit the list. To have cell editing, rather use a one-column TableObjectEditor.
 * 
 * @author Olivier Berlanger
 */
public class ListObjectEditor extends SimpleObjectEditor implements ObjectSelector {

    private static Log log = LogFactory.getLog(ListObjectEditor.class);

    private ListObjectModel listModel;
    private List<ObjectSelectionListener> listeners;
    private PolymorphicObjectFactory factory;
    private List<Object> selectedObjects;

    private JComponent editorPane;
    private JPanel objectPane;
    private JList listCmp;
    private JScrollPane scroll;
    private JToolBar toolbar;
    private ListCellRenderer renderer;

    private ActionRepository repo;
    private Action viewAction;
    private Action addAction;
    private Action removeAction;
    private Action moveUpAction;
    private Action moveDownAction;


    public ListObjectEditor() {
        listModel = createListModel();
        selectedObjects = new ArrayList<Object>();
        setDefaultProperties();
    }


    public ListObjectEditor(ListObjectModel externalListModel) {
        listModel = externalListModel;
        listModel.setEditor(this);
        selectedObjects = new ArrayList<Object>();
        setDefaultProperties();
    }


    private void setDefaultProperties() {
        setProperty("weightx", Double.valueOf(1.0));
        setProperty("weighty", Double.valueOf(1.0));
        setProperty("fill", Integer.valueOf(GridBagConstraints.BOTH));
    }


    public PolymorphicObjectFactory getObjectFactory() {
        return factory;
    }


    public void setObjectFactory(PolymorphicObjectFactory newFactory) {
        factory = newFactory;
    }


    protected boolean canViewItem() {
        EditorContext edContext = getEditorContext();
        return (edContext != null) && edContext.canEditInContext(false, listModel.getItemClass(), this);
    }


    protected boolean canAddItem() {
        return listModel.canAddItem();
    }


    protected boolean canRemoveItem() {
        return listModel.canRemoveItem();
    }


    protected boolean canSwitchItems() {
        return listModel.canSwitchItems();
    }


    public ListCellRenderer getCellRenderer() {
        return renderer;
    }


    public void setCellRenderer(ListCellRenderer newRenderer) {
        renderer = newRenderer;
    }


    protected ListObjectModel createListModel() {
        return new ListObjectModel(this);
    }


    protected ListObjectModel getListModel() {
        return listModel;
    }


    /**
     * Get the JTable in a scrollpane.
     */
    public JComponent getScrollComponent() {
        if (scroll == null) {
            scroll = new JScrollPane(getListComponent());
        }
        return scroll;
    }


    /**
     * Get the JTable used by this editor.
     */
    public JList getListComponent() {
        if (listCmp == null) {
            listCmp = createListComponent();
            configureListComponent(listCmp);
            synchronizeEditableState();
            updateView();
        }
        return listCmp;
    }


    protected JList createListComponent() {
        return new JList(listModel);
    }


    protected void configureListComponent(JList createdList) {
        createdList.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    viewSelectedItem();
                }
            }
        });
        createdList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent e) {
                checkActionStates();
                checkObjectSelection();
            }

        });
        createdList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        ListCellRenderer cellRenderer = getCellRenderer();
        if (cellRenderer != null) createdList.setCellRenderer(cellRenderer);
    }


    void checkObjectSelection() {
        List<Object> newSelection = getListSelection();
        if (listsAreDifferent(selectedObjects, newSelection)) {
            selectedObjects = newSelection;
            fireObjectSelectionChanged();
        }
    }


    private boolean listsAreDifferent(List<Object> list1, List<Object> list2) {
        int len = list1.size();
        if (len != list2.size()) {
            return true;
        } else {
            for (int i = 0; i < len; i++) {
                if (!list2.contains(list1.get(i))) {
                    return true;
                }
            }
        }
        return false;
    }


    private List<Object> getListSelection() {
        List<Object> selected = new ArrayList<Object>();
        int[] selectionIndices = (listCmp == null) ? null : listCmp.getSelectedIndices();
        int len = (selectionIndices == null) ? 0 : selectionIndices.length;
        for (int i = 0; i < len; i++) {
            int index = selectionIndices[i];
            if ((index >= 0) && (index < listModel.getSize())) {
                selected.add(listModel.getElementAt(index));
            }
        }
        return selected;
    }


    void checkActionStates() {
        boolean enabled = (getEditedObject() != null) && isEditable() && isEnabled();
        int selectedIndex = getSelectedIndex();
        boolean selected = (selectedIndex >= 0);
        if (viewAction != null) {
            viewAction.setEnabled(selected);
        }
        if (addAction != null) {
            addAction.setEnabled(enabled);
        }
        if (removeAction != null) {
            removeAction.setEnabled(enabled && selected);
        }
        if (moveUpAction != null) {
            moveUpAction.setEnabled(canSwitchItems() && enabled && selected && (selectedIndex > 0));
            moveDownAction.setEnabled(canSwitchItems() && enabled && selected && (selectedIndex < listModel.getSize() - 1));
        }
    }


    protected int getSelectedIndex() {
        int selectedIndex = (listCmp == null) ? -1 : listCmp.getSelectedIndex();
        if (selectedIndex >= listModel.getSize()) {
            selectedIndex = -1;
        }
        return selectedIndex;
    }


    public JComponent getObjectComponent() {
        if (objectPane == null) {
            objectPane = new JPanel(new BorderLayout());
            objectPane.add(getToolBar(), BorderLayout.NORTH);
            objectPane.add(getScrollComponent(), BorderLayout.CENTER);
        }
        return objectPane;
    }


    @Override
    public JComponent getEditorComponent() {
        if (editorPane == null) {
            editorPane = new JPanel(new GridBagLayout());
            // add label
            JComponent theLabel = getLabelComponent();
            if (theLabel != null) {
                UiUtils.resizeLabelFont(theLabel, 1.4f, false);
                editorPane.add(theLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.WEST,
                        GridBagConstraints.NONE, new Insets(0, 4, 4, 0), 0, 0));
            }
            // toolbar
            repo = new ActionRepository();
            createActions(repo);
            if (repo.getToolbarActionCount() > 0) {
                toolbar = repo.createToolBar();
                editorPane.add(toolbar, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.EAST,
                        GridBagConstraints.NONE, new Insets(0, 0, 4, 0), 0, 0));
            }
            // add list
            editorPane.add(getScrollComponent(), new GridBagConstraints(0, 1, 2, 1, 1.0, 1.0, GridBagConstraints.WEST,
                    GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        }
        return editorPane;
    }


    protected JToolBar getToolBar() {
        if (toolbar == null) {
            repo = new ActionRepository();
            createActions(repo);
            toolbar = repo.createToolBar();
        }
        return toolbar;
    }


    @Override
    public void updateModel() {
    }


    @Override
    public void updateView() {
        try {
            // block all possible unwanted events during update view
            getEditorModificationSupport().setEnableEvents(false);
            listModel.fireDataChanged();
            checkObjectSelection();
            checkActionStates();
        } finally {
            // allow events again
            getEditorModificationSupport().setEnableEvents(true);
        }
    }


    @SuppressWarnings("unchecked")
    protected void viewSelectedItem() {
        if (canViewItem()) {
            Object selected = getSelectedObject();
            if (selected != null) getEditorContext().editInContext(selected, (Class<Object>) selected.getClass(), this);
        }
    }


    @Override
    protected void synchronizeEditableState() {
        if (listCmp != null) {
            boolean tableEnabled = isEditable() && isEnabled();
            listCmp.setEnabled(true);
            listCmp.setBackground(tableEnabled ? Color.WHITE : Color.LIGHT_GRAY);
            listCmp.setForeground(tableEnabled ? Color.BLACK : Color.DARK_GRAY);
            listCmp.repaint();
        }
        checkActionStates();
    }


    @SuppressWarnings("serial")
    private void createActions(ActionRepository repos) {
        // view selected Item
        if (canViewItem()) {
            viewAction = repos.addAction("viewItem", "View", "View selected item", null, null, "a0",
                    SharedResources.getIcon("view.png"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            viewSelectedItem();
                        }
                    });
        }
        // add new Item
        if (canAddItem()) {
            addAction = repos.addAction("addItem", "Add", "Add a new item", null, null, "a1",
                    SharedResources.getIcon("addItem.png"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            addItem(ae);
                        }
                    });
        }
        // remove selected Item
        if (canRemoveItem()) {
            removeAction = repos.addAction("removeItem", "Remove", "Remove selected item", null, null, "a2",
                    SharedResources.getIcon("removeItem.gif"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            removeItem();
                        }
                    });
        }
        // move selected Item up
        if (canSwitchItems()) {
            moveUpAction = repos.addAction("moveUp", "Move up", "Move selected item up", null, null, "a3",
                    SharedResources.getIcon("moveUp.png"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            moveItem(-1);
                        }
                    });
            // move selected Item down
            moveDownAction = repos.addAction("moveDown", "Move down", "Move selected item down", null, null, "a4",
                    SharedResources.getIcon("moveDown.png"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            moveItem(1);
                        }
                    });
        }
    }


    protected void addItem(ActionEvent ae) {
        if (factory == null) {
            int selectedIndex = getSelectedIndex();
            if ((selectedIndex < 0) || (selectedIndex > listModel.getSize())) {
                selectedIndex = getListModel().getSize();
            }
            listModel.addItem(selectedIndex);
        } else {
            Class<?> modelClass = listModel.getIndexedAccess().getModelClass();
            List<String> availableTypes = factory.getAvailableObjectTypes(modelClass);
            int nbrTypes = (availableTypes == null) ? 0 : availableTypes.size();
            if (nbrTypes == 0) {
                log.error("No defined types to add in factory: " + factory);
            } else if (nbrTypes == 1) {
                addItem(availableTypes.get(0));
            } else {
                if (ae.getSource() instanceof JComponent) {
                    JComponent invoker = (JComponent) ae.getSource();
                    JPopupMenu popup = new JPopupMenu();
                    ActionListener lis = new ActionListener() {

                        public void actionPerformed(ActionEvent e) {
                            addItem(e.getActionCommand());
                        }

                    };
                    for (int i = 0; i < nbrTypes; i++) {
                        String key = availableTypes.get(i);
                        String value = LanguageSupport.getLocalizedString(key);
                        JMenuItem item = new JMenuItem(value);
                        item.setActionCommand(key);
                        item.addActionListener(lis);
                        popup.add(item);
                    }
                    popup.show(invoker, 0, invoker.getHeight());
                } else {
                    log.error("Source of add event is not a JComponent but: " + ae.getSource());
                }
            }
        }
    }


    protected void addItem(String itemType) {
        Object newItem = factory.newIntance(itemType);
        addObjectItem(newItem);
    }


    protected void addObjectItem(Object newItem) {
        int newIndex = getSelectedIndex();
        if (newIndex < 0) newIndex = listModel.getSize();
        listModel.addItem(newIndex, newItem);
        listCmp.setSelectedIndex(newIndex);
    }


    protected void moveItem(int movment) {
        if (canSwitchItems()) {
            int selectedIndex = getSelectedIndex();
            int targetIndex = selectedIndex + movment;
            int rowCount = listModel.getSize();
            if ((selectedIndex >= 0) && (targetIndex >= 0) && (selectedIndex < rowCount) && (targetIndex < rowCount)) {
                listModel.switchItems(selectedIndex, targetIndex);
                listCmp.setSelectedIndex(targetIndex);
            }
        }
    }


    protected void removeItem() {
        int selectedIndex = getSelectedIndex();
        if ((selectedIndex >= 0) && (selectedIndex < listModel.getSize())) {
            listModel.removeItem(selectedIndex);
        }
    }


    @Override
    protected Object getViewValue() {
        return null;
    }


    @Override
    protected boolean isViewReady() {
        return listCmp != null;
    }


    @Override
    protected void setViewValue(Object newViewValue) {
    }


    // --------------------- ObjectSelector interface implementation -------------------------------------------

    public String getId() {
        return getClass().getSimpleName() + "@" + getModelAccess().getFieldPath();
    }


    public void clearSelection() {
        if (listCmp != null) listCmp.clearSelection();
    }


    public Object getSelectedObject() {
        return (selectedObjects.size() == 0) ? null : selectedObjects.get(0);
    }


    public int getSelectedObjectCount() {
        return selectedObjects.size();
    }


    public List<Object> getSelectedObjects() {
        return new ArrayList<Object>(selectedObjects);
    }


    public boolean isMultiSelectionEnabled() {
        return true;
    }


    public void selectDefaultItem() {
        if ((listCmp != null) && (listModel.getSize() > 0)) {
            listCmp.setSelectedIndex(0);
        }
    }


    public boolean canSelectObjects(List<?> toSelect) {
        Object newSelection = null;
        if ((toSelect != null) && (toSelect.size() == 1)) {
            newSelection = toSelect.get(0);
        }
        return (newSelection == null) || (listModel.indexOf(newSelection) >= 0);
    }


    public void selectObjects(List<?> toSelect) {
        Object newSelection = null;
        if ((toSelect != null) && (toSelect.size() == 1)) {
            newSelection = toSelect.get(0);
        }
        if (listCmp != null) {
            if (newSelection == null) listCmp.clearSelection();
            else listCmp.setSelectedIndex(listModel.indexOf(newSelection));
        }
    }


    public void addObjectSelectionListener(ObjectSelectionListener lis) {
        if (listeners == null) listeners = new ArrayList<ObjectSelectionListener>();
        listeners.add(lis);
    }


    public void removeObjectSelectionListener(ObjectSelectionListener lis) {
        if (listeners != null) listeners.remove(lis);
    }


    protected void fireObjectSelectionChanged() {
        if (listeners != null) {
            for (ObjectSelectionListener lis : listeners) {
                lis.objectSelectionChanged(this);
            }
        }
    }

}
