/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import javax.swing.AbstractListModel;

import net.sf.sfac.gui.editor.ModelAccess;
import net.sf.sfac.gui.editor.access.IndexedModelAccess;
import net.sf.sfac.utils.Comparison;


/**
 * Class used by the ListObjectEditor implement its ListModel. <br>
 * This ListModel delegates all collection access to a IndexedModelAccess (so that model objects can see values
 * accessed/modified).
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class ListObjectModel extends AbstractListModel {


    private BaseObjectEditor editor;
    private IndexedModelAccess indexedAccess;


    public ListObjectModel() {
    }


    public ListObjectModel(BaseObjectEditor newEditor) {
        setEditor(newEditor);
    }


    public BaseObjectEditor getEditor() {
        return editor;
    }


    public void setEditor(BaseObjectEditor newEditor) {
        if (editor != null) removeListDataListener(editor.getEditorModificationSupport().getListenerForListModel());
        editor = newEditor;
        indexedAccess = null;
        if (editor != null) addListDataListener(editor.getEditorModificationSupport().getListenerForListModel());
    }


    protected IndexedModelAccess getIndexedAccess() {
        if (indexedAccess == null) {
            ModelAccess access = editor.getModelAccess();
            if (access instanceof IndexedModelAccess) {
                indexedAccess = (IndexedModelAccess) access;
            } else {
                indexedAccess = new IndexedModelAccess(access);
            }
        }
        return indexedAccess;
    }


    public Class<?> getItemClass() {
        return getIndexedAccess().getModelClass();
    }


    public int getSize() {
        return getIndexedAccess().getItemCount(editor.getEditedObject());
    }


    public Object getElementAt(int index) {
        return getIndexedAccess().getItem(editor.getEditedObject(), index);
    }


    public boolean canAddItem() {
        return getIndexedAccess().canAddItem();
    }


    public void addItem(int index) {
        addItem(index, null);
    }


    public void addItem(int index, Object objectToAdd) {
        getIndexedAccess().addItem(editor.getEditedObject(), index, objectToAdd);
        fireIntervalAdded(this, index, index);
    }


    public boolean canRemoveItem() {
        return getIndexedAccess().canRemoveItem();
    }


    public void removeItem(int index) {
        getIndexedAccess().removeItem(editor.getEditedObject(), index);
        fireIntervalRemoved(this, index, index);
    }


    public boolean canSwitchItems() {
        return getIndexedAccess().canSwitchItem();
    }


    public void switchItems(int index1, int index2) {
        getIndexedAccess().switchItems(editor.getEditedObject(), index1, index2);
        fireContentsChanged(this, index1, index2);
    }


    public void fireDataChanged() {
        fireContentsChanged(this, 0, getSize());
    }


    public int indexOf(Object obj) {
        int len = getSize();
        for (int i = 0; i < len; i++) {
            if (Comparison.equals(obj, getElementAt(i))) return i;
        }
        return -1;
    }
}
