/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.CardLayout;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sf.sfac.gui.editor.EditorContext;
import net.sf.sfac.gui.editor.EditorFactory;
import net.sf.sfac.gui.editor.ObjectEditor;
import net.sf.sfac.gui.editor.access.BasicModelAccess;
import net.sf.sfac.gui.utils.ObjectSelectionListener;
import net.sf.sfac.gui.utils.ObjectSelector;
import net.sf.sfac.lang.LanguageListener;
import net.sf.sfac.lang.LanguageSupport;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Panel displaying several editors in a card panel. <br>
 * You can ask to edit any object in this panel, it will always instantiate an editor to show/edit this object. The editors are
 * created using an EditorFactory (externally provided) and are displayed using a card panel.
 */
public class MultiObjectEditor extends BaseObjectEditor implements EditorContext, LanguageListener, ObjectSelector {

    private static Log log = LogFactory.getLog(MultiObjectEditor.class);
    /** the size of this editor history buffer. */
    private static final int HISTORY_SIZE = 20;

    private Map<Object, Object> userObjects;

    private JPanel editorPane;
    private CardLayout editorPaneLayout;
    private Object currentEdited;
    private ObjectEditor currentEditor;
    private boolean editable = true;
    private boolean enabled = true;
    /** index of the currently edited object in the edition history list. */
    private int currentObjectIndex = -1;
    /** the edition history list. */
    private List<Object> lastEditedObjects;
    /** Listeners of the currently edited object. */
    private List<ObjectSelectionListener> listeners;

    private Map<String, ObjectEditor> editors;
    private JLabel defaultDisplay;


    public MultiObjectEditor() {
        editors = new HashMap<String, ObjectEditor>();
        lastEditedObjects = new ArrayList<Object>();
        // set a 'fake' model access, it's the modelAccess of the currentEditor that will be used.
        setModelAccess(new BasicModelAccess(Object.class));
    }


    protected String getNoSelectionMessage() {
        return LanguageSupport.getOptionalLocalizedString("SELECT_A_NODE", "Select a node.");
    }


    @Override
    public void edit(Object objToEdit) {
        if (currentEdited != objToEdit) {
            if (log.isDebugEnabled()) log.debug("Change edited object from " + currentEdited + " to " + objToEdit);
            // get the new editor
            ObjectEditor newEditor = null;
            String editedObjectClassName = null;
            if (objToEdit != null) {
                editedObjectClassName = objToEdit.getClass().getName();
                newEditor = editors.get(editedObjectClassName);
                if (newEditor == null) {
                    newEditor = getEditorFactory().createEditor(objToEdit.getClass());
                    newEditor.setEditorContext(this);
                    newEditor.addEditorModificationListener(getEditorModificationSupport().getListenerForSubEditors());
                    editors.put(editedObjectClassName, newEditor);
                    getEditorComponent().add(newEditor.getEditorComponent(), editedObjectClassName);
                }
            }

            // close old editor
            if (currentEditor != null) {
                currentEditor.updateModel();
                // detach the current editor from its edited object
                if (currentEditor != newEditor) currentEditor.edit(null);
            }

            // setup new editor
            currentEditor = newEditor;
            currentEdited = objToEdit;
            adaptHistoryList(objToEdit);
            if (newEditor == null) {
                log.info("No editor pane found to edit node " + objToEdit);
                editorPaneLayout.show(editorPane, "noNode");
            } else {
                log.info("Display editor pane " + currentEditor + " to edit node " + objToEdit);
                newEditor.edit(objToEdit);
                editorPaneLayout.show(editorPane, editedObjectClassName);
            }

            synchronizeEditableState();
            fireObjectSelectionChanged();
            getEditorComponent().invalidate();
            getEditorComponent().revalidate();
        }
    }


    @Override
    public Object getEditedObject() {
        return currentEdited;
    }


    private void adaptHistoryList(Object edited) {
        int len = lastEditedObjects.size();
        currentObjectIndex++;
        if (currentObjectIndex < len) {
            if (edited != lastEditedObjects.get(currentObjectIndex)) {
                for (int i = len - 1; i > currentObjectIndex; i--) {
                    lastEditedObjects.remove(i);
                }
                lastEditedObjects.set(currentObjectIndex, edited);
            }
        } else {
            lastEditedObjects.add(edited);
        }
        if (lastEditedObjects.size() > HISTORY_SIZE) {
            lastEditedObjects.remove(0);
            currentObjectIndex--;
        }
    }


    public void moveInHistoryList(boolean forward) {
        int len = lastEditedObjects.size();
        if (forward) {
            if (currentObjectIndex < (len - 1)) {
                Object newEdited = lastEditedObjects.get(currentObjectIndex + 1);
                edit(newEdited);
            }
        } else {
            if (currentObjectIndex >= 1) {
                currentObjectIndex -= 2;
                Object newEdited = lastEditedObjects.get(currentObjectIndex + 1);
                edit(newEdited);
            }
        }
    }


    public boolean canMoveInHistory(boolean forward) {
        if (forward) {
            int len = lastEditedObjects.size();
            return currentObjectIndex < (len - 1);
        } else {
            return currentObjectIndex >= 1;
        }
    }


    public void clearHistory() {
        currentObjectIndex = -1;
        lastEditedObjects.clear();
    }


    // -------------------- EditorContext interface implementation -----------------------------------------

    public EditorFactory getEditorFactory() {
        EditorContext editorContext = getEditorContext();
        EditorFactory factory = (editorContext == null) ? null : editorContext.getEditorFactory();
        return (factory == null) ? EditorFactory.getInstance() : factory;
    }


    public Object getContextObject(String objectKey) {
        Object obj = (userObjects == null) ? null : userObjects.get(objectKey);
        if (obj == null) {
            EditorContext context = getEditorContext();
            if (context != null) obj = context.getContextObject(objectKey);
        }
        return obj;
    }


    public void putContextObject(String objectKey, Object obj) {
        if (userObjects == null) userObjects = new HashMap<Object, Object>();
        userObjects.put(objectKey, obj);
    }


    public boolean canEditInContext(boolean create, Class<?> objectClass, ObjectEditor source) {
        if (objectClass == null) return false;
        // avoid to open an editor for simple java objects like String or Date.
        if (objectClass.getName().startsWith("java.")) return false;
        return !create;
    }


    public <T> T editInContext(T obj, Class<T> objectClass, ObjectEditor source) {
        edit(obj);
        return null;
    }


    public Collection<ObjectEditor> getEditors() {
        return editors.values();
    }


    // -------------------- LanguageListener interface implementation -----------------------------------------

    public void languageChanged(Locale newLocale) {
        defaultDisplay.setText(getNoSelectionMessage());
        for (ObjectEditor editor : getEditors()) {
            if (editor instanceof LanguageListener) {
                ((LanguageListener) editor).languageChanged(newLocale);
            }
        }
    }


    // -------------------- ObjectEditor interface implementation -----------------------------------------

    public JComponent getEditorComponent() {
        if (editorPane == null) {
            editorPaneLayout = new CardLayout();
            editorPane = new JPanel(editorPaneLayout);
            // set a fixed preferred size to have correct behavior in the split pane.
            // --> cause problem if the panel has to provide it's size (in a MultiBorderLayout for example)
            // editorPane.setPreferredSize(new Dimension(50, 50));
            defaultDisplay = new JLabel(getNoSelectionMessage());
            defaultDisplay.setHorizontalAlignment(JLabel.CENTER);
            editorPane.add(defaultDisplay, "noNode");
        }
        return editorPane;
    }


    public void updateView() {
        if (currentEditor != null) currentEditor.updateView();
    }


    public void updateModel() {
        if (isEditable() && isEnabled()) {
            if (currentEditor != null) currentEditor.updateModel();
        }
    }


    @Override
    protected void synchronizeEditableState() {
        boolean newEditable = isEditable();
        boolean newEnabled = isEnabled();
        if ((editable != newEditable) || (enabled != newEnabled)) {
            editable = newEditable;
            enabled = newEnabled;
            getEditorModificationSupport().fireEditorEditabilityModified();
        }
    }


    /**
     * Override to disable creation of a label component.
     */
    @Override
    public JLabel getLabelComponent() {
        return null;
    }


    /**
     * As there is no label in this editor, the editorComponent is the same as the objectComponent.
     */
    public JComponent getObjectComponent() {
        return getEditorComponent();
    }


    // ------------------- ObjectSelector interface implementation ----------------------------------------

    public String getId() {
        return getClass().getSimpleName() + "@" + getModelAccess().getFieldPath();
    }


    public boolean canSelectObjects(List<?> toSelect) {
        if (toSelect == null) return true;
        return toSelect.size() <= 1;
    }


    public void clearSelection() {
        edit(null);
    }


    public Object getSelectedObject() {
        return currentEdited;
    }


    public int getSelectedObjectCount() {
        return (currentEdited == null) ? 0 : 1;
    }


    public List<?> getSelectedObjects() {
        if (currentEdited == null) return null;
        return Collections.singletonList(currentEdited);
    }


    public boolean isMultiSelectionEnabled() {
        return false;
    }


    public void selectDefaultItem() {
        edit(null);
    }


    public void selectObjects(List<?> toSelect) {
        if ((toSelect == null) || (toSelect.size() == 0)) edit(null);
        edit(toSelect.get(0));
    }


    public void addObjectSelectionListener(ObjectSelectionListener lis) {
        if (listeners == null) listeners = new ArrayList<ObjectSelectionListener>();
        listeners.add(lis);
    }


    public void removeObjectSelectionListener(ObjectSelectionListener lis) {
        if (listeners != null) listeners.remove(lis);
    }


    public void fireObjectSelectionChanged() {
        if (listeners != null) {
            for (ObjectSelectionListener lis : listeners) {
                lis.objectSelectionChanged(this);
            }
        }
    }
}
