/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.GridBagConstraints;
import java.lang.reflect.Constructor;

import javax.swing.JComponent;
import javax.swing.JTextField;

import net.sf.sfac.gui.editor.ModelAccess;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Editor used to edit a Number value (primitive or not). <br>
 * 
 * @author Olivier Berlanger
 */
public class NumberEditor extends SimpleObjectEditor {

    private static Log log = LogFactory.getLog(NumberEditor.class);

    /** Can be different from the getModelClass() result if model class is a primitive class. */
    private Class<?> objectModelClass;
    private Constructor<?> stringBasedConstructor;
    /** Tells if we edit a primitive value, if so, the 'null' cannot be returned as view value. */
    private boolean isPrimitive;

    private JTextField component;


    public NumberEditor() {
        setProperty("fill", Integer.valueOf(GridBagConstraints.NONE));
    }


    @Override
    public void setModelAccess(ModelAccess newAccess) {
        super.setModelAccess(newAccess);
        setupObjectModelClass();
    }


    private void setupObjectModelClass() {
        Class<?> accessModelClass = getModelClass();
        isPrimitive = accessModelClass.isPrimitive();
        if (isPrimitive) {
            if (accessModelClass == Integer.TYPE) objectModelClass = Integer.class;
            else if (accessModelClass == Long.TYPE) objectModelClass = Long.class;
            else if (accessModelClass == Double.TYPE) objectModelClass = Double.class;
            else if (accessModelClass == Float.TYPE) objectModelClass = Float.class;
            else if (accessModelClass == Byte.TYPE) objectModelClass = Byte.class;
            else if (accessModelClass == Short.TYPE) objectModelClass = Short.class;
            else if (accessModelClass == Character.TYPE) objectModelClass = Character.class;
            else throw new IllegalArgumentException("Unssuported primitive type: " + accessModelClass);
        } else {
            objectModelClass = accessModelClass;
        }
        try {
            stringBasedConstructor = objectModelClass.getConstructor(String.class);
        } catch (Exception e) {
            throw new IllegalStateException("No string-based constructor found for " + objectModelClass);
        }
    }


    public JComponent getObjectComponent() {
        if (component == null) {
            component = new JTextField(8);
            component.setMinimumSize(component.getPreferredSize());
            registerFocusListener(component);
            synchronizeEditableState();
            updateView();
        }
        return component;
    }


    @Override
    protected void synchronizeEditableState() {
        if (component != null) {
            component.setEditable(isEditable());
            component.setEnabled(isEnabled());
        }
    }


    @Override
    protected boolean isViewReady() {
        return (component != null);
    }


    @Override
    protected Object getViewValue() {
        Number viewValue = null;
        String text = component.getText();
        if (text != null) {
            text = text.trim();
            if (!text.equals("")) {
                viewValue = createNumber(text);
            }
        }
        if ((viewValue == null) && isPrimitive) viewValue = createNumber("0");
        return viewValue;
    }


    private Number createNumber(String text) {
        Number num;
        try {
            num = (Number) stringBasedConstructor.newInstance(text);
        } catch (Exception e) {
            if (log.isDebugEnabled()) log.debug("Bad view value '" + text + "' for " + objectModelClass + ": exception = " + e);
            try {
                num = isPrimitive ? (Number) stringBasedConstructor.newInstance("0") : null;
                component.setText((isPrimitive) ? "0" : "");
            } catch (Exception ee) {
                throw new IllegalStateException("Cannot construct Number form string '" + text + "'", e);
            }
        }
        return num;
    }


    @Override
    protected void setViewValue(Object modelValue) {
        String strValue = (modelValue == null) ? "" : modelValue.toString();
        component.setText(strValue);
    }

}
