/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.Color;
import java.awt.GridBagConstraints;

import javax.swing.JComponent;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;

import net.sf.sfac.utils.Comparison;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Editor used to edit a String value. <br>
 * Depending of the component type, a different widget is used:
 * <ul>
 * <li>line: (default) display a standard JTextField.
 * <li>text: display a JTextArea (inside a JScrollPane).
 * <li>password: display a JPasswordField.
 * </ul>
 * 
 * @author Olivier Berlanger
 */
public class StringEditor extends SimpleObjectEditor {

    public enum ComponentType {
        LINE, TEXT, PASSWORD
    }

    private static Log log = LogFactory.getLog(StringEditor.class);

    /** The component editing the string: a JTextField, JTextArea or JPasswordField. */
    private JTextComponent textField;
    /** The main editing component: the JTextField or a JScrollPane containing the JTextArea. */
    private JComponent component;
    /** The type of swing component displayed. */
    private ComponentType type;


    public StringEditor() {
        this(ComponentType.LINE);
    }


    public StringEditor(ComponentType newType) {
        setComponentType(newType);
    }


    public void setComponentType(ComponentType newType) {
        if (newType == null) throw new IllegalArgumentException("The type cannot be null");
        type = newType;
        boolean multilineText = (type == ComponentType.TEXT);
        setProperty("weightx", Double.valueOf(multilineText ? 0.2 : 0.0));
        // setProperty("weighty", Double.valueOf(multilineText ? 1.0 : 0.0));
        setProperty("fill", Integer.valueOf(multilineText ? GridBagConstraints.BOTH : GridBagConstraints.HORIZONTAL));
    }


    public ComponentType getComponentType() {
        return type;
    }


    public boolean isMultiline() {
        return type == ComponentType.TEXT;
    }


    public JComponent getObjectComponent() {
        if (component == null) {
            switch (type) {
                case TEXT:
                    JTextArea area = new JTextArea(5, 15);
                    area.setLineWrap(true);
                    area.setWrapStyleWord(true);
                    textField = area;
                    component = new JScrollPane(textField);
                    break;
                case LINE:
                    textField = new JTextField(15);
                    component = textField;
                    break;
                case PASSWORD:
                    textField = new JPasswordField();
                    component = textField;
            }
            component.setMinimumSize(component.getPreferredSize());
            registerFocusListener(textField);
            synchronizeEditableState();
            updateView();
            textField.getDocument().addDocumentListener(new DocumentListener() {

                public void changedUpdate(DocumentEvent newE) {
                    checkModified();
                }


                public void insertUpdate(DocumentEvent newE) {
                    checkModified();
                }


                public void removeUpdate(DocumentEvent newE) {
                    checkModified();
                }

            });
        }
        return component;
    }


    void checkModified() {
        if (getEditorModificationSupport().getEnableEvents() && isEditable()) {
            updateModel();
        }
    }


    @Override
    protected void synchronizeEditableState() {
        if (textField != null) {
            textField.setEditable(isEditable());
            textField.setEnabled(isEnabled());
            if (isMultiline()) {
                textField.setBackground(isEditable() ? Color.WHITE : Color.LIGHT_GRAY);
            }
        }
    }


    @Override
    protected boolean isViewReady() {
        return (textField != null);
    }


    @Override
    protected Object getViewValue() {
        String text = textField.getText();
        if (text != null) text = text.trim();
        return text;
    }


    @Override
    protected void setViewValue(Object modelValue) {
        final String modelText = (modelValue == null) ? null : modelValue.toString();
        String viewText = (String) getViewValue();
        if (!Comparison.equals(modelText, viewText)) {
            try {
                textField.setText(modelText);
                if (isMultiline()) {
                    // force the JScrollPane to scroll up and show first line of text area.
                    JTextArea area = (JTextArea) textField;
                    area.setCaretPosition(0);
                }
            } catch (IllegalStateException ise) {
                log.warn("Unable to update textFiled '" + getModelAccess().getFieldPath() + "' with value " + viewText + " --> "
                        + modelText, ise);
                SwingUtilities.invokeLater(new Runnable() {

                    public void run() {
                        setViewValue(modelText);
                    }
                });
            }
        }
    }


    @Override
    public void setProperty(String name, Object value) {
        if (name.equals("type")) {
            if (value == null) {
                setComponentType(ComponentType.LINE);
            } else if (value instanceof ComponentType) {
                setComponentType((ComponentType) value);
            } else {
                String enumName = value.toString().toUpperCase();
                ComponentType newType = ComponentType.valueOf(enumName);
                setComponentType(newType);
            }
        } else {
            super.setProperty(name, value);
        }
    }


    @Override
    public Object getProperty(String name, Object defaultValue) {
        if (name.equals("type")) {
            return type.name().toLowerCase();
        } else {
            return super.getProperty(name, defaultValue);
        }
    }


    @Override
    protected boolean valueHasChanged(Object oldValue, Object newValue) {
        return !Comparison.stringEquals(oldValue, newValue);
    }

}
