/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import javax.swing.table.AbstractTableModel;

import net.sf.sfac.gui.editor.ModelAccess;
import net.sf.sfac.gui.editor.access.IndexedModelAccess;
import net.sf.sfac.utils.Comparison;


/**
 * Class used by the TableObjectEditor implement its TableModel. <br>
 * This TableModel delegates all collection access to a IndexedModelAccess (so that model objects can see values accessed/modified).
 * By default, it only defines one column that displays the object converted to a String. If you want to have more columns, you will
 * have to subclass this TableObjectModel.
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class TableObjectModel extends AbstractTableModel {

    /** The editor owning this TableObjectModel. */
    private BaseObjectEditor editor;
    /** The IndexedModelAccess used to access the data inside edited object. */
    private IndexedModelAccess indexedAccess;
    /**
     * This boolean indicates if the model is sorted. If it is, the index of added/removed objects can be changed by the underlying
     * model. As the IndexedModelAccess doesn't support listeners, it means then that a all-rows-updated event must be fired in any
     * case. As 'sorted=true' is incompatible with switching, if this flag is null, we take !canSwitchItems() as default value.
     */
    private Boolean sorted;
    private boolean editable;


    public TableObjectModel() {
    }


    public TableObjectModel(boolean isModelSorted) {
        sorted = Boolean.valueOf(isModelSorted);
    }


    public TableObjectModel(BaseObjectEditor newEditor) {
        setEditor(newEditor);
    }


    @Override
    public String toString() {
        return getClass().getName() + "[size=" + getRowCount() + ", item type=" + getItemClass().getSimpleName() + "]";
    }


    public Class<?> getItemClass() {
        return getIndexedAccess().getModelClass();
    }


    public void setEditor(BaseObjectEditor newEditor) {
        if (editor != null) removeTableModelListener(editor.getEditorModificationSupport().getListenerForTableModel());
        editor = newEditor;
        indexedAccess = null;
        if (editor != null) addTableModelListener(editor.getEditorModificationSupport().getListenerForTableModel());
    }


    private boolean isSorted() {
        if (sorted != null) return sorted.booleanValue();
        return !canSwitchItems();
    }


    protected IndexedModelAccess getIndexedAccess() {
        if (indexedAccess == null) {
            ModelAccess access = editor.getModelAccess();
            if (access instanceof IndexedModelAccess) {
                indexedAccess = (IndexedModelAccess) access;
            } else {
                indexedAccess = new IndexedModelAccess(access);
            }
        }
        return indexedAccess;
    }


    public Object getObjectAt(int index) {
        return getIndexedAccess().getItem(editor.getEditedObject(), index);
    }


    public boolean canSetItem() {
        return getIndexedAccess().canSetItem();
    }


    public void setObjectAt(int index, Object newObject) {
        Object editedObject = editor.getEditedObject();
        Object currentValue = getIndexedAccess().getItem(editedObject, index);
        if (newObject != currentValue) {
            getIndexedAccess().setItem(editedObject, index, newObject);
            if (isSorted()) fireTableDataChanged();
            else fireTableRowsUpdated(index, index);
        }
    }


    public int getColumnCount() {
        return 1;
    }


    @Override
    public String getColumnName(int column) {
        return editor.getTranslatedLabel();
    }


    public int getRowCount() {
        return getIndexedAccess().getItemCount(editor.getEditedObject());
    }


    public Object getValueAt(int rowIndex, int columnIndex) {
        return getObjectAt(rowIndex);
    }


    @Override
    public void setValueAt(Object newValue, int rowIndex, int columnIndex) {
        setObjectAt(rowIndex, newValue);
    }


    public boolean canAddItem() {
        return getIndexedAccess().canAddItem();
    }


    public void addItem(int index) {
        getIndexedAccess().addItem(editor.getEditedObject(), index, null);
        if (isSorted()) fireTableDataChanged();
        else fireTableRowsInserted(index, index);
    }


    public boolean canRemoveItem() {
        return getIndexedAccess().canRemoveItem();
    }


    public void removeItem(int index) {
        getIndexedAccess().removeItem(editor.getEditedObject(), index);
        fireTableRowsDeleted(index, index);
    }


    public boolean canSwitchItems() {
        return getIndexedAccess().canSwitchItem();
    }


    public void switchItems(int index1, int index2) {
        getIndexedAccess().switchItems(editor.getEditedObject(), index1, index2);
        fireTableRowsUpdated(index1, index2);
    }


    public int indexOf(Object obj) {
        int len = getRowCount();
        for (int i = 0; i < len; i++) {
            if (Comparison.equals(obj, getObjectAt(i))) return i;
        }
        return -1;
    }


    public void setEditable(boolean newEditable) {
        editable = newEditable;
    }


    @Override
    public boolean isCellEditable(int newRowIndex, int newColumnIndex) {
        return editable && canSetItem();
    }

}
