/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.framework;


import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URI;
import java.net.URL;
import java.util.Arrays;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import net.sf.sfac.gui.ExceptionDialog;
import net.sf.sfac.lang.LanguageSupport;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Helper class to manage an 'About' dialog. <br>
 * This class defines the 'about' action and about dialog. When language support is used (so when setDirectText method is not
 * called), the following keys should be defined in bundles:
 * <ul>
 * <li>about_title : the title of the dialog
 * <li>about_text : the HTML text of the dialog
 * <li>about_license : the HTML license info of the dialog
 * <li>about_text_title : a title for the 'info' tab (containing the about_text)
 * <li>about_license_title : a title for the 'license' tab (containing the about_license)
 * </ul>
 * 
 * @author Olivier Berlanger
 */
public class AboutlHelper {

    private static Log log = LogFactory.getLog(AboutlHelper.class);

    /** A parent component for the about dialog. */
    private Component parentCmp;
    /** Direct string to use for title when no languageSupport is used. */
    private String title;
    private String text;
    private String license;
    private Icon aboutImage;


    /**
     * Create a new Look-and-feel helper.
     * 
     * @param sett
     *            The settings used to store the selected look-and-feel (null if nothing has to be stored)
     */
    public AboutlHelper(Component parent) {
        parentCmp = parent;
    }


    public void setAboutIcon(Icon aboutIcon) {
        aboutImage = aboutIcon;
    }


    /**
     * Set direct non-localizable strings to use. Calling this method will disable the use of language support.
     * 
     * @param aboutTitle
     *            title of the about dialog
     * @param aboutText
     *            text of the about dialog (expected to be HTML text)
     * @param aboutLicense
     *            license of the about dialog (expected to be HTML text)
     */
    public void setDirectText(String aboutTitle, String aboutText, String aboutLicense) {
        title = aboutTitle;
        text = aboutText;
        license = aboutLicense;
    }


    private boolean usesLanguageSupport() {
        return title == null;
    }


    private String getLocalizedString(String key, String defaultText) {
        if (usesLanguageSupport()) return LanguageSupport.getLocalizedString(key);
        return defaultText;
    }


    /**
     * Create the 'about' action.
     */
    @SuppressWarnings("serial")
    public void createAction(ActionRepository repo) {
        repo.addAction("about", "Help", "z/a2", null, SharedResources.getIcon("about.gif"), new AbstractAction() {

            public void actionPerformed(ActionEvent ae) {
                showAboutDialog();
            }
        });
    }


    public void showAboutDialog() {
        try {
            final JDialog dial = new JDialog(JOptionPane.getFrameForComponent(parentCmp));
            dial.setTitle(getLocalizedString("about_title", title));
            if (aboutImage != null) {
                JLabel aboutImageLabel = new JLabel(aboutImage);
                aboutImageLabel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
                dial.getContentPane().add(aboutImageLabel, BorderLayout.WEST);
            }
            JLabel aboutText = new JLabel(getLocalizedString("about_text", text));
            aboutText.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
            aboutText.setHorizontalAlignment(JLabel.CENTER);
            JEditorPane aboutLicense = new JEditorPane("text/html", getLocalizedString("about_license", license));
            aboutLicense.setEditable(false);
            aboutLicense.setBackground(aboutText.getBackground());
            aboutLicense.addHyperlinkListener(new HyperlinkListener() {

                public void hyperlinkUpdate(HyperlinkEvent e) {
                    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                        openUrl(e.getURL());
                    }
                }
            });
            aboutLicense.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
            JTabbedPane tabs = new JTabbedPane();
            tabs.setBorder(BorderFactory.createEmptyBorder(10, 0, 10, 10));
            tabs.addTab(getLocalizedString("about_text_title", "Info"), aboutText);
            tabs.addTab(getLocalizedString("about_license_title", "License"), aboutLicense);
            JPanel rightPanel = new JPanel(new BorderLayout());
            rightPanel.add(tabs, BorderLayout.CENTER);
            JPanel buttonPanel = new JPanel();
            JButton closeButton = new JButton(getLocalizedString("CLOSE", "Close"));
            closeButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    dial.dispose();
                }
            });
            buttonPanel.add(closeButton);
            rightPanel.add(buttonPanel, BorderLayout.SOUTH);
            dial.getContentPane().add(rightPanel, BorderLayout.CENTER);
            dial.pack();
            dial.setLocationRelativeTo(parentCmp);
            dial.setModal(true);
            dial.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
            dial.setVisible(true);
        } catch (Exception e) {
            log.error("Unable to display about dialog", e);
            ExceptionDialog.showExceptionDialog(parentCmp, "UNEXPECTED_EXCEPTION", "UNABLE_TO_SHOW_ABOUT_DIALOG", e);
        }
    }


    void openUrl(URL url) {
        try {
            URI uri = url.toURI();
            if (Desktop.isDesktopSupported()) {
                Desktop.getDesktop().browse(uri);
            } else {
                String osName = System.getProperty("os.name");
                if (osName.regionMatches(true, 0, "linux", 0, 5)) {
                    executeLinuxCommand(uri);
                } else {
                    executeWindowsCommand(uri);
                }
            }
        } catch (Exception e) {
            ExceptionDialog.showExceptionDialog(parentCmp, "DESKTOP_ERROR", "CANNOT_OPEN_URL",
                    new Object[] { url.toExternalForm() }, e);
        }
    }


    private static void executeWindowsCommand(URI uri) throws Exception {
        String uriString = uri.toASCIIString();
        String[] cmd = new String[] { "cmd", "/C", uriString };
        if (log.isDebugEnabled()) log.debug("Launch with windows command: " + Arrays.toString(cmd));
        Runtime.getRuntime().exec(cmd);
    }


    private static void executeLinuxCommand(URI uri) throws Exception {
        String uriString = uri.toString();
        String[] cmd = new String[] { "xdg-open", uriString };
        if (log.isDebugEnabled()) log.debug("Launch with linux command: " + Arrays.toString(cmd));
        Runtime.getRuntime().exec(cmd);
    }

}
