/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.framework;


import java.awt.event.ActionEvent;
import java.util.Locale;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.ImageIcon;

import net.sf.sfac.lang.LanguageListener;
import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.setting.Settings;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Multilingual support helper, this helper will manage actions and settings associated to language switching in the application
 * (switch language menu, toolbar, restore language used in previous sessions).
 * 
 * @author Olivier Berlanger
 */
public class LanguageHelper {

    /** Key used to store the current application language in settings. */
    public static final String LANGUAGE_KEY = "app.language";

    private static Log log = LogFactory.getLog(LanguageHelper.class);

    /** List of all the supported locale ids (locales for which a bundle is defined in LanguageSupport). */
    private String[] supportedLocaleIds;
    /** Action used to switch languages. */
    private Action[] languageActions;
    /** The setting where the current language choice can be stored (can be null). */
    private Settings setts;


    /**
     * Construct a new language helper with the given supported language IDs.
     * 
     * @param manager
     *            The language manager (holding language bundles)
     * @param sett
     *            The setting where the current language choice can be stored (null if state does not have to be stored).
     * @param supportedLocaleIDs
     *            List of all the supported locale ids (something like {"en","fr"} ).
     */
    public LanguageHelper(Settings sett, String[] supportedLocaleIDs) {
        LanguageSupport.addLanguageListener(new LanguageListener() {

            public void languageChanged(Locale newLocale) {
                languageChangedToLocale(newLocale);
            }
        });
        setts = sett;
        supportedLocaleIds = supportedLocaleIDs;
        if (setts != null) {
            String localeId = setts.getStringProperty(LANGUAGE_KEY, null);
            if (localeId != null) LanguageSupport.updateLanguage(localeId);
        }
    }


    /**
     * Getter for the Settings.
     * 
     * @return the Settings used by this helper.
     */
    public Settings getSettings() {
        return setts;
    }


    /**
     * Get the language action associated to the given locale ID.
     * 
     * @param localeId
     *            The locale ID.
     * @return The language action associated to the given locale ID.
     */
    private Action getLanguageAction(String localeId) {
        int len = supportedLocaleIds.length;
        for (int i = 0; i < len; i++) {
            if (localeId.equals(supportedLocaleIds[i])) return languageActions[i];
        }
        return null;
    }


    /**
     * Notification that the locale of the LanguageSupport has changed. This method will update the language setting and the
     * selected language action accordingly.
     * 
     * @param newLocale
     *            new locale of the LanguageSupport.
     */
    void languageChangedToLocale(Locale newLocale) {
        String localeId = newLocale.toString();
        if (setts != null) setts.setStringProperty(LANGUAGE_KEY, localeId);
        Action act = getLanguageAction(localeId);
        if (act != null) {
            if (!ActionRepository.isActionSelected(act)) ActionRepository.setActionSelected(act, true);
        } else {
            log.warn("Unknown locale ID in LanguageHelper: " + localeId);
        }
    }


    /**
     * Create a new action for each supported language and add it to the ActionRepository. <br>
     * This method uses the locale ID as action id, it means that the LanguageSupport should define corresponding action name and
     * description for the action for each supported locale ID. <br>
     * If the "en" locale is supported, the following keys have to be in the ActionRepository:
     * <ul>
     * <li>"en_name" : for the action name.
     * <li>"en_descr" : for the action description (optional).
     * </ul>
     * Additionnaly, the menuId+"_menu" must be defined for menu ids defined in the menu path.
     * 
     * @param repo
     *            the repository for all application actions.
     * @param menuPath
     *            path to the menu where a menuItem should be added.
     * @param menuGroup
     *            group ids of the menu and menuItems.
     * @param toolbarGroup
     *            group id of the toolbar action.
     * @see ActionRepository#addAction(String, String, String, String, Icon, Action)
     */
    @SuppressWarnings("serial")
    public void createActions(ActionRepository repo, String menuPath, String menuGroup, String toolbarGroup) {
        String startLocaleId = LanguageSupport.getCurrentLocale().toString();
        // create language actions
        ButtonGroup butGroup = new ButtonGroup();
        int len = supportedLocaleIds.length;
        languageActions = new Action[len];
        for (int i = 0; i < len; i++) {
            languageActions[i] = repo.addAction(supportedLocaleIds[i], menuPath, menuGroup, toolbarGroup, null,
                    new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            LanguageSupport.updateLanguage((String) getValue(ActionRepository.ACTION_ID));
                        }
                    });
            ActionRepository.setRadioAction(languageActions[i], butGroup);
            ActionRepository.setActionSelected(languageActions[i], supportedLocaleIds[i].equals(startLocaleId));
        }
        // set an icon on the language menu action (if it's not a top-level menu)
        if (menuPath != null) {
            int slashIndex = menuPath.lastIndexOf('/');
            if (slashIndex >= 0) {
                Action act = repo.getAction(menuPath.substring(slashIndex + 1));
                if ((act != null) && (act.getValue(Action.SMALL_ICON) == null)) {
                    ImageIcon icon = SharedResources.getIcon("language.png");
                    act.putValue(Action.SMALL_ICON, icon);
                }
            }
        }
    }

}
