/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.framework;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.JLabel;


/**
 * Manager used to allocate dynamically mnemonics to UI components.
 * 
 * @author Olivier Berlanger
 */
public class MnemonicManager {


    private List<Set<Integer>> mnemonics = new ArrayList<Set<Integer>>();


    public MnemonicManager() {
    }


    public void reset() {
        mnemonics.clear();
    }


    public void clear(int fromLevel) {
        for (int i = mnemonics.size() - 1; i >= fromLevel; i--) {
            mnemonics.remove(i);
        }
    }


    /**
     * Set mnemonic for an action.
     * 
     * @param act
     *            the action with mnemonic to be set.
     */
    public void assignActionMnemonic(Action act) {
        assignActionMnemonic(act, 0);
    }


    /**
     * Set mnemonic for an action.
     * 
     * @param act
     *            the action with mnemonic to be set.
     * @param level
     *            level of the action in the mnemonic stack.
     */
    public void assignActionMnemonic(Action act, int level) {
        String text = (String) act.getValue(Action.NAME);
        Integer mnemonic = getMnemonic(text, level);
        act.putValue(Action.MNEMONIC_KEY, mnemonic);
    }


    /**
     * Set mnemonic for a label.
     * 
     * @param lbl
     *            the label with mnemonic to be set.
     */
    public void assignLabelMnemonic(JLabel lbl) {
        assignLabelMnemonic(lbl, 0);
    }


    /**
     * Set mnemonic for a label.
     * 
     * @param lbl
     *            the label with mnemonic to be set.
     * @param level
     *            level of the action in the mnemonic stack.
     */
    public void assignLabelMnemonic(JLabel lbl, int level) {
        String text = lbl.getText();
        Integer mnemonic = getMnemonic(text, level);
        lbl.setDisplayedMnemonic((mnemonic == null) ? 0 : mnemonic.intValue());
    }


    /**
     * Set mnemonic for a button.
     * 
     * @param but
     *            the button with mnemonic to be set.
     */
    public void assignButtonMnemonic(AbstractButton but) {
        assignButtonMnemonic(but, 0);
    }


    /**
     * Set mnemonic for a button.
     * 
     * @param but
     *            the button with mnemonic to be set.
     * @param level
     *            level of the action in the mnemonic stack.
     */
    public void assignButtonMnemonic(AbstractButton but, int level) {
        String text = but.getText();
        Integer mnemonic = getMnemonic(text, level);
        but.setMnemonic((mnemonic == null) ? 0 : mnemonic.intValue());
    }


    private void checkArray(int level) {
        int len = mnemonics.size();
        int missingCount = level - len + 1;
        for (int i = 0; i < missingCount; i++) {
            mnemonics.add(new HashSet<Integer>());
        }
    }


    /**
     * Get a non already assigned mnemonic for the given text. (and remember it in the mnemonics stack).
     * 
     * @param text
     *            Text where to find a mnemonic character
     * @param level
     *            current level in the stack.
     * @return a mnemonic for the given text (as an Integer) or null if all the available characters are already assigned.
     */
    private Integer getMnemonic(String text, int level) {
        checkArray(level);
        int len = text.length();
        int ch;
        Integer charact;
        boolean notUsed;
        for (int i = 0; i < len; i++) {
            ch = Character.toUpperCase(text.charAt(i));
            charact = Integer.valueOf(ch);
            notUsed = true;
            for (int j = 0; j <= level; j++) {
                if (mnemonics.get(j).contains(charact)) {
                    notUsed = false;
                    break;
                }
            }
            if (notUsed) {
                mnemonics.get(level).add(charact);
                return charact;
            }
        }
        return null;
    }


}
