/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.framework;


import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.ImageIcon;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Shared UI resources (mainly icons). <br>
 * Those resources are strictly for UI, they have nothing to do with the database resources which are data-dependent.
 */
public final class SharedResources {

    private static Log log = LogFactory.getLog(SharedResources.class);

    private static final String DEFAULT_BASE_IMAGE_LOCATION = "/net/sf/sfac/images";

    private static List<String> imageLocations;
    private static Map<String, Object> resources = new HashMap<String, Object>();
    private static SharedResources instance = new SharedResources();

    static {
        imageLocations = new ArrayList<String>();
        imageLocations.add(DEFAULT_BASE_IMAGE_LOCATION);
    }


    private SharedResources() {
    }


    public static void addBaseImageLocation(String newLocation) {
        if (!imageLocations.contains(newLocation)) imageLocations.add(newLocation);
    }


    public static List<String> getBaseImageLocations() {
        return imageLocations;
    }


    /**
     * Get an image icon resource from the default icon location (net.sf.sfac.images).
     */
    public static ImageIcon getIcon(String iconPath) {
        return getIcon(iconPath, null, null);
    }


    /**
     * Get an image icon resource from the default icon location (net.sf.sfac.images).
     */
    public static ImageIcon getIcon(String iconPath, Class<?> baseClass) {
        return getIcon(iconPath, null, baseClass);
    }


    /**
     * Get an image icon resource from the default icon location (net.sf.sfac.images).
     */
    public static ImageIcon getIcon(String iconPath, String defaultIconPath) {
        return getIcon(iconPath, defaultIconPath, null);
    }


    /**
     * Get an image icon resource from the default icon location (net.sf.sfac.images).
     */
    public static ImageIcon getIcon(String iconPath, String defaultIconPath, Class<?> baseClass) {
        ImageIcon ico = (ImageIcon) resources.get(iconPath);
        if (ico == null) {
            URL imageURL = getResourceURL(iconPath, baseClass);
            if (imageURL == null) {
                if (defaultIconPath == null) {
                    throw new IllegalArgumentException("Image not found in classpath: " + iconPath + " [classLoader = "
                            + getClassLoader(baseClass) + "]");
                } else {
                    return getIcon(defaultIconPath, null, baseClass);
                }
            }
            try {
                ico = new ImageIcon(imageURL);
            } catch (Exception e) {
                throw new IllegalArgumentException("Invalid image in URL: " + iconPath, e);
            }
            resources.put(iconPath, ico);
        }
        return ico;
    }


    public static URL getResourceURL(String path, Class<?> baseClass) {
        URL resourceURL = null;
        if (baseClass == null) baseClass = instance.getClass();
        if (path.startsWith("/")) {
            // absolute resource
            resourceURL = baseClass.getResource(path);
        } else {
            for (String basePath : imageLocations) {
                String fullPath = basePath + "/" + path;
                resourceURL = baseClass.getResource(fullPath);
                if (resourceURL != null) break;
            }
            if (log.isDebugEnabled() && (resourceURL == null)) {
                log.debug("Resource: " + path + " not found in " + imageLocations);
            }
        }
        return resourceURL;
    }


    public static String getResourcePath(Object resource) {
        for (Entry<String, Object> e : resources.entrySet()) {
            if (e.getValue() == resource) {
                return e.getKey();
            }
        }
        return null;
    }


    private static ClassLoader getClassLoader(Class<?> baseClass) {
        if (baseClass == null) baseClass = instance.getClass();
        return baseClass.getClassLoader();
    }


    /**
     * Check if a resource exists.
     */
    public static boolean iconExists(String iconPath) {
        return iconExists(iconPath, null);
    }


    /**
     * Check if a resource exists.
     */
    public static boolean iconExists(String iconPath, Class<?> baseClass) {
        boolean exists = true;
        Object res = resources.get(iconPath);
        if (res == null) {
            exists = getResourceURL(iconPath, baseClass) != null;
        }
        return exists;
    }

}
