/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.layout;


import java.awt.Component;
import java.awt.Insets;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * 
 * @author Olivier Berlanger
 */
public class CellInfo {

    private static Log log = LogFactory.getLog(CellInfo.class);

    public static final int COLLAPSED_NONE = 0;
    public static final int COLLAPSED_VERTICAL = 1;
    public static final int COLLAPSED_HORIZONTAL = 2;

    // constraints
    private int gridx;
    private int gridy;
    private int gridwidth;
    private int gridheight;
    private double weightx;
    private double weighty;

    /** Components in this cell along with their specific constraints. */
    private ComponentInfo[] cmpInfos;
    /** Number of components in the cmpInfos array. */
    private int nbrCmp;
    /** Number of components in the cmpInfos array. */
    private int nbrFillX;
    /** Space reserved by borders around this CellInfo. */
    private Insets borderSpace;
    /** Flag saying if this cell is expanded/collapsed by a border. */
    private int cellCollapsed;
    /** Flag saying if this cell is enabled/disabled by a border. */
    private boolean cellEnabled;

    private int width;
    private int height;


    /**
     * Construct an empty CellInfo. Reserve the space for 1 component.
     */
    public CellInfo(MultiBorderConstraints constr) {
        gridx = constr.gridx;
        gridy = constr.gridy;
        gridwidth = constr.gridwidth;
        gridheight = constr.gridheight;
        weightx = constr.weightx;
        weighty = constr.weighty;
        cmpInfos = new ComponentInfo[1];
        borderSpace = new Insets(0, 0, 0, 0);
        cellEnabled = true;
    }


    /**
     * Add a component to the components list.
     * 
     * @param cmp
     *            component to add
     * @param constr
     *            constraint of the added component
     */
    public void addComponent(Component cmp, MultiBorderConstraints constr) {
        // check the size of arrays
        int len = cmpInfos.length;
        if (nbrCmp == len) {
            // enlarge the arrays to contain more components
            int newLen = len + 3;
            ComponentInfo[] newCmpInfos = new ComponentInfo[newLen];
            System.arraycopy(cmpInfos, 0, newCmpInfos, 0, len);
            cmpInfos = newCmpInfos;
        }
        // register to the borders (if not already done)
        if (nbrCmp == 0) {
            for (Iterator<MultiBorder> it = constr.getBordersIterator(); it.hasNext();) {
                MultiBorder border = it.next();
                border.add(this);
            }
        }
        // add the component
        cmp.setVisible(cellCollapsed == 0);
        cmp.setEnabled(cellEnabled);
        cmpInfos[nbrCmp] = new ComponentInfo(cmp, constr);
        nbrCmp++;
        // check the wheights and fills x
        if (weightx < constr.weightx) weightx = constr.weightx;
        if (weighty < constr.weighty) weighty = constr.weighty;
        if ((constr.fill == MultiBorderConstraints.HORIZONTAL) || (constr.fill == MultiBorderConstraints.BOTH)) nbrFillX++;
    }


    public int getWidth() {
        return width;
    }


    public int getHeight() {
        return height;
    }


    public int getGridX() {
        return gridx;
    }


    public int getLastX() {
        return gridx + gridwidth - 1;
    }


    public int getGridY() {
        return gridy;
    }


    public int getLastY() {
        return gridy + gridheight - 1;
    }


    public int getGridWith() {
        return gridwidth;
    }


    public int getGridHeight() {
        return gridheight;
    }


    public double getWeightX() {
        return weightx;
    }


    public double getWeightY() {
        return weighty;
    }


    public boolean beginAt(int x, int y) {
        return (gridx == x) && (gridy == y);
    }


    public boolean endAt(int x, int y) {
        return (getLastX() == x) && (getLastY() == y);
    }


    public int getTotalWidth() {
        return borderSpace.left + width + borderSpace.right;
    }


    public int getTotalHeight() {
        return borderSpace.top + height + borderSpace.bottom;
    }


    public int getTopBorderSpace() {
        return borderSpace.top;
    }


    public int getLeftBorderSpace() {
        return borderSpace.left;
    }


    public int getBottomBorderSpace() {
        return borderSpace.bottom;
    }


    public int getRightBorderSpace() {
        return borderSpace.right;
    }


    public void addTopBorderSpace(int space) {
        borderSpace.top += space;
    }


    public void addLeftBorderSpace(int space) {
        borderSpace.left += space;
    }


    public void addBottomBorderSpace(int space) {
        borderSpace.bottom += space;
    }


    public void addRightBorderSpace(int space) {
        borderSpace.right += space;
    }


    /**
     * Reset space reserved for borders.
     */
    public void resetBorder() {
        // reset the border space
        borderSpace.top = 0;
        borderSpace.left = 0;
        borderSpace.bottom = 0;
        borderSpace.right = 0;
    }


    /**
     * Calculate and cache the size of this component and reset the border insets.
     */
    public void calculateSize() {
        // calculate the size.
        height = 0;
        width = 0;
        if (nbrCmp > 0) {
            for (int i = 0; i < nbrCmp; i++) {
                ComponentInfo cmpInfo = cmpInfos[i];
                cmpInfo.calculateSize();
                if ((cellCollapsed & COLLAPSED_HORIZONTAL) == 0) width += cmpInfo.getWidth();
                if ((cellCollapsed & COLLAPSED_VERTICAL) == 0) {
                    if (cmpInfo.getHeight() > height) height = cmpInfo.getHeight();
                }
            }
        }
        if (log.isTraceEnabled()) log.trace("Cell (" + gridx + " - " + gridy + ") size calculated: " + width + "x" + height
                + " (borders t:" + borderSpace.top + ",l:" + borderSpace.left + ",b:" + borderSpace.bottom + ",r:"
                + borderSpace.right + ")");
    }


    /**
     * The extra size of cells with gridX or gridY greater than 1 is splitted equally and added to all rows/columns.
     * 
     * @param colInfos
     *            size of columns
     * @param rowInfos
     *            size of rows
     */
    public void balanceExtraSize(LineInfo[] colInfos, LineInfo[] rowInfos) {
        // balance width
        if (gridwidth > 1) {
            int existingWidth = 0;
            int gridLastX = getLastX();
            for (int i = gridx; i <= gridLastX; i++)
                existingWidth += colInfos[i].getTotalSpace();
            existingWidth -= colInfos[gridx].getBeforeSpace();
            existingWidth -= colInfos[gridLastX].getAfterSpace();
            if (width > existingWidth) {
                int neededWidth = width - existingWidth;
                int addedWidth = neededWidth / gridwidth;
                for (int i = gridx; i < gridLastX; i++) {
                    colInfos[i].setSpace(colInfos[i].getSpace() + addedWidth);
                }
                colInfos[gridLastX].setSpace(colInfos[gridLastX].getSpace() + neededWidth - (addedWidth * (gridwidth - 1)));
            }
        }
        // balance height
        if (gridheight > 1) {
            int existingHeight = 0;
            int gridLastY = getLastY();
            for (int i = gridy; i <= gridLastY; i++)
                existingHeight += rowInfos[i].getTotalSpace();
            existingHeight -= rowInfos[gridy].getBeforeSpace();
            existingHeight -= rowInfos[gridLastY].getAfterSpace();
            if (log.isDebugEnabled()) log.debug("Height check: existing=" + existingHeight + ", needed=" + height);
            if (height > existingHeight) {
                int neededHeight = height - existingHeight;
                int addedHeight = neededHeight / gridheight;
                for (int i = gridy; i < gridLastY; i++) {
                    rowInfos[i].setSpace(rowInfos[i].getSpace() + addedHeight);
                }
                rowInfos[gridLastY].setSpace(rowInfos[gridLastY].getSpace() + neededHeight - (addedHeight * (gridheight - 1)));
            }
        }
    }


    /**
     * Lays out all the components in this Cell.
     * 
     * @param startX
     *            the position of the left of the cell.
     * @param startY
     *            the position of the top of the cell.
     */
    public void layout(LineInfo[] colInfos, LineInfo[] rowInfos) {
        // get the with and height of the cell
        int startX = colInfos[gridx].getLayedStart();
        int layWidth = colInfos[gridx + gridwidth - 1].getLayedEnd() - startX;
        int startY = rowInfos[gridy].getLayedStart();
        int layHeight = rowInfos[gridy + gridheight - 1].getLayedEnd() - startY;
        // adjust the filled width and heights
        int totalWidth = (nbrFillX == 0) ? width : layWidth;
        int freeXSpace = (nbrFillX == 0) ? 0 : layWidth - width;
        int extraXSpace = (freeXSpace <= 0) ? 0 : freeXSpace / nbrFillX;
        int modulo = (freeXSpace <= 0) ? 0 : freeXSpace % nbrFillX;
        for (int i = 0; i < nbrCmp; i++) {
            ComponentInfo cmpInfo = cmpInfos[i];
            int cmpFill = cmpInfo.getFill();
            if ((cmpFill == MultiBorderConstraints.HORIZONTAL) || (cmpFill == MultiBorderConstraints.BOTH)) {
                if (extraXSpace > 0) cmpInfo.setFilledWidth(cmpInfo.getWidth() + extraXSpace);
                else cmpInfo.setFilledWidth(cmpInfo.getWidth());
                if (modulo > 0) {
                    cmpInfo.setFilledWidth(cmpInfo.getFilledWidth() + 1);
                    modulo--;
                }
            } else {
                cmpInfo.setFilledWidth(cmpInfo.getWidth());
            }
            if ((cmpFill == MultiBorderConstraints.VERTICAL) || (cmpFill == MultiBorderConstraints.BOTH)) {
                cmpInfo.setFilledHeight(layHeight);
            } else {
                cmpInfo.setFilledHeight(cmpInfo.getHeight());
            }
        }
        // lays out components
        int posX;
        int posY;
        int anchor = (nbrCmp > 0) ? cmpInfos[0].getAnchor() : 0;
        switch (anchor) {
            case MultiBorderConstraints.SOUTH:
            case MultiBorderConstraints.CENTER:
            case MultiBorderConstraints.NORTH:
                posX = startX + ((layWidth - totalWidth) / 2);
                break;
            case MultiBorderConstraints.NORTHEAST:
            case MultiBorderConstraints.EAST:
            case MultiBorderConstraints.SOUTHEAST:
                posX = startX + (layWidth - totalWidth);
                break;
            case MultiBorderConstraints.SOUTHWEST:
            case MultiBorderConstraints.WEST:
            case MultiBorderConstraints.NORTHWEST:
            default:
                posX = startX;
                break;
        }
        for (int i = 0; i < nbrCmp; i++) {
            ComponentInfo cmpInfo = cmpInfos[i];
            switch (cmpInfo.getAnchor()) {
                case MultiBorderConstraints.SOUTH:
                case MultiBorderConstraints.SOUTHEAST:
                case MultiBorderConstraints.SOUTHWEST:
                    posY = startY;
                    break;
                case MultiBorderConstraints.CENTER:
                case MultiBorderConstraints.EAST:
                case MultiBorderConstraints.WEST:
                default:
                    posY = startY + ((layHeight - cmpInfo.getFilledHeight()) / 2);
                    break;
                case MultiBorderConstraints.NORTH:
                case MultiBorderConstraints.NORTHWEST:
                case MultiBorderConstraints.NORTHEAST:
                    posY = startY + (layHeight - cmpInfo.getFilledHeight());
                    break;
            }
            cmpInfo.layout(posX, posY);
            posX += cmpInfo.getFilledWidth();
        }
    }


    /**
     * Returns a String representation of this object (for debug purpose).
     * 
     * @return a String representation of this object.
     */
    @Override
    public String toString() {
        return "c(" + gridx + "," + gridy + ")-" + width + "x" + height;
    }


    public void setEnabled(boolean enabled) {
        if (cellEnabled != enabled) {
            cellEnabled = enabled;
            for (int i = 0; i < nbrCmp; i++) {
                cmpInfos[i].setEnabled(enabled);
            }
        }
    }


    public void setCollapsed(int newCollapsed) {
        if (cellCollapsed != newCollapsed) {
            for (int i = 0; i < nbrCmp; i++) {
                cmpInfos[i].setVisible(newCollapsed == COLLAPSED_NONE);
            }
            cellCollapsed = newCollapsed;
        }
    }


    public boolean isCollapsed() {
        return cellCollapsed != COLLAPSED_NONE;
    }


    boolean removeLayoutComponent(Component comp) {
        for (int i = 0; i < nbrCmp; i++) {
            if (cmpInfos[i].checkComponent(comp)) {
                // remove component at index i
                if (i < (nbrCmp - 1)) System.arraycopy(cmpInfos, i + 1, cmpInfos, i, nbrCmp - (i + 1));
                nbrCmp--;
                cmpInfos[nbrCmp] = null;
                return true;
            }
        }
        return false;
    }


    boolean isEmpty() {
        return nbrCmp == 0;
    }

}
