/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.layout;


import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Border to use with MultiBorderLayout. This border will be drawn around a set of components of the <code>MultiBorderLayout</code>.
 * To put border around components, you have to put this border in the <code>MultiBorderConstraints</code> while adding component to
 * the <code>MultiBorderLayout</code>. This border will then reserve space, calculate his bounds and draw itself at the right
 * position. The real drawing is delegated to a standard implementation of <code>javax.swing.border.Border</code> which can be
 * passed in constructor.
 * 
 * @author Olivier Berlanger
 * @see MultiBorderLayout
 * @see MultiBorderConstraints
 */
public class MultiBorder implements Border {

    private static Log log = LogFactory.getLog(MultiBorder.class);

    /** x position of this border. */
    protected int x;
    /** y position of this border. */
    protected int y;
    /** width of this border. */
    protected int width;
    /** height of this border. */
    protected int height;
    /** The insets of this MultiBorder (taking extra components size into account). */
    protected Insets updatedInsets;
    /** The insets of border painted by this MultiBorder. */
    protected Insets paintedBorderInsets;

    private int minCellX;
    private int maxCellX;
    private int minCellY;
    private int maxCellY;

    /** List containing the cellInfos enclosed by this border. */
    protected List<CellInfo> enclosedCells;
    /** List containing all the borders inside this border. */
    protected List<Border> nestedBorders;
    /** List containing the extra components (typically buttons) of this border. */
    private List<Component> borderComponents;
    /** Border used to delegate the real paint method. */
    protected Border paintedBorder;
    /** Space already used by other borders when this border added it own space. */
    private Insets currentBorderSpace;
    /** Flag saying if the border is visible. */
    private boolean borderVisible;
    /** Flag saying if the border is enabled (applies to the extra components). */
    private boolean borderEnabled;
    /** Flag saying if the border is visible. */
    private boolean extraComponentsSealed;
    /** Reference to the MultiBorderLayout using this border. */
    private MultiBorderLayout layout;


    /**
     * Creates an MultiBorder using an EtchedBorder.
     */
    public MultiBorder() {
        this(UIManager.getBorder("TitledBorder.border"));
    }


    /**
     * Creates an MultiBorder using an TitledBorder with the given title.
     * 
     * @param title
     *            the title of the titled border.
     */
    public MultiBorder(String title) {
        this(new TitledBorder(title));
    }


    /**
     * Creates an MultiBorder using the given border.
     * 
     * @param thePaintedBorder
     *            the border painted by this MultiBorder
     */
    public MultiBorder(Border thePaintedBorder) {
        paintedBorder = thePaintedBorder;
        currentBorderSpace = new Insets(0, 0, 0, 0);
        borderVisible = true;
        borderEnabled = true;
        minCellX = Integer.MAX_VALUE;
        minCellY = Integer.MAX_VALUE;
        enclosedCells = new ArrayList<CellInfo>();
    }


    protected void setLayout(MultiBorderLayout ownerLayout) {
        layout = ownerLayout;
    }


    public boolean isBorderEnabled() {
        return borderEnabled;
    }


    /**
     * Add a CellInfo to the set of cells enclosed by this border.
     * 
     * @param CellInfo
     *            the added CellInfo.
     */
    protected void add(CellInfo cell) {
        enclosedCells.add(cell);
        setBorderAroundCell(cell);
    }


    private void setBorderAroundCell(CellInfo cell) {
        int gridx = cell.getGridX();
        int gridy = cell.getGridY();
        int lastx = gridx + cell.getGridWith() - 1;
        int lasty = gridy + cell.getGridHeight() - 1;
        if (minCellX > gridx) minCellX = gridx;
        if (maxCellX < lastx) maxCellX = lastx;
        if (minCellY > gridy) minCellY = gridy;
        if (maxCellY < lasty) maxCellY = lasty;
    }


    /**
     * Check if this border contains the given cell.
     * 
     * @param cell
     *            the cell to check.
     * @return true iff this border contains the given cell.
     */
    public boolean containsCell(CellInfo cell) {
        int cellX = cell.getGridX();
        int cellY = cell.getGridY();
        return (minCellX <= cellX) && (cellX <= maxCellX) && (minCellY <= cellY) && (cellY <= maxCellY);
    }


    /**
     * @return true if this border contains only null or empty cells.
     */
    public boolean isEmpty() {
        for (int i = minCellX; i <= maxCellX; i++) {
            for (int j = minCellY; j <= maxCellY; j++) {
                CellInfo cell = layout.getCellInfo(i, j);
                if ((cell != null) && !cell.isEmpty()) return false;
            }
        }
        return true;
    }


    /**
     * Reserve space between the cells to be able to draw this border. This is done by adding a kind of insets to the CellInfos and
     * then increase their size.
     * 
     * @param c
     *            The Container (usually a JPanel) containing this border and using a MultiBorderLayout.
     */
    protected void adaptCellSizes(Container c) {
        // reset current border space
        currentBorderSpace.top = 0;
        currentBorderSpace.left = 0;
        currentBorderSpace.bottom = 0;
        currentBorderSpace.right = 0;
        Insets insts = getBorderInsets(c);
        // add the border to the cells
        if (borderVisible && (minCellX <= maxCellX) && (minCellY <= maxCellY)) {
            // top-bottom cells
            for (int i = minCellX; i <= maxCellX; i++) {
                // top cells
                CellInfo topCell = layout.getCellInfo(i, minCellY);
                if ((topCell != null) && topCell.beginAt(i, minCellY)) {
                    int topSpace = topCell.getTopBorderSpace();
                    if (currentBorderSpace.top < topSpace) currentBorderSpace.top = topSpace;
                    if (log.isDebugEnabled()) log.debug("Border " + this + " add " + insts.top + " space to cell " + topCell);
                    topCell.addTopBorderSpace(insts.top);
                }
                // bottom cells
                CellInfo bottomCell = layout.getCellInfo(i, maxCellY);
                if ((bottomCell != null) && bottomCell.endAt(i, maxCellY)) {
                    int bottomSpace = bottomCell.getBottomBorderSpace();
                    if (currentBorderSpace.bottom < bottomSpace) currentBorderSpace.bottom = bottomSpace;
                    bottomCell.addBottomBorderSpace(insts.bottom);
                }
            }
            // left-right cells
            for (int j = minCellY; j <= maxCellY; j++) {
                // left cells
                CellInfo leftCell = layout.getCellInfo(minCellX, j);
                if ((leftCell != null) && leftCell.beginAt(minCellX, j)) {
                    int leftSpace = leftCell.getLeftBorderSpace();
                    if (currentBorderSpace.left < leftSpace) currentBorderSpace.left = leftSpace;
                    leftCell.addLeftBorderSpace(insts.left);
                }
                // right cells
                CellInfo rightCell = layout.getCellInfo(maxCellX, j);
                if ((rightCell != null) && rightCell.endAt(maxCellX, j)) {
                    int rightSpace = rightCell.getRightBorderSpace();
                    if (currentBorderSpace.right < rightSpace) currentBorderSpace.right = rightSpace;
                    rightCell.addRightBorderSpace(insts.right);
                }
            }
        }
    }


    /**
     * Calculate the bounds of this border.
     */
    protected void layout(Container cont) {
        if ((minCellX <= maxCellX) && (minCellY <= maxCellY) && borderVisible) {
            x = layout.getColumnInfo(minCellX).getPos() + currentBorderSpace.left;
            y = layout.getRowInfo(minCellY).getPos() + currentBorderSpace.top;
            width = (layout.getColumnInfo(maxCellX).getLayedNextPos() - x) - currentBorderSpace.right;
            height = (layout.getRowInfo(maxCellY).getLayedNextPos() - y) - currentBorderSpace.bottom;
            sealExtraComponents();
            layoutExtraComponents(cont);
        } else {
            x = 0;
            y = 0;
            width = 0;
            height = 0;
        }
        if (log.isTraceEnabled()) log.trace("Layout border " + x + "," + y + " (" + width + "x" + height + ")");
    }


    /**
     * Paint this border.
     * 
     * @param c
     *            The Container (usually a JPanel) containing this border and using a MultiBorderLayout.
     * @param g
     *            graphics where paint is done.
     */
    protected void paint(Component c, Graphics g) {
        log.trace("Paint in MultiBorder");
        if ((width > 0) && (height > 0)) paintBorder(c, g, x, y, width, height);
    }


    /**
     * Set this border and all it's content enabled/disbaled
     */
    public void setBorderEnabled(boolean enabled) {
        sealExtraComponents();
        if (borderEnabled != enabled) {
            borderEnabled = enabled;
            // set nested borders enabled
            int len = (nestedBorders == null) ? 0 : nestedBorders.size();
            for (int i = 0; i < len; i++) {
                MultiBorder border = (MultiBorder) nestedBorders.get(i);
                border.setBorderEnabled(enabled);
            }
            // set the border extra components enabled
            if (borderComponents != null) {
                int nbrCmp = borderComponents.size();
                for (int i = 0; i < nbrCmp; i++) {
                    Component cmp = borderComponents.get(i);
                    cmp.setEnabled(enabled);
                }
            }
        }
    }


    /**
     * Set this border visible/invisible. An invisible border will have no height but will still reserve horizontal space (so the
     * layout is only shrinked vertically). It this border contains other borders, it will ensure that those inner borders will also
     * be visible/invisible.
     * 
     * @param visible
     *            true if this border should be visible.
     */
    public void setBorderVisible(boolean visible) {
        sealExtraComponents();
        if (borderVisible != visible) {
            borderVisible = visible;
            // set nested borders visible
            int len = (nestedBorders == null) ? 0 : nestedBorders.size();
            for (int i = 0; i < len; i++) {
                MultiBorder border = (MultiBorder) nestedBorders.get(i);
                border.setBorderVisible(visible);
            }
            // set the border extra components visible
            if (borderComponents != null) {
                for (Component cmp : borderComponents) {
                    cmp.setVisible(visible);
                }
            }
        }
    }


    /**
     * Returns true if this border is visible.
     * 
     * @return true if this border is visible.
     */
    public boolean isBorderVisible() {
        return borderVisible;
    }


    /**
     * Add an reference to an other border contained in this border (if it is not already done).
     * 
     * @param border
     *            the inner border
     */
    protected void setNestedBorder(MultiBorder border) {
        if (nestedBorders == null) nestedBorders = new ArrayList<Border>();
        if (!nestedBorders.contains(border)) nestedBorders.add(border);
    }


    @Override
    public String toString() {
        String title = "";
        if (paintedBorder == null) title = "null";
        // else if (paintedBorder instanceof TitledBorder2) title = ((TitledBorder2) paintedBorder).getTitle();
        else if (paintedBorder instanceof TitledBorder) title = ((TitledBorder) paintedBorder).getTitle();
        else title = paintedBorder.getClass().getName();
        return getClass().getName() + "['" + title + "']";
    }


    // public void dump() {
    // System.out.println(toString()) ;
    // int nbrNested=(nestedBorders==null) ? 0 : nestedBorders.size() ;
    // System.out.println(" -> "+nbrNested+" nested borders") ;
    // for (int i=0 ; i<nbrNested ; i++) {
    // System.out.println(" = "+nestedBorders.get(i)) ;
    // }
    // int nbrCells=enclosedCells.size() ;
    // System.out.println(" -> "+nbrCells+" cells") ;
    // for (int i=0 ; i<nbrCells ; i++) {
    // System.out.println(" = "+enclosedCells.get(i)) ;
    // }
    // }

    // ---------------------- Extra components management --------------------------------------------

    /**
     * Called by the generalLayout to give this border the opporunity to add some control component to the parent panel.
     * 
     * @param cont
     *            The container (usually a JPanel) containing this border and using a MultiBorderLayout.
     */
    protected void addExtraComponents(Container cont) {
        sealExtraComponents();
        if (borderComponents != null) {
            for (Component cmp : borderComponents) {
                // hack: use this as constraints to notify the general layout that the
                // layout of the added component is managed by this border.
                cont.add(cmp, this);
            }
        }
    }


    protected void layoutExtraComponents(Container cont) {
        // to be overriden
    }


    /**
     * Request any sub-class to add it's extra components and then seal the extra component list. Once it's sealed no more component
     * can be added. (So the size of the border cannot change once rendered)
     */
    private void sealExtraComponents() {
        if (!extraComponentsSealed) registerExtraComponents();
        extraComponentsSealed = true;
    }


    /**
     * Request to add the extra border components list. This method is just called once. Subclasses can override this method to add
     * some default component using the addBorderComponent(...) method.
     */
    protected void registerExtraComponents() {
        // to be overriden
    }


    protected void registerExtraComponent(Component cmp) {
        if (extraComponentsSealed) throw new IllegalStateException("Border components must be added"
                + "before the first time the border is shown");
        if (borderComponents == null) borderComponents = new ArrayList<Component>();
        borderComponents.add(cmp);
    }


    protected Insets getUpdatedInsets(Component cmp) {
        if (updatedInsets == null) createCachedInsets(cmp);
        return updatedInsets;
    }


    protected Insets getPaintedBorderInsets(Component cmp) {
        if (paintedBorderInsets == null) createCachedInsets(cmp);
        return paintedBorderInsets;
    }


    private void createCachedInsets(Component cmp) {
        paintedBorderInsets = paintedBorder.getBorderInsets(cmp);
        updatedInsets = (Insets) paintedBorderInsets.clone();
        updateInsets(updatedInsets);
    }


    protected void updateInsets(Insets insetToUpdate) {
        // to be overriden
    }


    // /////////////// Border interface implementation //////////////////////////////////////////////

    /**
     * Paints the border for the specified component with the specified position and size.
     * 
     * @param c
     *            the component for which this border is being painted
     * @param g
     *            the paint graphics
     * @param x1
     *            the x position of the painted border
     * @param y1
     *            the y position of the painted border
     * @param width1
     *            the width of the painted border
     * @param height1
     *            the height of the painted border
     */
    public void paintBorder(Component c, Graphics g, int x1, int y1, int width1, int height1) {
        int leftGap = (updatedInsets.left - paintedBorderInsets.left) / 2;
        int rightGap = (updatedInsets.right - paintedBorderInsets.right) / 2;
        int topGap = (updatedInsets.top - paintedBorderInsets.top) / 2;
        int bottomGap = (updatedInsets.bottom - paintedBorderInsets.bottom) / 2;
        if (log.isTraceEnabled()) log.trace("Adjust multi border to " + (x1 + leftGap) + "," + (y1 + topGap) + " ("
                + (width1 - (leftGap + rightGap)) + "x" + (height1 - (topGap + bottomGap)) + ")");
        paintedBorder.paintBorder(c, g, x1 + leftGap, y1 + topGap, width1 - (leftGap + rightGap), height1 - (topGap + bottomGap));
    }


    /**
     * Returns the insets of the border.
     * 
     * @param c
     *            the component for which this border insets value applies
     * @return the insets of the border.
     */
    public Insets getBorderInsets(Component c) {
        return (Insets) getUpdatedInsets(c).clone();
    }


    /**
     * This method is ignored by MultiBorderLayout.
     * 
     * @return if the painted border is opaque or not.
     */
    public boolean isBorderOpaque() {
        return paintedBorder.isBorderOpaque();
    }


    public void dispose(Container cont) {
        // remove extra components
        if (borderComponents != null) {
            if (cont != null) {
                int len = borderComponents.size();
                for (int i = 0; i < len; i++) {
                    JComponent cmp = (JComponent) borderComponents.get(i);
                    cont.remove(cmp);
                }
            }
            borderComponents.clear();
        }
        extraComponentsSealed = false;
        // remove enclosed cells & borders
        enclosedCells.clear();
        nestedBorders = null;
        layout = null;
    }

}
