/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.layout;


import java.awt.Insets;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 * Constraints object used to add components to a panel using a MultiBorderLayout. <br>
 * This constrains behave, in great lines, like the GridBagConstraints.
 * 
 * @see MultiBorderLayout for more info
 */
public class MultiBorderConstraints implements Cloneable {


    // constant for relative positioning
    public static final int NEXT = -1;
    public static final int CURRENT = -1000;
    public static final int NEW = -1001;
    public static final int NEXT_IF_COLLISION = -1002;
    public static final int UNDEFINED = -1003;

    // resize - disposition
    public static final int NONE = 0;
    public static final int BOTH = 1;
    public static final int HORIZONTAL = 2;
    public static final int VERTICAL = 3;

    // Alignment
    public static final int CENTER = 10;
    public static final int NORTH = 11;
    public static final int NORTHEAST = 12;
    public static final int EAST = 13;
    public static final int SOUTHEAST = 14;
    public static final int SOUTH = 15;
    public static final int SOUTHWEST = 16;
    public static final int WEST = 17;
    public static final int NORTHWEST = 18;

    public int gridx;
    public int gridy;
    public int gridwidth;
    public int gridheight;
    public double weightx;
    public double weighty;
    public int anchor;
    public int fill;
    public Insets insets;

    private List<MultiBorder> borders;
    private int borderFlag;


    /**
     * Constructs constraints with default values.
     */
    public MultiBorderConstraints() {
        gridx = NEXT;
        gridy = CURRENT;
        gridwidth = 1;
        gridheight = 1;
        weightx = 0.0;
        weighty = 0.0;
        anchor = WEST;
        fill = NONE;
        insets = null;
    }


    public MultiBorderConstraints(int newGridX, int newGridY, int newGridWidth, int newGridHeight, double newWeightX,
            double newWeightY, int newAnchor, int newFill, Insets newInsets) {
        gridx = newGridX;
        gridy = newGridY;
        gridwidth = newGridWidth;
        gridheight = newGridHeight;
        weightx = newWeightX;
        weighty = newWeightY;
        anchor = newAnchor;
        fill = newFill;
        insets = newInsets;
    }


    /**
     * Construct copy of the MultiBorderConstraints given as parameter.
     * 
     * @param other
     *            the constraints to copy.
     */
    public MultiBorderConstraints(MultiBorderConstraints other) {
        gridx = other.gridx;
        gridy = other.gridy;
        gridwidth = other.gridwidth;
        gridheight = other.gridheight;
        weightx = other.weightx;
        weighty = other.weighty;
        anchor = other.anchor;
        fill = other.fill;
        insets = (other.insets == null) ? null : new Insets(other.insets.top, other.insets.left, other.insets.bottom,
                other.insets.right);
    }


    public MultiBorderConstraints setConstraints(int newGridX, int newGridY, int newGridWidth, int newGridHeight,
            double newWeightX, double newWeightY, int newAnchor, int newFill, Insets newInsets) {
        gridx = newGridX;
        gridy = newGridY;
        gridwidth = newGridWidth;
        gridheight = newGridHeight;
        weightx = newWeightX;
        weighty = newWeightY;
        anchor = newAnchor;
        fill = newFill;
        insets = newInsets;
        return this;
    }


    public MultiBorderConstraints setConstraints(int newGridX, int newGridY, int newGridWidth, int newGridHeight,
            double newWeightX, double newWeightY, int newAnchor, int newFill) {
        gridx = newGridX;
        gridy = newGridY;
        gridwidth = newGridWidth;
        gridheight = newGridHeight;
        weightx = newWeightX;
        weighty = newWeightY;
        anchor = newAnchor;
        fill = newFill;
        return this;
    }


    public MultiBorderConstraints setConstraints(int newGridX, int newGridY, int newGridWidth, int newGridHeight,
            double newWeightX, double newWeightY, int newAnchor) {
        gridx = newGridX;
        gridy = newGridY;
        gridwidth = newGridWidth;
        gridheight = newGridHeight;
        weightx = newWeightX;
        weighty = newWeightY;
        anchor = newAnchor;
        return this;
    }


    public MultiBorderConstraints setConstraints(int newGridX, int newGridY, int newGridWidth, int newGridHeight,
            double newWeightX, double newWeightY) {
        gridx = newGridX;
        gridy = newGridY;
        gridwidth = newGridWidth;
        gridheight = newGridHeight;
        weightx = newWeightX;
        weighty = newWeightY;
        return this;
    }


    public MultiBorderConstraints setConstraints(int newGridX, int newGridY, int newGridWidth, int newGridHeight) {
        gridx = newGridX;
        gridy = newGridY;
        gridwidth = newGridWidth;
        gridheight = newGridHeight;
        return this;
    }


    public MultiBorderConstraints setConstraints(int newGridX, int newGridY) {
        gridx = newGridX;
        gridy = newGridY;
        return this;
    }


    public void setInsets(int top, int left, int bottom, int right) {
        if (insets == null) insets = new Insets(top, left, bottom, right);
        else {
            insets.top = top;
            insets.left = left;
            insets.bottom = bottom;
            insets.right = right;
        }
    }


    public void addBorder(MultiBorder brdr) {
        if (borders == null) borders = new ArrayList<MultiBorder>();
        borders.add(brdr);
        borderFlag++;
    }


    public void removeBorder(MultiBorder brdr) {
        if (borders != null) borders.remove(brdr);
    }


    public void removeAllBorders() {
        if (borders != null) borders.clear();
    }


    @Override
    public Object clone() {
        try {
            return super.clone();
        } catch (CloneNotSupportedException cnse) {
            throw new InternalError("Should not happen");
        }
    }


    protected Iterator<MultiBorder> getBordersIterator() {
        if (borders == null) borders = new ArrayList<MultiBorder>();
        return borders.iterator();
    }


    protected int getBorderFlag() {
        return borderFlag;
    }


    /**
     * Update the current position used by a MultiBorderLayout. This method will resolve all the relative indexes to absolute
     * ones and copy the result in this currentPosition. Other values are simply copied to this currentPosition.
     */
    protected void updateCurrentPosition(MultiBorderLayout layout, MultiBorderConstraints relativePosition) {
        // set absolute gridy position
        switch (relativePosition.gridy) {
            case NEXT:
                gridy++;
                break;
            case CURRENT:
                if (gridy < 0) gridy = 0;
                break;
            case NEW:
                gridy = layout.getNbrRow(relativePosition.gridx);
                break;
            default:
                // rem: negatives values mean increment from current position
                if (relativePosition.gridy < 0) gridy -= relativePosition.gridy;
                else gridy = relativePosition.gridy;
        }
        // set absolute gridx position
        switch (relativePosition.gridx) {
            case NEXT:
                gridx++;
                break;
            case CURRENT:
                if (gridx < 0) gridx = 0;
                break;
            case NEW:
                gridx = layout.getNbrColumn(gridy);
                break;
            default:
                // rem: negatives values mean increment from current position
                if (relativePosition.gridx < 0) gridx -= relativePosition.gridx;
                else gridx = relativePosition.gridx;
        }
        // copy other values
        gridwidth = relativePosition.gridwidth;
        gridheight = relativePosition.gridheight;
        weightx = relativePosition.weightx;
        weighty = relativePosition.weighty;
        anchor = relativePosition.anchor;
        fill = relativePosition.fill;
        insets = relativePosition.insets;
        borderFlag = (borders == relativePosition.borders) ? relativePosition.borderFlag : -1;
        borders = relativePosition.borders;
    }


    /**
     * Init a constraint without relative values to be used as the current position in a <code>MultiBorderLayout</code>.
     */
    protected void initCurrentPosition() {
        gridx = -1;
        gridy = -1;
        gridwidth = 1;
        gridheight = 1;
    }


}
