/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.layout;


import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.LayoutManager2;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.border.Border;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Layout manager working like a GridBagLayout but allowing to add several borders on the same JPanel.
 */
public final class MultiBorderLayout implements LayoutManager2 {

    private static Log log = LogFactory.getLog(MultiBorderLayout.class);

    /** Global flag to turn the visual debug mode of the layout. When true it draws the cell boreders in red. */
    private static boolean debugMode = false;

    /** Indicates if the cached size informations are valid or not. */
    private boolean cacheValid;
    /** Matrix holding all the cells info for this layout. */
    private SparseMatrix cells;
    /** List holding all the borders set to this layout in the order of appearance. */
    private List<MultiBorder> bordersList;
    /** Flag to be notified when list of borders in currentPosition has changed. */
    private int borderFlag;
    /** Total width of the layout. */
    private int width;
    /** Total height of the layout. */
    private int height;
    /** Info about each column of this layout (width, weight, position, ...). */
    private LineInfo[] colInfos;
    /** Info about each row of this layout (height, weight, position, ...). */
    private LineInfo[] rowInfos;
    /** Total x weight of the layout. */
    private double weightX;
    /** Total y weight of the layout. */
    private double weightY;

    /** Alignment of the content of the layout. */
    private int layoutAlignment;
    /** Retain the position used to add the last component. */
    private MultiBorderConstraints currentPosition;
    /** The container using this layout manager. */
    private JComponent layoutContainer;
    /** global insets. */
    private Insets globalInsets = new Insets(0, 0, 0, 0);


    public MultiBorderLayout() {
        cells = new SparseMatrix();
        bordersList = new ArrayList<MultiBorder>();
        cacheValid = false;
        currentPosition = new MultiBorderConstraints();
        currentPosition.initCurrentPosition();
        layoutAlignment = MultiBorderConstraints.CENTER;
        // init the row/col infos
        colInfos = new LineInfo[5];
        rowInfos = new LineInfo[5];
        for (int i = 0; i < 5; i++) {
            colInfos[i] = new LineInfo();
            rowInfos[i] = new LineInfo();
        }
    }


    public static boolean getDebugMode() {
        return debugMode;
    }


    public static void setDebugMode(boolean newDebugMode) {
        debugMode = newDebugMode;
    }


    /**
     * Returns a String representation of this object (for debug purpose).
     * 
     * @return a String representation of this object.
     */
    @Override
    public String toString() {
        return getClass().getName() + "[" + cells.getMaxNbrColumns() + "x" + cells.getNbrRows() + " cells, align="
                + layoutAlignment + "]";
    }


    public void setLayoutAlignment(int alignment) {
        layoutAlignment = alignment;
    }


    public int getLayoutAlignment() {
        return layoutAlignment;
    }


    /**
     * Get the global insets. <br>
     * 
     * @see #setGlobalInsets
     * @return The global insets.
     */
    public Insets getGlobalInsets() {
        return globalInsets;
    }


    /**
     * Set a global Inset around all the components of this layout. <br>
     * It behaves as if you add an empty border on the container using this layout (but remmember that, when using the
     * MultiBorderLayout, you cannot add a Border directly on the container. You have to add borders using the
     * {@link MultiBorderConstraints}).
     * <p>
     * By default this global Insets is empty (0,0,0,0).
     * </p>
     * 
     * @param globalInsets
     *            the new global insets.
     */
    public void setGlobalInsets(Insets newGlobalInsets) {
        globalInsets = newGlobalInsets;
        if (globalInsets == null) globalInsets = new Insets(0, 0, 0, 0);
    }


    // --------------------- Implementation of the LayoutManager interface ------------------------------------------

    /**
     * Adds the specified component with the specified name to the layout.
     * 
     * @param name
     *            the component name
     * @param comp
     *            the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {
        throw new IllegalArgumentException("MultiBorderLayout requires a MultiBorderConstraints to add a component");
    }


    /**
     * Removes the specified component from the layout.
     * 
     * @param comp
     *            the component to be removed
     */
    public void removeLayoutComponent(Component comp) {
        invalidateCache();
        currentPosition.initCurrentPosition();
        // scan the cells to find the component
        int nbrRow = cells.getNbrRows();
        for (int y = 0; y < nbrRow; y++) {
            int nbrX = cells.getNbrColumns(y);
            for (int x = 0; x < nbrX; x++) {
                CellInfo cell = (CellInfo) cells.get(x, y);
                if (cell != null) {
                    if (cell.removeLayoutComponent(comp)) {
                        // found the cell containing the component
                        if (cell.isEmpty()) {
                            // remove the cell iteself
                            int cellEndX = cell.getGridX() + cell.getGridWith();
                            int cellEndY = cell.getGridY() + cell.getGridHeight();
                            for (int i = cell.getGridX(); i < cellEndX; i++) {
                                for (int j = cell.getGridY(); j < cellEndY; j++) {
                                    cells.set(i, j, null);
                                }
                            }
                            // check if the border containing this cell must also be removed
                            checkBordersForRemove(cell);
                        }
                        return;
                    }
                }
            }
        }
        log.warn("The component [" + comp + "] to remove is not in this layoutManager");
    }


    private void checkBordersForRemove(CellInfo removedCell) {
        int len = bordersList.size();
        for (int i = 0; i < len; i++) {
            MultiBorder border = bordersList.get(i);
            if (border.containsCell(removedCell) && border.isEmpty()) {
                // remove border
                removeBorder(border);
            }
        }
    }


    private void removeBorder(MultiBorder border) {
        bordersList.remove(border);
        border.dispose(layoutContainer);
    }


    /**
     * Calculates the preferred size dimensions for the specified panel given the components in the specified parent container.
     * 
     * @param parent
     *            the component to be laid out
     * @see #minimumLayoutSize
     */
    public Dimension preferredLayoutSize(Container container) {
        if (container != layoutContainer) setLayoutContainer(container);
        validateCache(container);
        return new Dimension(width, height);
    }


    /**
     * Calculates the minimum size dimensions for the specified panel given the components in the specified parent container.
     * 
     * @param parent
     *            the component to be laid out
     * @see #preferredLayoutSize
     */
    public Dimension minimumLayoutSize(Container container) {
        if (container != layoutContainer) setLayoutContainer(container);
        validateCache(container);
        return new Dimension(width, height);
    }


    /**
     * Lays out the container in the specified panel.
     * 
     * @param parent
     *            the component which needs to be laid out
     */
    public void layoutContainer(Container container) {
        if (container != layoutContainer) setLayoutContainer(container);
        validateCache(container);
        layout(container);
        layoutBorders(container);
    }


    // --------------------- Implementation of the LayoutManager2 interface ------------------------------------------

    /**
     * Adds the specified component to the layout, using the specified constraint object.
     * 
     * @param comp
     *            the component to be added
     * @param constraints
     *            where/how the component is added to the layout.
     */
    public void addLayoutComponent(Component comp, Object constr) {
        // If the component is added by a MultiBorder, it will lay it out itself.
        if (constr instanceof MultiBorder) {
            if (log.isDebugEnabled()) log.debug("Added extra component: " + comp.getClass());
            return;
        }
        // Add component using MultiBorderConstraints
        if (constr instanceof MultiBorderConstraints) {
            if (log.isDebugEnabled()) log.debug("Added layout component: " + comp.getClass());
            currentPosition.updateCurrentPosition(this, (MultiBorderConstraints) constr);
            CellInfo info = (CellInfo) cells.get(currentPosition.gridx, currentPosition.gridy);
            if (info == null) {
                info = new CellInfo(currentPosition);
                int lastX = currentPosition.gridx + currentPosition.gridwidth - 1;
                int lastY = currentPosition.gridy + currentPosition.gridheight - 1;
                // iterating from top to bottom to avoid resizing internal arrays twice
                for (int y = lastY; y >= currentPosition.gridy; y--) {
                    for (int x = lastX; x >= currentPosition.gridx; x--) {
                        cells.set(x, y, info);
                    }
                }
            }
            info.addComponent(comp, currentPosition);
            // add the borders
            MultiBorder lastBorder = null;
            if (borderFlag != currentPosition.getBorderFlag()) {
                for (Iterator<MultiBorder> it = currentPosition.getBordersIterator(); it.hasNext();) {
                    MultiBorder border = it.next();
                    if (!bordersList.contains(border)) {
                        bordersList.add(border);
                        border.setLayout(this);
                        if (layoutContainer != null) {
                            if (log.isDebugEnabled()) log.debug("addLayoutComponent Call addComponents on border: " + border);
                            border.addExtraComponents(layoutContainer);
                        }
                    }
                    if (lastBorder != null) lastBorder.setNestedBorder(border);
                    lastBorder = border;
                }
                borderFlag = currentPosition.getBorderFlag();
                if (borderFlag < 0) borderFlag = 0;
            }
            // this invalidates the cached information
            invalidateCache();
        } else {
            // wrong constraint type
            throw new IllegalArgumentException("MultiBorderLayout requires a MultiBorderConstraints to add a component");
        }
    }


    /**
     * Returns the maximum size of this component.
     * 
     * @see java.awt.Component#getMinimumSize()
     * @see java.awt.Component#getPreferredSize()
     * @see LayoutManager
     */
    public Dimension maximumLayoutSize(Container container) {
        if (container != layoutContainer) setLayoutContainer(container);
        return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
    }


    /**
     * Returns the alignment along the x axis. This specifies how the component would like to be aligned relative to other
     * components. The value should be a number between 0 and 1 where 0 represents alignment along the origin, 1 is aligned the
     * furthest away from the origin, 0.5 is centered, etc.
     */
    public float getLayoutAlignmentX(Container container) {
        if (container != layoutContainer) setLayoutContainer(container);
        return 0.5f;
    }


    /**
     * Returns the alignment along the y axis. This specifies how the component would like to be aligned relative to other
     * components. The value should be a number between 0 and 1 where 0 represents alignment along the origin, 1 is aligned the
     * furthest away from the origin, 0.5 is centered, etc.
     */
    public float getLayoutAlignmentY(Container container) {
        if (container != layoutContainer) setLayoutContainer(container);
        return 0.5f;
    }


    /**
     * Invalidates the layout, indicating that if the layout manager has cached information it should be discarded.
     */
    public void invalidateLayout(Container container) {
        if (container != layoutContainer) setLayoutContainer(container);
        invalidateCache();
    }


    // ---------------------------------------------------------------------------------------------------------

    int getNbrRow(int gridx) {
        int nbrRow = cells.getNbrRows();
        if (gridx >= 0) {
            if (gridx >= cells.getMaxNbrColumns()) return 0;
            for (; (nbrRow > 0) && (cells.get(gridx, nbrRow - 1) == null); nbrRow--) {
                // just to update the nbrRow variable
            }
        }
        return nbrRow;
    }


    int getNbrColumn(int gridy) {
        return cells.getNbrColumns(gridy);
    }


    CellInfo getCellInfo(int gridx, int gridy) {
        return (CellInfo) cells.get(gridx, gridy);
    }


    LineInfo getColumnInfo(int gridx) {
        return colInfos[gridx];
    }


    LineInfo getRowInfo(int gridy) {
        return rowInfos[gridy];
    }


    /**
     * invalidates the cached information in the manager.
     */
    private void invalidateCache() {
        cacheValid = false;
    }


    /**
     * validates the cached information in the layout manager.
     */
    private void validateCache(Container cont) {
        if (!cacheValid) {
            width = 0;
            height = 0;
            resetBorder();
            calculateBorders(cont);
            resetRowColInfo();
            calculateSize();
            cacheValid = true;
        }
    }


    /**
     * Reset the cached variables keeping space used by borders.
     */
    private void resetBorder() {
        int nbrY = cells.getNbrRows();
        for (int y = 0; y < nbrY; y++) {
            int nbrX = cells.getNbrColumns(y);
            for (int x = 0; x < nbrX; x++) {
                CellInfo cell = (CellInfo) cells.get(x, y);
                if ((cell != null) && cell.beginAt(x, y)) cell.resetBorder();
            }
        }
    }


    /**
     * Calculate the cached variables keeping space used by borders.
     */
    private void calculateBorders(Container cont) {
        int len = bordersList.size();
        for (int i = 0; i < len; i++) {
            MultiBorder border = bordersList.get(i);
            border.adaptCellSizes(cont);
        }
    }


    private void resetRowColInfo() {
        // reset column info
        int nbrColInfo = colInfos.length;
        int nbrCol = cells.getMaxNbrColumns();
        if (nbrCol > nbrColInfo) {
            for (int i = 0; i < nbrColInfo; i++)
                colInfos[i].reset();
            LineInfo[] newColInfos = new LineInfo[nbrCol];
            System.arraycopy(colInfos, 0, newColInfos, 0, nbrColInfo);
            for (int i = nbrColInfo; i < nbrCol; i++)
                newColInfos[i] = new LineInfo();
            colInfos = newColInfos;
        } else {
            for (int i = 0; i < nbrCol; i++)
                colInfos[i].reset();
        }
        // reset row info
        int nbrRowInfo = rowInfos.length;
        int nbrRow = cells.getNbrRows();
        if (nbrRow > nbrRowInfo) {
            for (int i = 0; i < nbrRowInfo; i++)
                rowInfos[i].reset();
            LineInfo[] newRowInfos = new LineInfo[nbrRow];
            System.arraycopy(rowInfos, 0, newRowInfos, 0, nbrRowInfo);
            for (int i = nbrRowInfo; i < nbrRow; i++)
                newRowInfos[i] = new LineInfo();
            rowInfos = newRowInfos;
        } else {
            for (int i = 0; i < nbrRow; i++)
                rowInfos[i].reset();
        }
    }


    /**
     * Calculate the size of the layout and each of it's components. Also cache calculate the weight of reach row/column
     */
    private void calculateSize() {
        int nbrCol = cells.getMaxNbrColumns();
        int nbrRow = cells.getNbrRows();
        // first pass - 1 sized cells
        for (int y = 0; y < nbrRow; y++) {
            int nbrX = cells.getNbrColumns(y);
            LineInfo rowInfo = rowInfos[y];
            for (int x = 0; x < nbrX; x++) {
                CellInfo cell = (CellInfo) cells.get(x, y);
                LineInfo colInfo = colInfos[x];
                if (cell != null) {
                    if (cell.beginAt(x, y)) {
                        cell.calculateSize();
                        if ((cell.getGridWith() == 1) && (colInfo.getSpace() < cell.getWidth())) colInfo.setSpace(cell.getWidth());
                        if ((cell.getGridHeight() == 1) && (rowInfo.getSpace() < cell.getHeight())) rowInfo.setSpace(cell
                                .getHeight());
                        if (colInfo.getBeforeSpace() < cell.getLeftBorderSpace()) colInfo.setBeforeSpace(cell.getLeftBorderSpace());
                        if (rowInfo.getBeforeSpace() < cell.getTopBorderSpace()) rowInfo.setBeforeSpace(cell.getTopBorderSpace());
                    }
                    double weightX1 = cell.getWeightX() / cell.getGridWith();
                    if (colInfo.getWeight() < weightX1) colInfo.setWeight(weightX1);
                    double weightY1 = cell.getWeightY() / cell.getGridHeight();
                    if (rowInfo.getWeight() < weightY1) rowInfo.setWeight(weightY1);
                    if (cell.endAt(x, y)) {
                        if (colInfo.getAfterSpace() < cell.getRightBorderSpace()) colInfo.setAfterSpace(cell.getRightBorderSpace());
                        if (rowInfo.getAfterSpace() < cell.getBottomBorderSpace()) rowInfo.setAfterSpace(cell
                                .getBottomBorderSpace());
                    }
                }
            }
        }
        // second pass - n sized cells
        for (int y = 0; y < nbrRow; y++) {
            int nbrX = cells.getNbrColumns(y);
            for (int x = 0; x < nbrX; x++) {
                CellInfo cell = (CellInfo) cells.get(x, y);
                if ((cell != null) && cell.beginAt(x, y)) cell.balanceExtraSize(colInfos, rowInfos);
            }
        }
        // accumulate the row/col sizes and weights
        width = 0;
        height = 0;
        weightX = 0.0;
        weightY = 0.0;
        for (int i = 0; i < nbrCol; i++) {
            LineInfo colInfo = colInfos[i];
            width += colInfo.getTotalSpace();
            weightX += colInfo.getWeight();
        }
        for (int i = 0; i < nbrRow; i++) {

            LineInfo rowInfo = rowInfos[i];
            if (log.isDebugEnabled()) log.debug("Row " + i + " height = " + rowInfo.getBeforeSpace() + " - " + rowInfo.getSpace()
                    + " - " + rowInfo.getAfterSpace());
            height += rowInfo.getTotalSpace();
            weightY += rowInfo.getWeight();
        }
        // debug info
        if (log.isDebugEnabled()) log.debug("MultiBorderLayout sized: " + width + "x" + height);
    }


    // public void dump() {
    // System.out.println(getClass().getName()) ;
    // System.out.println(" -> size: "+width+"x"+height) ;
    // System.out.println(" -> weightX:"+weightX+", weightY"+weightY) ;
    // System.out.print(" -> weights X: [") ;
    // for(int i=0 ; i<colInfos.length ; i++) {
    // if (i>0) System.out.print(", ") ;
    // System.out.print(colInfos[i].weight) ;
    // }
    // System.out.println("]") ;
    // System.out.print(" -> weights y: [") ;
    // for(int i=0 ; i<rowInfos.length ; i++) {
    // if (i>0) System.out.print(", ") ;
    // System.out.print(rowInfos[i].weight) ;
    // }
    // System.out.println("]") ;
    // cells.dump() ;
    // }

    /**
     * Lays out all the layout components.
     */
    public void layout(Container container) {
        int remainderX = container.getWidth() - width - globalInsets.left - globalInsets.right;
        int weightedRemainderX = (int) (remainderX * (1.0 - weightX));
        int remainderY = container.getHeight() - height - globalInsets.top - globalInsets.bottom;
        int weightedRemainderY = (int) (remainderY * (1.0 - weightY));
        if (log.isTraceEnabled()) log.trace("Layout from " + width + "x" + height + " to " + container.getWidth() + "x"
                + container.getHeight() + " (reminder " + remainderX + "x" + remainderY + ")");
        int nbrY = cells.getNbrRows();
        int nbrMaxX = cells.getMaxNbrColumns();
        // align the content of the layout
        int posX;
        int posY;
        switch (layoutAlignment) {
            case MultiBorderConstraints.CENTER:
                posX = globalInsets.left + weightedRemainderX / 2;
                posY = globalInsets.top + weightedRemainderY / 2;
                break;
            case MultiBorderConstraints.NORTH:
                posX = globalInsets.left + weightedRemainderX / 2;
                posY = globalInsets.top;
                break;
            case MultiBorderConstraints.NORTHEAST:
                posX = globalInsets.left + weightedRemainderX;
                posY = globalInsets.top;
                break;
            case MultiBorderConstraints.EAST:
                posX = globalInsets.left + weightedRemainderX;
                posY = globalInsets.top + weightedRemainderY / 2;
                break;
            case MultiBorderConstraints.SOUTHEAST:
                posX = globalInsets.left + weightedRemainderX;
                posY = globalInsets.top + weightedRemainderY;
                break;
            case MultiBorderConstraints.SOUTH:
                posX = globalInsets.left + weightedRemainderX / 2;
                posY = globalInsets.top + weightedRemainderY;
                break;
            case MultiBorderConstraints.SOUTHWEST:
                posX = globalInsets.left;
                posY = globalInsets.top + weightedRemainderY;
                break;
            case MultiBorderConstraints.WEST:
                posX = globalInsets.left;
                posY = globalInsets.top + weightedRemainderY / 2;
                break;
            default:
            case MultiBorderConstraints.NORTHWEST:
                posX = globalInsets.left;
                posY = globalInsets.top;
                break;
        }
        // Apply position X and weight X to columns
        for (int x = 0; x < nbrMaxX; x++) {
            if ((colInfos[x].getWeight() > 0.0) && (remainderX > 0)) {
                if (colInfos[x].getWeight() > 1.0) colInfos[x].setWeightedSpace(colInfos[x].getSpace()
                        + (int) (remainderX * (colInfos[x].getWeight() / weightX)));
                else colInfos[x].setWeightedSpace(colInfos[x].getSpace() + (int) (remainderX * colInfos[x].getWeight()));
            } else {
                colInfos[x].setWeightedSpace(colInfos[x].getSpace());
            }
            if (log.isTraceEnabled()) log.trace("  * Col " + x + ": weighted at " + colInfos[x].getWeight() + " from "
                    + colInfos[x].getSpace() + " to " + colInfos[x].getWeightedSpace());
            colInfos[x].setPos(posX);
            posX += colInfos[x].getLayedTotalSpace();
        }
        // Apply position Y and weights Y to rows
        for (int y = 0; y < nbrY; y++) {
            if ((rowInfos[y].getWeight() > 0.0) && (remainderY > 0)) {
                if (rowInfos[y].getWeight() > 1.0) {
                    rowInfos[y].setWeightedSpace(rowInfos[y].getSpace() + (int) (remainderY * (rowInfos[y].getWeight() / weightY)));
                } else {
                    rowInfos[y].setWeightedSpace(rowInfos[y].getSpace() + (int) (remainderY * rowInfos[y].getWeight()));
                }
            } else {
                rowInfos[y].setWeightedSpace(rowInfos[y].getSpace());
            }
            if (log.isTraceEnabled()) log.trace("  * Row " + y + ": weighted at " + rowInfos[y].getWeight() + " from "
                    + rowInfos[y].getSpace() + " to " + rowInfos[y].getWeightedSpace());
            rowInfos[y].setPos(posY);
            posY += rowInfos[y].getLayedTotalSpace();
        }
        // lays out cells
        for (int y = 0; y < nbrY; y++) {
            int nbrX = cells.getNbrColumns(y);
            for (int x = 0; x < nbrX; x++) {
                CellInfo cell = (CellInfo) cells.get(x, y);
                if ((cell != null) && cell.beginAt(x, y)) cell.layout(colInfos, rowInfos);
            }
        }
    }


    private void layoutBorders(Container container) {
        int len = bordersList.size();
        for (int i = 0; i < len; i++) {
            MultiBorder border = bordersList.get(i);
            border.layout(container);
        }
    }


    /**
     * Sets the reference of the container using this layout.
     * <p>
     * <b>Important:</b> the <code>MultiBorderLayout</code> manages borders itself, for doing that it adds here a invisible border
     * on the container. This border cannot be removed. <code>MultiBorderLayout</code> throws an IllegalArgumentException if other
     * borders are set with the <code>setBorder</code> method. Borders should be added using the <code>MultiBorderConstraints</code>
     * (with the <code>addBorder(MultiBorder)</code> method).
     * </p>
     * 
     * @see MultiBorderConstraints
     * @see MultiBorder
     */
    private void setLayoutContainer(Component cmp) {
        try {
            // check validity of container
            if (layoutContainer != null) throw new IllegalArgumentException("MultiBorderLayout can only be layout of one container");
            layoutContainer = (JComponent) cmp;
            // add the border adapter catching the paintBorder calls
            layoutContainer.setBorder(new BorderAdapter());
            layoutContainer.addPropertyChangeListener("border", new PropertyChangeListener() {

                public void propertyChange(PropertyChangeEvent evt) {
                    throw new IllegalArgumentException("You cannot change the border set by MultiBorderLayout");
                }
            });
            // add the current borders components
            int len = bordersList.size();
            for (int i = 0; i < len; i++) {
                MultiBorder border = bordersList.get(i);
                if (log.isDebugEnabled()) log.debug("setLayoutContainer Call addComponents on border: " + border);
                border.addExtraComponents(layoutContainer);
            }
        } catch (ClassCastException cce) {
            throw new IllegalArgumentException("MultiBorderLayout can only be used on JComponents");
        }
    }


    /**
     * Paint all the borders set on this MultiBorderLayout.
     */
    void paintBorders(Component cmp, Graphics g) {
        int len = bordersList.size();
        for (int i = 0; i < len; i++) {
            MultiBorder border = bordersList.get(i);
            border.paint(cmp, g);
        }
        if (debugMode) painCells(cmp, g);
    }


    private void painCells(Component cmp, Graphics g) {
        int nbrY = cells.getNbrRows();
        Color oldColor = g.getColor();
        Font oldFont = g.getFont();
        g.setColor(Color.RED);
        g.setFont(new Font("Dialog", Font.PLAIN, 8));
        for (int y = 0; y < nbrY; y++) {
            int nbrX = cells.getNbrColumns(y);
            for (int x = 0; x < nbrX; x++) {
                CellInfo cell = (CellInfo) cells.get(x, y);
                if ((cell != null) && cell.beginAt(x, y) && !cell.isCollapsed()) {
                    int posX = colInfos[x].getLayedStart();
                    int posY = rowInfos[y].getLayedStart();
                    int drawWidth = getUsedSpace(colInfos, x, x + cell.getGridWith() - 1);
                    int drawHeight = getUsedSpace(rowInfos, y, y + cell.getGridHeight() - 1);
                    g.drawRect(posX, posY, drawWidth, drawHeight);
                    if ((drawWidth > 12) && (drawHeight > 10)) {
                        String cellPos = "" + cell.getGridX() + "-" + cell.getGridY();
                        g.drawString(cellPos, posX + 2, posY + 10);
                    }
                }
            }
        }
        g.setFont(oldFont);
        g.setColor(oldColor);
    }


    private int getUsedSpace(LineInfo[] infos, int from, int to) {
        int space = 0;
        for (int i = from; i <= to; i++) {
            if (i != from) space += infos[i].getBeforeSpace();
            space += infos[i].getWeightedSpace();
            if (i != to) space += infos[i].getAfterSpace();
        }
        return space;
    }

    // ------------------- Inner Class to be notified when borders need to be painted --------------------------------------

    /**
     * This class is a hack to be notified by the container when the borders need to be painted. By adding this <code>Border</code>
     * on the container the <code>paintBorder(Graphics g)</code> method of the container will finally call the
     * <code>paintBorders(Component cmp, Graphics g)</code> method of the <code>MultiBorderLayout</code>.
     */
    class BorderAdapter implements Border {

        public void paintBorder(Component c, Graphics g, int x, int y, int width1, int height1) {
            paintBorders(c, g);
        }


        public Insets getBorderInsets(Component c) {
            return new Insets(10, 10, 10, 10);
        }


        public boolean isBorderOpaque() {
            return false;
        }

    }

}
