/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.table;


import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;

import net.sf.sfac.setting.Settings;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Table column model allowing to show/hide selected columns.
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class HidingTableColumnModel extends DefaultTableColumnModel {

    private static Log log = LogFactory.getLog(HidingTableColumnModel.class);

    private static final String KEY_PREFIX = "ui.table.";
    private static final String COLS_POSTFIX = ".columns";
    private static final String WIDTHS_POSTFIX = ".widths";

    private List<TableColumn> allColumns;

    private Settings setts;
    private String columnsSettingKey;
    private String widthsSettingKey;
    private PropertyChangeListener widthListener;

    private JDialog hidingSelectionDialog;
    private JCheckBox[] hidingCheckBoxes;


    public HidingTableColumnModel() {
        allColumns = new ArrayList<TableColumn>();
        widthListener = new PropertyChangeListener() {

            public void propertyChange(PropertyChangeEvent evt) {
                saveConfigSettings();
            }
        };
    }


    public void useSettings(Settings settngs, String id) {
        setts = settngs;
        // register to new settings
        if (setts != null) {
            columnsSettingKey = KEY_PREFIX + id + COLS_POSTFIX;
            widthsSettingKey = KEY_PREFIX + id + WIDTHS_POSTFIX;
            // set the current values from settings
            int[] visibleColumns = setts.getIntArrayProperty(columnsSettingKey, null);
            if (visibleColumns != null) setVisibleColumns(visibleColumns);
            int[] columnSizes = setts.getIntArrayProperty(widthsSettingKey, null);
            if (columnSizes != null) setColumnSizes(columnSizes);
        }
    }


    void saveConfigSettings() {
        if (setts != null) {
            setts.setIntArrayProperty(columnsSettingKey, getVisibleColumns());
            setts.setIntArrayProperty(widthsSettingKey, getColumnSizes());
        }
    }


    public void showSelectionDialog(Component cmp) {
        JDialog dial = getSelectionDialog(cmp);
        refreshSelectionCheckBoxes();
        Point p = new Point(20, 20);
        SwingUtilities.convertPointToScreen(p, cmp);
        if (p.x < 0) p.x = 0;
        if (p.y < 0) p.y = 0;
        dial.setLocation(p);
        dial.setVisible(true);
    }


    public void addDefaultPopupMenuItems(JPopupMenu menu, final JTableHeader header) {
        if (menu.getComponentCount() > 0) menu.addSeparator();
        JMenuItem selectCols = new JMenuItem("Select Columns...");
        selectCols.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                showSelectionDialog(header);
            }
        });
        menu.add(selectCols);
    }


    private JDialog getSelectionDialog(Component cmp) {
        if (hidingSelectionDialog == null) {
            hidingSelectionDialog = new JDialog(JOptionPane.getFrameForComponent(cmp), "Column selection", true);
            int nbrCols = allColumns.size();
            JPanel checkBoxPanel = new JPanel(new GridLayout(nbrCols, 1));
            checkBoxPanel.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
            hidingCheckBoxes = new JCheckBox[nbrCols];
            for (int i = 0; i < nbrCols; i++) {
                TableColumn col = allColumns.get(i);
                Object header = col.getHeaderValue();
                hidingCheckBoxes[i] = new JCheckBox((header == null) ? "" : header.toString());
                checkBoxPanel.add(hidingCheckBoxes[i]);
            }
            JPanel buttonPane = new JPanel();
            JButton okButton = new JButton("Ok");
            okButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    commitAndClose();
                }
            });
            buttonPane.add(okButton);
            JButton cancelButton = new JButton("Cancel");
            cancelButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    getHidingSelectionDialog().setVisible(false);
                }
            });
            buttonPane.add(cancelButton);
            hidingSelectionDialog.getContentPane().add(checkBoxPanel, BorderLayout.CENTER);
            hidingSelectionDialog.getContentPane().add(buttonPane, BorderLayout.SOUTH);
            hidingSelectionDialog.pack();
        }
        return hidingSelectionDialog;
    }


    JDialog getHidingSelectionDialog() {
        return hidingSelectionDialog;
    }


    private void refreshSelectionCheckBoxes() {
        int nbrCols = allColumns.size();
        for (int i = 0; i < nbrCols; i++) {
            TableColumn col = allColumns.get(i);
            // To Do: can be optimized by using the modelIndex of the tableColumn
            hidingCheckBoxes[i].setSelected(tableColumns.contains(col));
        }
    }


    void commitAndClose() {
        // avoid to hide all the columns
        int len = hidingCheckBoxes.length;
        int nbrShown = 0;
        for (int i = 0; i < len; i++) {
            if (hidingCheckBoxes[i].isSelected()) nbrShown++;
        }
        if (nbrShown == 0) {
            JOptionPane.showMessageDialog(hidingSelectionDialog, "You cannot hide all the columns", "Wrong selection",
                    JOptionPane.ERROR_MESSAGE);
        } else {
            // set the columns shown/hidden
            hidingSelectionDialog.setVisible(false);
            for (int i = 0; i < len; i++) {
                TableColumn col = allColumns.get(i);
                if (hidingCheckBoxes[i].isSelected()) showColumn(col);
                else hideColumn(col);
            }
        }
    }


    public TableColumn getBaseColumn(int columnModelIndex) {
        return allColumns.get(columnModelIndex);
    }


    public void showColumn(TableColumn aColumn) {
        if (!tableColumns.contains(aColumn)) {
            super.addColumn(aColumn);
            saveConfigSettings();
        }
    }


    public void hideColumn(TableColumn aColumn) {
        if (tableColumns.contains(aColumn)) {
            super.removeColumn(aColumn);
            saveConfigSettings();
        }
    }


    @Override
    public void addColumn(TableColumn aColumn) {
        // if this method is called, column number is updated from the outside.
        // Most likely, a new column set is added after all previous columns have been removed.
        // In this case we consider that all the columns are shown
        // and we have to resynchronize with supreclass content
        synchWithSuperclass();
        // add the column
        aColumn.removePropertyChangeListener(widthListener);
        aColumn.addPropertyChangeListener(widthListener);
        allColumns.add(aColumn);
        hidingSelectionDialog = null;
        super.addColumn(aColumn);
    }


    @Override
    public void removeColumn(TableColumn column) {
        // if this method is called, column number is updated from the outside.
        // Most likely, all columns will be removed.
        // In this case, as the tables knows only the shown columns,
        // we have first to resynchronize with supreclass content
        synchWithSuperclass();
        // remove the column
        column.removePropertyChangeListener(widthListener);
        boolean removed = allColumns.remove(column);
        if (removed) {
            if (log.isDebugEnabled()) log.debug("Remove column " + column.getHeaderValue() + " = " + column.getModelIndex());
        } else {
            log.warn("Column not found " + column.getHeaderValue() + " = " + column.getModelIndex());
        }
        hidingSelectionDialog = null;
        super.removeColumn(column);
    }


    /**
     * Synchronize the total set of column of this class with the displayed set of columns of the super class. It means that all the
     * hidden columns will be lost. It's not a problem since this method is called when the column set is replaced, and in this case
     * (during the change) there should be no hidden column.
     */
    private void synchWithSuperclass() {
        int totalNbrCols = allColumns.size();
        int superNbrCols = getColumnCount();
        if (totalNbrCols != superNbrCols) {
            allColumns.clear();
            for (int i = 0; i < superNbrCols; i++)
                allColumns.add(getColumn(i));
        }
    }


    public int[] getVisibleColumns() {
        int nbrCols = getColumnCount();
        int[] cols = new int[nbrCols];
        TableColumn col;
        for (int i = 0; i < nbrCols; i++) {
            col = getColumn(i);
            cols[i] = col.getModelIndex();
        }
        if (log.isDebugEnabled()) log.debug("Current visible table columns: : " + Arrays.toString(cols));
        return cols;
    }


    public int[] getColumnSizes() {
        int nbrCols = getColumnCount();
        int[] sizes = new int[nbrCols];
        TableColumn col;
        for (int i = 0; i < nbrCols; i++) {
            col = getColumn(i);
            sizes[i] = col.getPreferredWidth();
        }
        if (log.isDebugEnabled()) log.debug("Current table columns sizes: " + Arrays.toString(sizes));
        return sizes;
    }


    public void setVisibleColumns(int[] cols) {
        int len = (cols == null) ? 0 : cols.length;
        int nbrCols = allColumns.size();
        if (len > 0) {
            if (log.isDebugEnabled()) log.debug("Restore visible table columns: " + Arrays.toString(cols));
            // first remove all columns
            while (getColumnCount() > 0) {
                super.removeColumn(getColumn(0));
            }
            // re-add the good ones
            TableColumn col;
            for (int i = 0; i < len; i++) {
                if (cols[i] < nbrCols) {
                    col = allColumns.get(cols[i]);
                    super.addColumn(col);
                }
            }
        }
    }


    public void setColumnSizes(int[] sizes) {
        int len = (sizes == null) ? 0 : sizes.length;
        if (len > 0) {
            if (log.isDebugEnabled()) log.debug("Restore table columns sizes: " + Arrays.toString(sizes));
            if (len > getColumnCount()) {
                log.warn("Wrong number of saved column size: " + len + " instead of " + getColumnCount());
                len = getColumnCount();
            }
            TableColumn col;
            for (int i = 0; i < len; i++) {
                col = getColumn(i);
                col.setPreferredWidth(sizes[i]);
            }
        }
    }

}
