/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.table;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import net.sf.sfac.setting.Settings;


/**
 * Chained table model performing row sort.
 * <p>
 * This class will associate a <code>java.util.Comparator</code> to each column to allow sorting. It will provide a default
 * comparator for java basic types (String, Boolean, Integer, Date, Double, ...) and all classes implementing the
 * <code>java.lang.Comparable</code> interface. <br>
 * For other classes, the sort is disabled by default but the developer can provide a custom <code>Comparator</code> with the
 * <code>setComparatorForColumn(Comparator, int)</code> or <code>setDefaultComparatorForClass(Comparator, Class)</code> methods.
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class SortModel extends AbstractTableModel implements TableModelListener {

    private static final String KEY_PREFIX = "ui.table.";
    private static final String SORT_POSTFIX = ".sort";
    /** Type of column sort: column not sorted. */
    public static final int NO_SORT = 0;
    /** Type of column sort: column sorted in default order (of the column comparator). */
    public static final int SORT_NORMAL = 1;
    /** Type of column sort: column sorted in reverse order (of the column comparator). */
    public static final int SORT_REVERSE = -1;

    /** Comparator used to compare the string and objects not implementing the <code>java.lang.Comparable</code> interface. */
    private static final Comparator<Object> STRING_COMPARATOR = new StringComparator();
    /** Comparator used to compare the objects implementing the <code>java.lang.Comparable</code> interface. */
    private static final Comparator<Object> COMPARABLE_COMPARATOR = new ComparableComparator();
    /** Comparator used to compare Boolean objects. */
    private static final Comparator<Boolean> BOOLEAN_COMPARATOR = new BooleanComparator();
    /** Comparator used to compare table rows (using the order defined in this class instance). */
    private final Comparator<Integer> ROW_COMPARATOR = new RowComparator();

    /** The chained model (from which the rows are sorted by this model). */
    private TableModel chainedModel;
    /** Global flag allowing to enable/disable the sort. */
    private boolean sortingEnabled;
    /** The comparators associated to each column (null comparator means that the column cannot be sorted). */
    private Comparator<?>[] columnSorters;
    /** The indexes of the sorted columns (null array meaning no sort). */
    private int[] sortColumnIndexes;
    /** List of Integer for mapping row indexes (this model row -> chained model row). */
    private List<Integer> modelIndexes;
    /** Array of int for mapping row indexes (chained model row -> this model row). */
    private int[] viewIndexes;
    /** Map holding custom association between column Class and column Comparator (if any). */
    private Map<Class<?>, Comparator<?>> defaultSorterMap;
    /** Reference to all the header component (JTableHeader) of tables using this model. */
    private List<JTableHeader> tableHeaders;
    /** Mouse listener added to all table headers. */
    private MouseListener tableHeaderListener;
    /** Mouse listener added to all TableColumnModel. */
    private TableColumnModelListener columnModelListener;

    private Settings setts;
    private String sortSettingKey;


    /**
     * Build a SortModel sorting the rows of the given model.
     * 
     * @param chainedTableModel
     *            chained TableModel.
     */
    public SortModel(TableModel chainedTableModel) {
        modelIndexes = new ArrayList<Integer>();
        tableHeaders = new ArrayList<JTableHeader>();
        sortingEnabled = true;
        setChainedModel(chainedTableModel);
        buildListeners();
    }


    public void useSettings(Settings settngs, String id) {
        setts = settngs;
        // register to new settings
        if (setts != null) {
            sortSettingKey = KEY_PREFIX + id + SORT_POSTFIX;
            // set the current values from settings
            int[] sort = setts.getIntArrayProperty(sortSettingKey, null);
            if (sort != null) setSortColumnIndexes(sort);
        }
    }


    /**
     * Check if the sorting is currently enabled. If sorting is disabled, all the sort settings will be ignored and so, this model
     * will be totally transparent.
     * 
     * @return true iff the sorting is currently enabled.
     */
    public boolean isSortingEnabled() {
        return sortingEnabled;
    }


    /**
     * Enable/disable the sorting. If sorting is disabled, this model will be totally transparent. Else the rows of the chained
     * model will be sorted according the current sort settings.
     * 
     * @param enabled
     *            true to enable sorting, false to disable it.
     */
    public void setSortingEnabled(boolean enabled) {
        if (sortingEnabled != enabled) {
            sortingEnabled = enabled;
            sortConfigChanged();
        }
    }


    /**
     * Get the comparator currently used to sort the given column.
     * 
     * @param column
     *            the column index.
     * @return the comparator currently used to sort the given column.
     */
    public Comparator<?> getSorterForColumn(int column) {
        return columnSorters[column];
    }


    /**
     * Get the number of sorted column.
     * 
     * @return the number of sorted column.
     */
    public int getNumberOfSortedColumns() {
        return sortingEnabled ? getNbrSortColumn() : 0;
    }


    /**
     * Check if the given cloumn is currently sorted.
     * 
     * @param column
     *            the column index.
     * @return true iff the given cloumn is currently sorted.
     */
    public boolean isColumnSorted(int column) {
        return sortingEnabled && isColumnSortedImpl(column);
    }


    /**
     * Get the sort direction of the given column. the result will be one of the three constant defined in this class:
     * <ul>
     * <li>NO_SORT : column is not sorted.
     * <li>SORT_NORMAL : column is sorted in the default comparator order.
     * <li>SORT_REVERSE : column is sorted in the reverse comparator order.
     * </ul>
     * 
     * @param column
     *            the column index.
     * @return the sort direction of the given column, one of : NO_SORT, SORT_NORMAL, SORT_REVERSE
     */
    public int getSortDirection(int column) {
        if (sortingEnabled) {
            int len = getNbrSortColumn();
            for (int i = 0; i < len; i++) {
                if (getSortColumnAt(i) == column) return getSortDirectionAt(i);
            }
        }
        return NO_SORT;
    }


    /**
     * Get the index of given column in the list of sorted column. This method returns -1 if the given column is not sorted.
     * 
     * @param column
     *            the column index
     * @return the index of given column in the list of sorted column.
     */
    public int getSortIndex(int column) {
        if (sortingEnabled) {
            int len = getNbrSortColumn();
            for (int i = 0; i < len; i++) {
                if (getSortColumnAt(i) == column) return i;
            }
        }
        return -1;
    }


    /**
     * Set the list of the sorted column to (a singleton containing) the given column. If the given column is already the only
     * sorted column the sort direction is inverted.
     * 
     * @param column
     *            the column index.
     */
    public void setSortColumn(int column) {
        if (columnSorters[column] != null) {
            int nbrIndexes = getNbrSortColumn();
            if (nbrIndexes != 1) {
                sortColumnIndexes = new int[] { column };
            } else {
                if (getSortColumnAt(0) == column) invertSortColumnDirectionAt(0);
                else sortColumnIndexes[0] = column;
            }
            sortConfigChanged();
        }
    }


    /**
     * Add the given column to the list of sorted columns. If the list of sorted columns already contains the given column, the
     * column is not added but it's sort direction is inverted.
     * 
     * @param column
     *            the column index.
     */
    public void addSortColumn(int column) {
        if (columnSorters[column] != null) {
            int nbrIndexes = getNbrSortColumn();
            if (nbrIndexes == 0) {
                // set main index
                sortColumnIndexes = new int[] { column };
            } else if (isColumnSortedImpl(column)) {
                // reverse the sort on existing index
                for (int i = 0; i < nbrIndexes; i++) {
                    if (getSortColumnAt(i) == column) {
                        invertSortColumnDirectionAt(i);
                        break;
                    }
                }
            } else {
                // add secondary index
                int[] newSortIndexes = new int[nbrIndexes + 1];
                System.arraycopy(sortColumnIndexes, 0, newSortIndexes, 0, nbrIndexes);
                newSortIndexes[nbrIndexes] = column;
                sortColumnIndexes = newSortIndexes;
            }
            sortConfigChanged();
        }
    }


    public void setSortColumnIndexes(int[] newIndexes) {
        sortColumnIndexes = newIndexes;
        sortConfigChanged();
    }


    public int[] getSortColumnIndexes() {
        return (sortColumnIndexes == null) ? null : (int[]) sortColumnIndexes.clone();
    }


    /**
     * Remove the given column from the list of sorted columns. (do nothing if the column was not in the list of sorted columns)
     * 
     * @param column
     *            the column to remove from the list of sorted columns.
     */
    public void removeSortColumn(int column) {
        int removedIndex = getSortIndex(column);
        if (sortingEnabled && (removedIndex >= 0)) {
            int nbrIndexes = getNbrSortColumn();
            if (nbrIndexes <= 1) {
                sortColumnIndexes = null;
            } else {
                // remove the column
                int[] newSortIndexes = new int[nbrIndexes - 1];
                System.arraycopy(sortColumnIndexes, 0, newSortIndexes, 0, removedIndex);
                System.arraycopy(sortColumnIndexes, removedIndex + 1, newSortIndexes, removedIndex, nbrIndexes - removedIndex - 1);
                sortColumnIndexes = newSortIndexes;
            }
            sortConfigChanged();
        }
    }


    /**
     * Reset the list of sorted column.
     */
    public void resetSorting() {
        int nbrIndexes = getNbrSortColumn();
        if (nbrIndexes > 0) {
            sortColumnIndexes = null;
            if (sortingEnabled) sortConfigChanged();
        }
    }


    /**
     * Check if this model is currently sorting.
     * 
     * @return true iff this model is currently sorting.
     */
    public boolean isSorting() {
        return (sortingEnabled) ? (getNbrSortColumn() > 0) : false;
    }


    /**
     * Associate a specific comparator to a column. Setting null as comparator will disable the sort on the column. Note that
     * comparators are stateless objects, so the same comparator can be associated to more that one column.
     * <p>
     * The Comparator/column association set here will be discarded if this model receive a 'TableStructureChanged' event form
     * chained model or if setDefaultComparatorForClass(..) is called for the column class. In this case, a default comparator is
     * associated to each (new) column.
     * <p>
     * Calling this method reset any current sorting.
     * 
     * @param sorter
     *            The comparator to associate to a column (null to disable column sort)
     * @param column
     *            the column index.
     * @see #setDefaultComparatorForClass
     */
    public void setComparatorForColumn(Comparator<?> sorter, int column) {
        columnSorters[column] = sorter;
        resetSorting();
    }


    /**
     * Associate a specific comparator to a class. Each time this model will associate comparators to columns (at startup and after
     * each 'TableStructureChanged' event), it will use the specified comparator. The TableModel column class be of the exact class
     * specified here (and not a sub-class or Interface implementation) to be associated with the given comparator.
     * <p>
     * Columns of the specified class will be directly associated to the given comparator. So it will discard associations
     * previously set with <code>setComparatorForColumn(..)</code> for any column of this class.
     * <p>
     * You can pass a null comparator value to disable sorting on columns of the specifed class.
     * <p>
     * Note that comparators are stateless objects, so the same comparator can be associated to more that one class.
     * <p>
     * Calling this method will reset any current sorting.
     * 
     * @param sorter
     *            The comparator to associate to a column class (null to disable column sort)
     * @param cls
     *            The class associated to the comparator.
     * @see #setComparatorForColumn
     */
    public void setDefaultComparatorForClass(Comparator<?> sorter, Class<?> cls) {
        if (defaultSorterMap == null) defaultSorterMap = new HashMap<Class<?>, Comparator<?>>(10);
        defaultSorterMap.put(cls, sorter);
        // directly apply the new association
        int nbrCols = getColumnCount();
        for (int i = 0; i < nbrCols; i++) {
            if (cls == getColumnClass(i)) columnSorters[i] = sorter;
        }
        resetSorting();
    }


    /**
     * Register all necassary listener to the table objects, so this model can react to click on column headers and display arrows
     * for sort direction in column headers.
     * <p>
     * More than one table can be registerd to this model.
     * 
     * @param table
     *            the table using this model.
     */
    public void registerToTableColumns(JTable table) {
        TableColumnModel cm = table.getColumnModel();
        cm.addColumnModelListener(columnModelListener);
        setTableColumnHeaderRenderers(cm, 0, cm.getColumnCount() - 1);
        JTableHeader tableHeader = table.getTableHeader();
        tableHeader.addMouseListener(tableHeaderListener);
        tableHeaders.add(tableHeader);
    }


    /**
     * Unregister all the listeners set in the <code>registerToTableColumns(JTable)</code> method.
     * 
     * @param table
     *            the table no more using this model.
     * @see #registerToTableColumns
     */
    public void unregisterFormTableColumns(JTable table) {
        TableColumnModel cm = table.getColumnModel();
        cm.removeColumnModelListener(columnModelListener);
        int len = cm.getColumnCount();
        for (int i = 0; i < len; i++) {
            TableColumn col = cm.getColumn(i);
            if (col != null) {
                TableCellRenderer headerRenderer = col.getHeaderRenderer();
                if (headerRenderer instanceof SortHeaderRenderer) {
                    SortHeaderRenderer sorterRenderer = (SortHeaderRenderer) headerRenderer;
                    sorterRenderer.setSortModel(null);
                }
            }
        }
        JTableHeader tableHeader = table.getTableHeader();
        tableHeader.removeMouseListener(tableHeaderListener);
        tableHeaders.remove(tableHeader);
    }


    // /////// private methods //////////////////////////////////////////////////////////////////

    /**
     * Get the number of sorted columns.
     * <p>
     * Unlike the <code>getNumberOfSortedColumns()</code> method, this method does not check if the sorting is enabled or not.
     * 
     * @return the number of sorted columns.
     */
    int getNbrSortColumn() {
        return (sortColumnIndexes == null) ? 0 : sortColumnIndexes.length;
    }


    /**
     * Get the column index at specifed index in the sorted column list.
     * <p>
     * The sort direction is coded with the column index in a single integer:
     * <ul>
     * <li>column n, normal direction -> value = n
     * <li>column n, reverse direction -> value = -(n+1)
     * </ul>
     * 
     * @param index
     *            the index in the sorted column list.
     * @return the index of the sorted column (in the model)
     */
    int getSortColumnAt(int index) {
        int column = sortColumnIndexes[index];
        if (column < 0) column = -(column + 1);
        return column;
    }


    /**
     * Get the sorted column direction at specifed index in the sorted column list.
     * <p>
     * The sort direction is coded with the column index in a single integer:
     * <ul>
     * <li>column n, normal direction -> value = n
     * <li>column n, reverse direction -> value = -(n+1)
     * </ul>
     * 
     * @param index
     *            the index in the sorted column list.
     * @return the the sorted column direction, one of SORT_REVERSE or SORT_NORMAL.
     */
    int getSortDirectionAt(int index) {
        return (sortColumnIndexes[index] < 0) ? SORT_REVERSE : SORT_NORMAL;
    }


    /**
     * Invert the sorted column direction at specifed index in the sorted column list.
     * <p>
     * The sort direction is coded with the column index in a single integer:
     * <ul>
     * <li>column n, normal direction -> value = n
     * <li>column n, reverse direction -> value = -(n+1)
     * </ul>
     * 
     * @param index
     *            the index in the sorted column list.
     */
    private void invertSortColumnDirectionAt(int index) {
        int column = sortColumnIndexes[index];
        sortColumnIndexes[index] = -(column + 1);
    }


    /**
     * Check if the given column is currently sorted.
     * <p>
     * Unlike the <code>isColumnSorted(int)</code> method, this method does not check if the sorting is enabled or not.
     * 
     * @param col
     *            the column index.
     * @return true iff the given column is currently sorted.
     */
    private boolean isColumnSortedImpl(int col) {
        int nbrIndexes = getNbrSortColumn();
        for (int i = 0; i < nbrIndexes; i++) {
            if (getSortColumnAt(i) == col) return true;
        }
        return false;
    }


    /**
     * Notification that the sort configuration has changed. This method will re-sort the data and fire appropriate events to
     * listeners.
     */
    private void sortConfigChanged() {
        sort();
        fireTableDataChanged();
        notifyTableHeaders();
        if (setts != null) {
            setts.setIntArrayProperty(sortSettingKey, getSortColumnIndexes());
        }
    }


    /**
     * Get the comparator associated to the given class. This method will first look in custom associations and if nothing is found
     * there, it will associate default comparator following thes rules:
     * <ul>
     * <li>String class : user default <code>Collator</code>
     * <li>Boolean class : use <code>BooleanComparator</code> inner class.
     * <li>intances of <code>Comparable</code> : use <code>ComparableComparator</code> inner class.
     * <li>other : no comparator (sort disabled on this column)
     * </ul>
     * 
     * @param cls
     *            The class for witch we want a comparator.
     * @return the comparator associated to the given class, null if none.
     */
    private Comparator<?> getComparatorForClass(Class<?> cls) {
        Comparator<?> sorter = null;
        if ((defaultSorterMap != null) && (defaultSorterMap.containsKey(cls))) {
            sorter = defaultSorterMap.get(cls);
        } else {
            if (cls == String.class) sorter = STRING_COMPARATOR;
            else if (cls == Boolean.class) sorter = BOOLEAN_COMPARATOR;
            else if (Comparable.class.isAssignableFrom(cls)) sorter = COMPARABLE_COMPARATOR;
            else sorter = STRING_COMPARATOR;
        }
        return sorter;
    }


    /**
     * Initialize the sorters for new model or after a TableStructureChanged event.
     */
    private void initComparators() {
        int nbrCols = getColumnCount();
        columnSorters = new Comparator[nbrCols];
        for (int i = 0; i < nbrCols; i++) {
            Class<?> columnClass = getColumnClass(i);
            columnSorters[i] = getComparatorForClass(columnClass);
        }
        sortColumnIndexes = null;
    }


    /**
     * Sort the rows according the current sort settings.
     * <p>
     * Note: don't forget to fire a TableModel event after calling this method
     */
    private void sort() {
        int nbrIndexes = getNbrSortColumn();
        checkArraySizes();
        if (sortingEnabled && (nbrIndexes > 0)) Collections.sort(modelIndexes, ROW_COMPARATOR);
        else Collections.sort(modelIndexes);
        int len = modelIndexes.size();
        for (int i = 0; i < len; i++)
            viewIndexes[getChainedModelRow(i)] = i;
    }


    /**
     * Check the size of internal arrays (<code>modelIndexes</code> and <code>viewIndexes</code>) holding row mapping to see if they
     * are large enough (compared to the number of rows in chained model).
     */
    private void checkArraySizes() {
        int nbrRows = chainedModel.getRowCount();
        // update viewIndexes vector size.
        // rem: always called from sort, so the correct values will be set in this new array later.
        if ((viewIndexes == null) || (viewIndexes.length < nbrRows)) viewIndexes = new int[nbrRows + 10];
        // update modelIndexes
        int currentModelNbr = modelIndexes.size();
        if (nbrRows != currentModelNbr) {
            if (currentModelNbr > nbrRows) {
                // remove rows
                Collections.sort(modelIndexes);
                int nbrToRemove = currentModelNbr - nbrRows;
                for (int i = 1; i <= nbrToRemove; i++)
                    modelIndexes.remove(currentModelNbr - i);
            } else {
                // add rows
                for (int i = currentModelNbr; i < nbrRows; i++) {
                    modelIndexes.add(Integer.valueOf(i));
                }
            }
        }
    }


    /**
     * Check if the renderers of the specifed column of the given TableColumnModel are well instance of SortFilterHeaderRenderer (if
     * not replace them by a new SortFilterHeaderRenderer). So they can display an arrow for sort notification. Also register any
     * existing SortFilterHeaderRenderer so it uses this SortModel.
     * 
     * @param cm
     *            TableColumnModel of a table using this model.
     * @param fromIndex
     *            start index of columns to check
     * @param toIndex
     *            end index of columns to check (inclusive)
     */
    void setTableColumnHeaderRenderers(TableColumnModel cm, int fromIndex, int toIndex) {
        for (int i = fromIndex; i <= toIndex; i++) {
            TableColumn col = cm.getColumn(i);
            if (col != null) {
                int thisModelIndex = col.getModelIndex();
                TableCellRenderer headerRenderer = col.getHeaderRenderer();
                if ((headerRenderer != null) && (headerRenderer instanceof SortHeaderRenderer)) {
                    SortHeaderRenderer sorterRenderer = (SortHeaderRenderer) headerRenderer;
                    sorterRenderer.setSortModel(this);
                } else {
                    col.setHeaderRenderer(new SortHeaderRenderer((thisModelIndex < 0) ? null : this, thisModelIndex));
                }
            }
        }
    }


    /**
     * Notify all JTableHeaders using this model to repaint themselve because the sort settings have changed (and so the displayed
     * arrows indicating sort order should change).
     */
    private void notifyTableHeaders() {
        int len = tableHeaders.size();
        for (int i = 0; i < len; i++) {
            JTableHeader header = tableHeaders.get(i);
            header.repaint();
        }
    }


    /**
     * Create instances of listeners to add on the JTableHeaders and TableColumnModelListener. This is only done once when class is
     * created.
     */
    private void buildListeners() {
        // create MouseListener for JTableHeaders
        tableHeaderListener = new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {
                mouseClickedInHeader(e);
            }
        };
        // create TableColumnModelListener for TableColumnModels
        columnModelListener = new TableColumnModelListener() {

            public void columnAdded(TableColumnModelEvent e) {
                TableColumnModel cm = (TableColumnModel) e.getSource();
                setTableColumnHeaderRenderers(cm, e.getFromIndex(), e.getToIndex());
            }


            public void columnRemoved(TableColumnModelEvent e) {
            }


            public void columnMoved(TableColumnModelEvent e) {
            }


            public void columnMarginChanged(ChangeEvent e) {
            }


            public void columnSelectionChanged(ListSelectionEvent e) {
            }
        };
    }


    /**
     * Invoked when the mouse has been clicked on a JTableHeader. It will update the sorted column list accordingly:
     * <ul>
     * <li>Left click : setSortColumn(clicked column)
     * <li>SHIFT+Left click : addSortColumn(clicked column)
     * <li>CTRL+Left click : removeSortColumn(clicked column)
     * </ul>
     * 
     * @param e
     *            the MouseEvent
     * @see #setSortColumn
     * @see #addSortColumn
     */
    void mouseClickedInHeader(MouseEvent e) {
        if ((e.getClickCount() == 1) && ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)) {
            JTableHeader tableHeader = (JTableHeader) e.getSource();
            int columnModelIndex = getColumnModelIndex(tableHeader, e);
            if (columnModelIndex >= 0) {
                boolean shift = (e.getModifiers() & MouseEvent.SHIFT_MASK) != 0;
                boolean control = (e.getModifiers() & MouseEvent.CTRL_MASK) != 0;
                if (shift) addSortColumn(columnModelIndex);
                else if (control) removeSortColumn(columnModelIndex);
                else setSortColumn(columnModelIndex);
            }
        }
    }


    /**
     * Get the model index of a column clicked in the given table header.
     * 
     * @param tableHeader
     *            the table header
     * @param e
     *            a mouse-click event on the table header
     * @return the model index of a column clicked in the table header.
     */
    private int getColumnModelIndex(JTableHeader tableHeader, MouseEvent e) {
        int columnModelIndex = -1;
        TableColumnModel columnModel = tableHeader.getColumnModel();
        int index = columnModel.getColumnIndexAtX(e.getX());
        if (index >= 0) {
            TableColumn column = columnModel.getColumn(index);
            columnModelIndex = column.getModelIndex();
        }
        return columnModelIndex;
    }


    /**
     * Add menu items to control table sorting in the given popup menu. The menu items added will depend of the sort state of this
     * model (and of the clicked column). <br>
     * Possible added menu items depening of the state are:
     * <ul>
     * <li>No table column sorted: <br>
     * 'Sort This Column' <br>
     * <li>Some table column sorted but clicked column not sorted: <br>
     * 'Sort This Column' <br>
     * 'Add Column To Sort' <br>
     * 'Clear Sort' <br>
     * <li>Only clicked column sorted: <br>
     * 'Invert Sort Order' <br>
     * 'Clear Sort' <br>
     * <li>Some (other) table column sorted and clicked column sorted: <br>
     * 'Sort This Column' <br>
     * 'Invert Sort Order' <br>
     * 'Clear Sort' <br>
     * </ul>
     * Note: if sorting is diabled, no menu items are added.
     * 
     * @param menu
     *            The menu where menu items are added.
     * @param tableHeader
     *            The table header where mouse was (right) clicked.
     * @param mouseEvt
     *            The (right) mouse click event triggering the popup menu.
     */
    public void addDefaultPopupMenuItems(JPopupMenu menu, JTableHeader tableHeader, MouseEvent mouseEvt) {
        if (isSortingEnabled()) {
            if (menu.getComponentCount() > 0) menu.addSeparator();
            final int columnModelIndex = getColumnModelIndex(tableHeader, mouseEvt);
            boolean columnSorted = isColumnSortedImpl(columnModelIndex);
            // add sort on this column
            if ((getNumberOfSortedColumns() != 1) || !columnSorted) {
                JMenuItem sortThis = new JMenuItem("Sort This Column");
                sortThis.addActionListener(new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        setSortColumn(columnModelIndex);
                    }
                });
                menu.add(sortThis);
            }
            if (columnSorted) {
                JMenuItem invertOrder = new JMenuItem("Invert Sort Order");
                invertOrder.addActionListener(new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        addSortColumn(columnModelIndex);
                    }
                });
                menu.add(invertOrder);
            } else if (getNumberOfSortedColumns() > 0) {
                JMenuItem addToSort = new JMenuItem("Add Column To Sort");
                addToSort.addActionListener(new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        addSortColumn(columnModelIndex);
                    }
                });
                menu.add(addToSort);
            }
            if (getNumberOfSortedColumns() > 0) {
                JMenuItem clearSort = new JMenuItem("Clear Sort");
                clearSort.addActionListener(new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        resetSorting();
                    }
                });
                menu.add(clearSort);
            }
        }
    }


    // -------------------- ChainedTableModel implementation ----------------------------------------

    /**
     * Set a new chained Model. This method will un-register all listeners from the old model and register to the new one.
     * 
     * @param chainedTableModel
     *            the new the chained Model.
     */
    public void setChainedModel(TableModel chainedTableModel) {
        if (chainedTableModel == null) throw new IllegalArgumentException("Chained table model cannot be null");
        // de-register from old chained model
        if (chainedModel != null) chainedModel.removeTableModelListener(this);
        // register to new chained model
        chainedModel = chainedTableModel;
        chainedModel.addTableModelListener(this);
        initComparators();
        sort();
        fireTableStructureChanged();
    }


    /**
     * Get the current chained Model.
     * 
     * @return the current chained Model.
     */
    public TableModel getChainedModel() {
        return chainedModel;
    }


    public Comparator<?>[] getColumnSorters() {
        return columnSorters;
    }


    /**
     * Get the index of the row in the chained model corresponding to the given sorted row index (in this model). If sorting is
     * disabled, the returned row is the same as the parameter row.
     * 
     * @param row
     *            a row index in this model
     * @return the corresponding row index in the chained model.
     */
    public int getChainedModelRow(int row) {
        return modelIndexes.get(row).intValue();
    }


    /**
     * Get the row of this model corresponding to the given row of the chained model. If sorting is disabled, the returned row is
     * the same as the parameter row.
     * 
     * @param chainedModelRow
     *            A row of the chained model.
     * @return the row of this model corresponding to the given row of the chained model.
     */
    public int getRow(int chainedModelRow) {
        if (chainedModelRow >= modelIndexes.size()) throw new ArrayIndexOutOfBoundsException("Sorted row index to high: "
                + chainedModelRow + " >= " + modelIndexes.size());
        return viewIndexes[chainedModelRow];
    }


    // ---------------- TableModelListener implementation ------------------------------------------------

    /**
     * This fine grain notification tells listeners the exact range of cells, rows, or columns that changed. This model will
     * translate the event and forward it to any listener.
     * 
     * @param e
     *            the table model event from the chained model.
     */
    public void tableChanged(TableModelEvent e) {
        if (isSorting()) {
            // Must interpret the event to know how it affects the sortered model
            switch (e.getType()) {
                case TableModelEvent.DELETE:
                    tableRowsDeleted(e);
                    break;
                case TableModelEvent.INSERT:
                    tableRowsInserted(e);
                    break;
                case TableModelEvent.UPDATE:
                    if (e.getFirstRow() == TableModelEvent.HEADER_ROW) tableStructureChanged(e);
                    else tableRowsUpdated(e);
                    break;
            }
        } else {
            if (e.getFirstRow() == TableModelEvent.HEADER_ROW) initComparators();
            // call checkArraySize(...) in case of row count have changed
            sort();
            // Simply forward the event
            TableModelEvent newEvent = new TableModelEvent(this, e.getFirstRow(), e.getLastRow(), e.getColumn(), e.getType());
            fireTableChanged(newEvent);
        }
    }


    /**
     * Notification that rows have been deleted in chained model.
     * 
     * @param e
     *            the table model event from the chained model.
     */
    private void tableRowsDeleted(TableModelEvent e) {
        // check the number of deleted rows
        int deletedRow = e.getFirstRow();
        if (deletedRow == e.getLastRow()) {
            int sortedDeletedRow = getRow(deletedRow);
            // We have to resort anyway because row count has changed
            sort();
            // If there is only one row deleted we can fine a single delete event
            fireTableRowsDeleted(sortedDeletedRow, sortedDeletedRow);
        } else {
            // here, we cannot assume that the deleted range remains consecutive.
            sort();
            fireTableDataChanged();
        }
    }


    /**
     * Notification that rows have been inserted in chained model.
     * 
     * @param e
     *            the table model event from the chained model.
     */
    private void tableRowsInserted(TableModelEvent e) {
        // We have to resort anyway because row count has changed
        sort();
        // check the number of inserted rows
        int addedRow = e.getFirstRow();
        if (addedRow == e.getLastRow()) {
            // Only one added row -> fire a single insert event
            int sortedAddedRow = getRow(addedRow);
            fireTableRowsInserted(sortedAddedRow, sortedAddedRow);
        } else {
            // here, we cannot assume that the inserted range remains consecutive.
            fireTableDataChanged();
        }
    }


    /**
     * Notification that chained model structure has changed.
     * 
     * @param e
     *            the table model event from the chained model.
     */
    private void tableStructureChanged(TableModelEvent e) {
        // reset any existing Comparator as the structure has changed
        initComparators();
        sort();
        fireTableStructureChanged();
    }


    /**
     * Notification that rows have been updated in chained model.
     * 
     * @param e
     *            the table model event from the chained model.
     */
    private void tableRowsUpdated(TableModelEvent e) {
        int chainedRow = e.getFirstRow();
        if (chainedRow == e.getLastRow()) {
            // only one row changed, most likely it won't affect sort
            int initialPlace = getRow(chainedRow);
            sort();
            int newPlace = getRow(chainedRow);
            if (initialPlace > newPlace) {
                // swap
                int temp = initialPlace;
                initialPlace = newPlace;
                newPlace = temp;
            }
            fireTableRowsUpdated(initialPlace, newPlace);
        } else {
            sort();
            fireTableDataChanged();
        }
    }


    // ///////////// TableModel implementation /////////////////////////////////////////

    /**
     * Returns the number of rows in the model. A <code>JTable</code> uses this method to determine how many rows it should display.
     * This method should be quick, as it is called frequently during rendering.
     * 
     * @return the number of rows in the model
     * @see #getColumnCount
     */
    public int getRowCount() {
        return chainedModel.getRowCount();
    }


    /**
     * Returns the number of columns in the model. A <code>JTable</code> uses this method to determine how many columns it should
     * create and display by default.
     * 
     * @return the number of columns in the model
     * @see #getRowCount
     */
    public int getColumnCount() {
        return chainedModel.getColumnCount();
    }


    /**
     * Returns the name of the column at <code>columnIndex</code>. This is used to initialize the table's column header name. Note:
     * this name does not need to be unique; two columns in a table can have the same name.
     * 
     * @param columnIndex
     *            the index of the column
     * @return the name of the column
     */
    @Override
    public String getColumnName(int columnIndex) {
        return chainedModel.getColumnName(columnIndex);
    }


    /**
     * Returns the most specific superclass for all the cell values in the column. This is used by the <code>JTable</code> to set up
     * a default renderer and editor for the column.
     * 
     * @param columnIndex
     *            the index of the column
     * @return the common ancestor class of the object values in the model.
     */
    @Override
    public Class<?> getColumnClass(int columnIndex) {
        return chainedModel.getColumnClass(columnIndex);
    }


    /**
     * Returns true if the cell at <code>rowIndex</code> and <code>columnIndex</code> is editable. Otherwise,
     * <code>setValueAt</code> on the cell will not change the value of that cell.
     * 
     * @param rowIndex
     *            the row whose value to be queried
     * @param columnIndex
     *            the column whose value to be queried
     * @return true if the cell is editable
     * @see #setValueAt
     */
    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return chainedModel.isCellEditable(getChainedModelRow(rowIndex), columnIndex);
    }


    /**
     * Returns the value for the cell at <code>columnIndex</code> and <code>rowIndex</code>.
     * 
     * @param rowIndex
     *            the row whose value is to be queried
     * @param columnIndex
     *            the column whose value is to be queried
     * @return the value Object at the specified cell
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        return chainedModel.getValueAt(getChainedModelRow(rowIndex), columnIndex);
    }


    /**
     * Sets the value in the cell at <code>columnIndex</code> and <code>rowIndex</code> to <code>aValue</code>.
     * 
     * @param aValue
     *            the new value
     * @param rowIndex
     *            the row whose value is to be changed
     * @param columnIndex
     *            the column whose value is to be changed
     * @see #getValueAt
     * @see #isCellEditable
     */
    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        chainedModel.setValueAt(aValue, getChainedModelRow(rowIndex), columnIndex);
    }

    // ----------------- Default comparator classes ---------------------------------------------------

    /**
     * This comparator will handle Strings (and objects with no comparator found using their <code>toString()</code> method). The
     * comparison itself is done using the default <code>Collator</code> instance. This comparator ensure that the strings passed to
     * the <code>Collator</code> are not null.
     * 
     * @author Olivier Berlanger
     */
    static class StringComparator implements Comparator<Object> {

        /** The default collator (using default locale) for string comparison */
        private Collator collat = Collator.getInstance();


        /**
         * Compare two objects as String. This comparator also accept null values.
         * 
         * @param obj1
         *            The first Comparable.
         * @param obj2
         *            The second Comparable (should be accepted by the first Comparable as parameter of the
         *            <code>compareTo(Object}</code> method. So typically it must be of the same class as obj1.
         * @return -1, 0 or 1
         */
        public int compare(Object obj1, Object obj2) {
            if (obj1 == obj2) return 0;
            if (obj1 == null) return -1;
            if (obj2 == null) return 1;
            return collat.compare(obj1.toString(), obj2.toString());
        }

    }

    /**
     * This comparator will handle almost all the java basic types (Integer, Date, Double, ...) because they all implements the
     * <code>java.lang.Comparable</code> interface.
     * 
     * @author Olivier Berlanger
     */
    @SuppressWarnings("unchecked")
    static class ComparableComparator implements Comparator<Object> {

        /**
         * Compare two Comparable objects. This comparator also accept null values.
         * 
         * @param obj1
         *            The first Comparable
         * @param obj2
         *            The second Comparable (should be accepted by the first Comparable as parameter of the
         *            <code>compareTo(Object}</code> method. So typically it must be of the same class as obj1.
         * @return -1, 0 or 1
         */
        public int compare(Object obj1, Object obj2) {
            if (obj1 == obj2) return 0;
            if (obj1 == null) return -1;
            if (obj2 == null) return 1;
            if (obj1 instanceof Comparable) return ((Comparable<Object>) obj1).compareTo(obj2);
            return obj1.toString().compareTo(obj2.toString());
        }

    }

    /**
     * This comparator will handle Boolean objects. (Boolean does not implement the <code>java.lang.Comparable</code> interface).
     * 
     * @author Olivier Berlanger
     */
    static class BooleanComparator implements Comparator<Boolean> {

        /**
         * Compare two Boolean objects. This comparator also accept null values.
         * 
         * @param obj1
         *            The first Boolean
         * @param obj2
         *            The second Boolean
         * @return -1, 0 or 1
         */
        public int compare(Boolean obj1, Boolean obj2) {
            if (obj1 == obj2) return 0;
            if (obj1 == null) return -1;
            if (obj2 == null) return 1;
            if (obj1.booleanValue()) return (obj2.booleanValue()) ? 0 : -1;
            return (obj2.booleanValue()) ? 1 : 0;
        }

    }

    /**
     * Comparator used to perform the actual table sorting. It combines all the comparators activated in this sortModel to have a
     * global sort order.
     * 
     * @author Olivier Berlanger
     */
    class RowComparator implements Comparator<Integer> {

        /**
         * Compare two Integer objects: those Integers represent index of rows of the chained model. So sorting those integers will
         * sort rows of chained model. To compare two rows, we look at the sorted column list and applies all the corresponding
         * comparator until we find some difference.
         * 
         * @param obj1
         *            Integer representing the first row to compare
         * @param obj2
         *            Integer representing the second row to compare
         * @return -1, 0 or 1
         */
        @SuppressWarnings("unchecked")
        public int compare(Integer obj1, Integer obj2) {
            int index1 = obj1.intValue();
            int index2 = obj2.intValue();
            int nbrIndexes = getNbrSortColumn();
            for (int i = 0; i < nbrIndexes; i++) {
                int column = getSortColumnAt(i);
                Object val1 = getChainedModel().getValueAt(index1, column);
                Object val2 = getChainedModel().getValueAt(index2, column);
                int cmp = ((Comparator<Object>) getColumnSorters()[column]).compare(val1, val2);
                if (cmp != 0) return getSortDirectionAt(i) * cmp;
            }
            return 0;
        }

    }

}
